/**@file
 *
 * This package contains the set of functions provided for backward-compatibility
 *
 * @author Piotr Golonka (EN/ICE-SCD)
 * @date   September 2011
 */
global string _fwConfigurationDB_fileVersion_fwConfigurationDB_Deprecataed_ctl="3.5.0";


/** Gets the list of recipes available in database
 *
 * @param hierarchyType     dummy parameter now
 * @param recipeTags       the recipe names will be returned in this variable
 * @param exceptionInfo     standard exception handling variable
 *
 * @ingroup RecipeFunctions
 * @deprecated use @ref fwConfigurationDB_getRecipesInDB or @ref fwConfigurationDB_findRecipesInDB

 * @see @ref qstart_recipe_apply section of the Quick Start
 */
void fwConfigurationDB_GetRecipesInDB(string hierarchyType, dyn_string &recipeTags, dyn_string &exceptionInfo)
{
    fwConfigurationDB_findRecipesInDB (recipeTags, exceptionInfo,"*",hierarchyType);
}

/** Gets the list of recipes in database, for specified node
 *
 * @param nodeName          the name of the node (or device, or datapoint) for which recipes are queried
 * @param recipeNames       the recipe names will be returned in this variable
 * @param exceptionInfo     standard exception handling variable
 *
 * @ingroup RecipeFunctions
 * @see @ref qstart_recipe_apply section of the Quick Start
 * @deprecated use @ref fwConfigurationDB_getRecipesInDB or @ref fwConfigurationDB_findRecipesInDB
 */
void fwConfigurationDB_GetDBRecipesForNode(string nodeName, dyn_string &recipeNames, dyn_string &exceptionInfo)
{
    fwConfigurationDB_findRecipesInDB (recipeNames, exceptionInfo,"*","","*","*",nodeName);
}


/** Checks if the node called "/" for logical hierarchy is already present...
*/
void _fwConfigurationDB_checkCreateCommonRootNodeForLogicalHierarchy(dyn_string &exceptionInfo)
{
    string queryRootNodeSql="SELECT ID FROM ITEMS WHERE NAME=\'/\'"+
        " AND HVER="+g_fwConfigurationDB_DBHierarchyIDs[fwDevice_LOGICAL];
    dyn_dyn_mixed aRootNode;
    _fwConfigurationDB_executeDBQuery(queryRootNodeSql,g_fwConfigurationDB_DBConnection, aRootNode, exceptionInfo);
    if (dynlen(exceptionInfo)) return;
    if (dynlen(aRootNode)>0) return; // OK. it exists...

    dyn_int diSeq;
    _fwConfigurationDB_getBatchOfSequenceNumbers(g_fwConfigurationDB_DBConnection, "ITEMS_ID_SEQ", 1, diSeq, exceptionInfo);
    if (dynlen(exceptionInfo)) return;
    string nextId=diSeq[1];
    string insertRootNodeSql="INSERT INTO ITEMS (HVER,ID,PARENT,NAME,TYPE,DETAIL,DESCRIPTION,DPID,DPNAME,DPT_ID) "+
        "VALUES ("+g_fwConfigurationDB_DBHierarchyIDs[fwDevice_LOGICAL]+","+nextId+",NULL,"+
        "\'/\',\'SYSTEM\',\'SYSTEM\','Top of unified LOGICAL hierarchy',NULL,NULL,NULL)";
    fwConfigurationDB_executeSqlSimple(insertRootNodeSql, g_fwConfigurationDB_DBConnection,exceptionInfo);
    if (dynlen(exceptionInfo)) return;

//    _fwConfigurationDB_updateHierarchyView(g_fwConfigurationDB_DBConnection, exceptionInfo);
//    if (dynlen(exceptionInfo)) return;

DebugN("created root node in DB for unified LOGICAL hierarchy");

}

void _fwConfigurationDB_updateHierarchyView(dbConnection dbConn, dyn_string &exceptionInfo)
{
    DebugN("WARNING! updateHierarchyView is deprecated!");
}

/** Gets the database errors from PVSS

    WARNING! an "invalid connection" is not signalled as error,
        and it is not put in the exceptionInfo;
        It is trated separately -> indicated in the invalidConnection parameter.

*/
void _fwConfigurationDB_catchDbError(dbConnection &dbCon, bool &invalidConnection, dyn_string &exceptionInfo)
{
    DebugN("WARNING: _fwConfigurationDB_catchDbError should not be used! IT IS DEPRECATED. Please report to the developer");
    fwException_raise(exceptionInfo,"WARNING",
                      "_fwConfigurationDB_catchDbError should not be used! IT IS DEPRECATED. Please report to the developer","");
    return;
    /*
    int errorCount, errorNumber, errorNative;
    string errorDescription, SQLState;
    int rc=rdbGetError (dbCon, errorCount, errorNumber, errorNative, errorDescription, SQLState);
DebugN("RDBGetError:",errorCount, errorNumber, errorNative, errorDescription, SQLState);
    if (errorCount) {
        fwException_raise(exceptionInfo,"DB ERROR",errorDescription+"\n"+SQLState,"");
        DebugN("Database error",errorDescription,SQLState);
        return;
    }
*/
}

void fwConfigurationDB_GetDBRecipeMetaInfo(string tag,
                                           string &recipeComment,
                                           int &numDevices,
                                           int &numValues,
                                           int &numAlerts,
                                           dyn_string &exceptionInfo)
{
DebugN("WARNING: fwConfigurationDB_GetDBRecipeMetaInfo is deprecated");
}


void fwConfigurationDB_GetRecipeVersionsInDB(dyn_int &versions, dyn_string &descriptions,
                                             dyn_string &userCreated, dyn_string &dateCreated,
					     dyn_int &nDevices,
					     dyn_string &exceptionInfo)
{
DebugN("WARNING: fwConfigurationDB_GetRecipeVersionsInDB is deprecated");
}

/** Gets the list of recipes available in recipe caches
 *
 * @param hierarchyType     the type of Framework hierarchy, dummy now
 * @param recipeCacheNames  the recipe names will be returned in this variable
 * @param recipeCacheDPs    the list of datapoints corresponding to the recipe names will be stored in
 *                          returned in this variable
 * @param exceptionInfo     standard exception handling variable
 * @param system	    allows to look for recipe caches on remote systems
 *
 * @ingroup RecipeFunctions
 * @see @ref qstart_recipe_apply section of the Quick Start
 * @deprecated use @ref fwConfigurationDB_getRecipesInCache instead
 */
void fwConfigurationDB_GetRecipeCaches(string hierarchyType, dyn_string &recipeCacheNames,
 dyn_string &recipeCacheDPs, dyn_string &exceptionInfo, string system="")
{
    DebugN("WARNING: fwConfigurationDB_GetRecipeCaches is deprecated");

    dynClear(recipeCacheNames);
    dynClear(recipeCacheDPs);
    fwConfigurationDB_getRecipesInCache (recipeCacheNames, exceptionInfo, "*", system);

    for (int i=1;i<=dynlen(recipeCacheNames);i++) {
	recipeCacheDPs[i]=_fwConfigurationDB_getRecipeCacheDP(recipeCacheNames[i], exceptionInfo, FALSE);
	if (dynlen(exceptionInfo)) return;
    }

/*        

    dynClear(recipeCacheNames);
    dynClear(recipeCacheDPs);

    dyn_string tmpRecipeCacheDPs=dpNames("*","_FwRecipeCache");

    for (int i=1;i<=dynlen(tmpRecipeCacheDPs);i++) {
        string recipeName,hierarchy;
        dpGet(tmpRecipeCacheDPs[i]+".RecipeName",recipeName);
        // if the name is not defined, correct the situation immediately
        if (recipeName==""){
            recipeName=recipeCacheDPs[i];
            dpSetWait(tmpRecipeCacheDPs[i]+".RecipeName",recipeName);
        }
        dynAppend(recipeCacheDPs,tmpRecipeCacheDPs[i]);
        dynAppend(recipeCacheNames,recipeName);

    }
*/
}

/** Gets the list of cached recipes for specified node
 *
 * @param nodeName          the name of the node (or device, or datapoint) for which recipes are queried
 * @param recipeNames       the recipe names will be returned in this variable
 * @param exceptionInfo     standard exception handling variable
 * @param system	    allows to look for recipe caches on remote systems
 *
 * @ingroup RecipeFunctions
 * @see @ref qstart_recipe_apply section of the Quick Start
 * @deprecated use @ref fwConfigurationDB_getRecipesInCache instead
 */
void fwConfigurationDB_GetCacheRecipesForNode(string nodeName, dyn_string &recipeNames, dyn_string &exceptionInfo, 
    string system="")
{
    DebugN("WARNING: fwConfigurationDB_GetCacheRecipesForNode is deprecated");


    fwConfigurationDB_getRecipesInCache (recipeNames, exceptionInfo, nodeName, system);
    if (dynlen(exceptionInfo)) return;
/*

    string sysName=getSystemName();
    dynClear(recipeNames);
    dyn_string tmpRecipeCacheDPs=dpNames("*","_FwRecipeCache");

    for (int i=1;i<=dynlen(tmpRecipeCacheDPs);i++) {
        string recipeName;
        dyn_string devList;
        dpGet(tmpRecipeCacheDPs[i]+".RecipeName",recipeName,
              tmpRecipeCacheDPs[i]+".DataPoints.DPNames",devList);
        // if the name is not defined, correct the situation immediately
        if (recipeName==""){
            recipeName=tmpRecipeCacheDPs[i];
            dpSetWait(tmpRecipeCacheDPs[i]+".RecipeName",recipeName);
        }
        if ( (dynContains(devList,nodeName))||(nodeName=="")||(nodeName==sysName)) dynAppend(recipeNames,recipeName);

    }
*/
}


/**
 * @deprecated
  */
void fwConfigurationDB_storeDevicesInDB(string topDevice, string hierarchyType, dyn_string deviceDpNames,
                                           dyn_string &exceptionInfo, string systemName="")
{
    fwException_raise(exceptionInfo,"ERROR", "fwConfigurationDB_storeDevicesInDB() is deprecated","");
    return;
}

////////////////////////////////////////////////////////////////////////////////////
/** Saves the device configuration to the database

@param	saveConfigs	may either be of bool, or of int type; in the former case, passing
    FALSE means that only the minimal (hierarchy) information is stored; passing TRUE means
    all configs of the devices are stored.@br
    If the int-typed parameter is used it should contain the sum of options refering to the
    configs that should be stored: the @ref deviceConfigsIndices constants should be used.
    To store no configs (i.e. only the minimal hierarchy information), one should pass 0;
    to store all properties one should pass the
    @c fwConfigurationDB_deviceConfig_ALLDEVPROPS constant.
*/
void fwConfigurationDB_saveDeviceConfigurationInDB_OLD(dyn_string deviceList, string hierarchyType,
    string configurationName, anytype saveConfigs, dyn_string &exceptionInfo, string systemName="", string confDescription="")
{

    int configsToSave=0;
    if (getType(saveConfigs)==BOOL_VAR) {
	if (saveConfigs==TRUE) configsToSave=fwConfigurationDB_deviceConfig_ALLDEVPROPS;
    } else if (getType(saveConfigs)==INT_VAR) {
	configsToSave=saveConfigs;
    } else {
	fwException_raise(exceptionInfo,"ERROR in fwConfigurationDB_saveDeviceConfigurationInDB",
	    "The saveConfigs parameter is neither BOOL not INT","");
	return;
    }


    string topDevice="";

    time t0=getCurrentTime();
    string date=t0;

    if (configurationName=="") {
        fwException_raise(exceptionInfo,"ERROR","Configuration name may not be empty","");
        return;
    }

    _fwConfigurationDB_completeDevicesInHierarchy(deviceList, hierarchyType, systemName, exceptionInfo);
    if (dynlen(exceptionInfo)) return;

    // commit previous transaction, if exists...
    rdbCommitTransaction(g_fwConfigurationDB_DBConnection);
    int rc=rdbBeginTransaction(g_fwConfigurationDB_DBConnection);
    if (rc) {
	//bool invalidConnection;
        //_fwConfigurationDB_catchDbError(g_fwConfigurationDB_DBConnection, invalidConnection, exceptionInfo);
        fwException_raise(exceptionInfo,"ERROR","Cannot start new transaction","");
        if (dynlen(exceptionInfo)) return;
    };


    dyn_dyn_mixed deviceListObject;
    _fwConfigurationDB_ensureDeviceListObjectValid(deviceListObject);

   if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceMetaData,"Storing type information",  10.0, exceptionInfo)) return;




    // expand the types - we should do it in more optimized way...
    dyn_dyn_mixed devListObj;
    fwConfigurationDB_expandToDeviceListObject(hierarchyType, deviceList, devListObj, exceptionInfo);

    dyn_string dpTypes=devListObj[fwConfigurationDB_DLO_DPTYPE];
    dynUnique(dpTypes);
    dyn_int dpTypeIds;
    dyn_mixed dpTypeElements,dpTypeElementIds;
    DebugTN("saving types");
    _fwConfigurationDB_checkSaveDpTypes(dpTypes,dpTypeIds,dpTypeElements,dpTypeElementIds,t0, exceptionInfo);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

   if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceMetaData,"Storing version information",  70.0, exceptionInfo)) return;



    DebugTN("saving hierarchy");
    _fwConfigurationDB_saveDeviceHierarchyToDB(topDevice,  hierarchyType, deviceList, exceptionInfo, systemName, deviceListObject);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}




    //DebugN("Types OK",dpTypes,dpTypeIds);
    DebugTN("saving configuration version info");
    int conf_id=_fwConfigurationDB_checkCreateDeviceConfiguration(hierarchyType,configurationName, confDescription, exceptionInfo);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

   if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceMetaData,"Done",  100.0, exceptionInfo)) return;

    DebugN("OK. ConfID",conf_id);






    // resolve device types to device ids, resolve device element ids and names as well
    dyn_int deviceTypeIds; // parallel to deviceList;
    dyn_mixed deviceElementIds;

    for (int i=1;i<=dynlen(deviceListObject[fwConfigurationDB_DLO_DPNAME]);i++) {
        string dptype=deviceListObject[fwConfigurationDB_DLO_DPTYPE][i];
        int idx=dynContains(dpTypes,dptype);
        deviceTypeIds[i]=dpTypeIds[idx];
        if (dptype=="FwNode" && hierarchyType==fwDevice_HARDWARE) {
            deviceElementIds[i]=makeDynInt();
            deviceListObject[fwConfigurationDB_DLO_PROPNAMES][i]=makeDynString();
        } else {
            deviceElementIds[i]=dpTypeElementIds[idx];
            deviceListObject[fwConfigurationDB_DLO_PROPNAMES][i]=dpTypeElements[idx];
        }
    }

     if (hierarchyType==fwDevice_LOGICAL) {
        _fwConfigurationDB_resolveLogicalIds(deviceListObject, exceptionInfo,systemName);
    }


    DebugTN("Saving devices...");
    _fwConfigurationDB_saveDevicesInConfiguration(deviceListObject,hierarchyType, conf_id,
        deviceTypeIds, exceptionInfo, date);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    //DebugN("citemids:",deviceListObject[fwConfigurationDB_DLO_CITEM_ID]);



//////////////////

    if (hierarchyType==fwDevice_LOGICAL) {

	DebugN("saving logical mapping...");
	//dyn_dyn_int changedLogicalDeviceIds;
	//_fwConfigurationDB_checkLogicalChanged(deviceListObject, configurationName, changedLogicalDeviceIds, exceptionInfo);
	//if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
	fwConfigurationDB_saveReferences(configurationName, deviceListObject,exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

	// HERE WE SHOULD PROBABLY STORE THE ELEMENTS FOR ALL FwNode's: i.e. the panels, etc...
	// right now this is inhibited by configsToSave=0 :-(
	// most likely, we would only like to store these, if they differ, or something...
	// or maybe we could move this into _saveReferences part?
	configsToSave=0;

    }


    if (configsToSave!=0) {

    DebugN("storing props now...");



    bool storeValues=configsToSave & fwConfigurationDB_deviceConfig_VALUE;

    _fwConfigurationDB_saveDevicePropertiesToDB(deviceListObject, hierarchyType, deviceElementIds,
                                                date, exceptionInfo, storeValues);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDevicesAndElements,"DONE",  100.0, exceptionInfo)) return;



    DebugN("properties saved OK");

    // get the list of all properties (needed for summary alerts)
    dyn_string allDpesInConfiguration;
    dyn_int allIpropIdsInConfiguration;

    _fwConfigurationDB_getIpropIdsInConfiguration(configurationName,
        allDpesInConfiguration, allIpropIdsInConfiguration,exceptionInfo);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

    // form the lists of dpes and corresponding ipropIds...
    dyn_string dpes;
    dyn_int ipropIds;
/*
    for (int i=1;i<=dynlen(deviceListObject[fwConfigurationDB_DLO_DPNAME]);i++){

        dyn_int devIpropIds=deviceListObject[fwConfigurationDB_DLO_PROPIDS][i];
        dyn_string elementNames=deviceListObject[fwConfigurationDB_DLO_PROPNAMES][i];
        string devName=deviceListObject[fwConfigurationDB_DLO_DPNAME][i];
        for (int j=1;j<=dynlen(devIpropIds);j++) {
            dynAppend(dpes,devName+elementNames[j]);;
            dynAppend(ipropIds,devIpropIds[j]);
        }
    }
*/
   if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Preparing...",  10.0, exceptionInfo)) return;

    dyn_dyn_string dpeSupportedConfigs;

    // select only the ones that reflect the devices we save...
    for (int i=1;i<=dynlen(allDpesInConfiguration);i++){
	dyn_string spl=strsplit(allDpesInConfiguration[i],".");
	string devName=spl[1];
	if (dynContains(deviceList,devName)) {
	    string _dpe=allDpesInConfiguration[i];

	    // exclude dpes of type DPEL_DPID: fix #17818
	    if (dpElementType(_dpe)==DPEL_DPID) continue;
            dyn_string supportedConfigs=dpGetAllConfigs(_dpe);

            dynAppend(dpes,_dpe);
            dynAppend(ipropIds, allIpropIdsInConfiguration[i]);
            dpeSupportedConfigs[dynlen(dpeSupportedConfigs)+1]=supportedConfigs;
	}
    }

    dyn_dyn_mixed configurationData;

    if (configsToSave & fwConfigurationDB_deviceConfig_ADDRESS) {
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Extracting addressing data",  20.0, exceptionInfo)) return;

	DebugN("storing addressing data");
	dynClear(configurationData);
	
	// form a list of supported dpes...
	dyn_string supportedDpes;
	dyn_int    supportedIpropIds;
	for (int i=1;i<=dynlen(dpes);i++) {
	    if (dynContains(dpeSupportedConfigs[i],"_address")) {
		dynAppend(supportedDpes,dpes[i]);
		dynAppend(supportedIpropIds,ipropIds[i]);
	    }
	}	
	
//	_fwConfigurationDB_extractAddressingConfiguration(dpes,ipropIds, configurationData, exceptionInfo);
	_fwConfigurationDB_extractAddressingConfiguration(supportedDpes,supportedIpropIds, configurationData, exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Storing addressing data",  25.0, exceptionInfo)) return;

	_fwConfigurationDB_saveAddressingConfiguration(configurationData, exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    }

    if (configsToSave & fwConfigurationDB_deviceConfig_ALERT) {
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Extracting alerts data",  30.0, exceptionInfo)) return;

	DebugN("storing alerts data");
	dynClear(configurationData);
	// form a list of supported dpes...
	dyn_string supportedDpes;
	dyn_int    supportedIpropIds;
	for (int i=1;i<=dynlen(dpes);i++) {
	    if (dynContains(dpeSupportedConfigs[i],"_alert_hdl")) {
		dynAppend(supportedDpes,dpes[i]);
		dynAppend(supportedIpropIds,ipropIds[i]);
	    }
	}	

	_fwConfigurationDB_extractAlertConfiguration(supportedDpes,supportedIpropIds,
                             allDpesInConfiguration, allIpropIdsInConfiguration,
                             configurationData,exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Storing alerts data",  35.0, exceptionInfo)) return;

	_fwConfigurationDB_saveAlertConfiguration(configurationData, exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    }

    if (configsToSave & fwConfigurationDB_deviceConfig_ARCHIVING) {
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Extracting archiving data",  40.0, exceptionInfo)) return;

	DebugN("storing archiving data");
	dynClear(configurationData);
	// form a list of supported dpes...
	dyn_string supportedDpes;
	dyn_int    supportedIpropIds;
	for (int i=1;i<=dynlen(dpes);i++) {
	    if (dynContains(dpeSupportedConfigs[i],"_archive")) {
		dynAppend(supportedDpes,dpes[i]);
		dynAppend(supportedIpropIds,ipropIds[i]);
	    }
	}	
	_fwConfigurationDB_extractArchivingConfiguration(supportedDpes,supportedIpropIds, configurationData, exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Storing archiving data",  45.0, exceptionInfo)) return;
	_fwConfigurationDB_saveArchivingConfiguration(configurationData, exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    }

    if (configsToSave & fwConfigurationDB_deviceConfig_DPFUNCTION) {
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Extracting dpfunction data",  50.0, exceptionInfo)) return;

	DebugN("storing dp-function data");
	dynClear(configurationData);
	// form a list of supported dpes...
	dyn_string supportedDpes;
	dyn_int    supportedIpropIds;
	for (int i=1;i<=dynlen(dpes);i++) {
	    if (dynContains(dpeSupportedConfigs[i],"_dp_fct")) {
		dynAppend(supportedDpes,dpes[i]);
		dynAppend(supportedIpropIds,ipropIds[i]);
	    }
	}	
	_fwConfigurationDB_extractDpFunctionConfiguration(supportedDpes,supportedIpropIds, configurationData, exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Storing dpfunction data",  55.0, exceptionInfo)) return;

	_fwConfigurationDB_saveDpFunctionConfiguration(configurationData, exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    }

    if (configsToSave & fwConfigurationDB_deviceConfig_CONVERSION) {
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Extracting conversion data",  60.0, exceptionInfo)) return;

	DebugN("storing conversion data");
	dynClear(configurationData);
	// form a list of supported dpes...
	dyn_string supportedDpes;
	dyn_int    supportedIpropIds;
	for (int i=1;i<=dynlen(dpes);i++) {
	    if (dynContains(dpeSupportedConfigs[i],"_msg_conv")||dynContains(dpeSupportedConfigs[i],"_cmd_conv")) {
		dynAppend(supportedDpes,dpes[i]);
		dynAppend(supportedIpropIds,ipropIds[i]);
	    }
	}	
	_fwConfigurationDB_extractConversionConfiguration(supportedDpes,supportedIpropIds, configurationData, exceptionInfo);
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Storing conversion data",  65.0, exceptionInfo)) return;
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
	_fwConfigurationDB_saveConversionConfiguration(configurationData, exceptionInfo);
    }

    if (configsToSave & fwConfigurationDB_deviceConfig_PVRANGE) {
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Extracting pvrange data",  70.0, exceptionInfo)) return;

	DebugN("storing pvrange data");
	dynClear(configurationData);
	// form a list of supported dpes...
	dyn_string supportedDpes;
	dyn_int    supportedIpropIds;
	for (int i=1;i<=dynlen(dpes);i++) {
	    if (dynContains(dpeSupportedConfigs[i],"_pv_range")) {
		dynAppend(supportedDpes,dpes[i]);
		dynAppend(supportedIpropIds,ipropIds[i]);
	    }
	}	
	_fwConfigurationDB_extractPvRangeConfiguration(supportedDpes,supportedIpropIds, configurationData, exceptionInfo);
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Storing pvrange data",  75.0, exceptionInfo)) return;
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
	_fwConfigurationDB_savePvRangeConfiguration(configurationData, exceptionInfo);
    }

    if (configsToSave & fwConfigurationDB_deviceConfig_SMOOTHING) {
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Extracting smoothing data",  80.0, exceptionInfo)) return;

	DebugN("storing smoothing data");
	dynClear(configurationData);
	// form a list of supported dpes...
	dyn_string supportedDpes;
	dyn_int    supportedIpropIds;
	for (int i=1;i<=dynlen(dpes);i++) {
	    if (dynContains(dpeSupportedConfigs[i],"_smooth")) {
		dynAppend(supportedDpes,dpes[i]);
		dynAppend(supportedIpropIds,ipropIds[i]);
	    }
	}	
	_fwConfigurationDB_extractSmoothingConfiguration(supportedDpes,supportedIpropIds, configurationData, exceptionInfo);
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Storing smoothing data",  85.0, exceptionInfo)) return;

	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
	_fwConfigurationDB_saveSmoothingConfiguration(configurationData, exceptionInfo);
    }

    if (configsToSave & fwConfigurationDB_deviceConfig_UNITANDFORMAT) {
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Extracting unit and format data",  90.0, exceptionInfo)) return;

	DebugN("storing unit and format data");
	dynClear(configurationData);
	_fwConfigurationDB_extractUnitAndFormatConfiguration(dpes,ipropIds, configurationData, exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
	if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"Storing unit and format data",  50.0, exceptionInfo)) return;

	_fwConfigurationDB_saveUnitAndFormatConfiguration(configurationData, exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    }

}


    DebugN("Committing db transaction");
    rc=rdbCommitTransaction(g_fwConfigurationDB_DBConnection);
    if (rc) {
           //bool invalidConnection;
           //_fwConfigurationDB_catchDbError(g_fwConfigurationDB_DBConnection, invalidConnection, exceptionInfo);
           fwException_raise(exceptionInfo,"ERROR","Cannot commit transaction","");
           return;
    };

    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveDeviceProperties,"DONE",  100.0, exceptionInfo)) return;

    DebugN("DONE!");
}




/** Extracts addressing configuration from PVSS
 *
 * @param dpes      list of the data point element names for which configuration is requested
 * @param ipropIds  corresponding list of IPROP_ID identifiers (in the database) of the data-point elements
 * @param addressingData on return contains the configuration data (internal format described below)
 * @param exceptionInfo standard exception handling variable
 *
 * the addressingData variable has the following format:
 * @li addressingData[i] - represent complete address settings for a single dpe:
 * @li addressingData[i][1] is string-typed dpe name (i.e. dp + element name)
 * @li addressingData[i][2] contains associated IPROP_ID (int)
 * @li addressingData[i][3] is addressConfig object (dyn_anytype)
*/
void _fwConfigurationDB_extractAddressingConfiguration(dyn_string dpes, dyn_string ipropIds,
    dyn_dyn_mixed &addressingData, dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("extractAddressingConfiguration", t0);
    dynClear(addressingData);
    for (int i=1;i<=dynlen(dpes);i++) {
        string dpe=dpes[i];
        bool configExists, isActive;
        dyn_anytype addressConfigObject;
        fwPeriphAddress_get(dpe, configExists, addressConfigObject, isActive, exceptionInfo);
        if (dynlen(exceptionInfo)) return;
        if (configExists) {
            int idx=dynlen(addressingData)+1;
            addressingData[idx][1]=dpe;
            addressingData[idx][2]=ipropIds[i];
            addressingData[idx][3]=addressConfigObject;
        }
    }
    _fwConfigurationDB_endFunction("extractAddressingConfiguration", t0);
}



/** Saves addressing configuration to Configuration Database
 *
 * @param addressingData should contain the configuration data to be stored (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_saveAddressingConfiguration(dyn_dyn_mixed addressingData, dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("saveAddressingConfiguration", t0);

    anytype cmd;
    string sql="INSERT INTO CONFIG_ADDRESSES (IPROP_ID, TYPE, DRIVER, DIRECTION, DATATYPE, ACTIVE, OPTIONS ) "+
        " VALUES (:IPROP_ID, :TYPE, :DRIVER, :DIRECTION, :DATATYPE, :ACTIVE, :OPTIONS)";

    _fwConfigurationDB_startCommand(sql, g_fwConfigurationDB_DBConnection,
                                    cmd, exceptionInfo);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

    for (int i=1;i<=dynlen(addressingData);i++) {
        mapping params;
        dyn_anytype addressObject=addressingData[i][3];
        params[":IPROP_ID"]=addressingData[i][2];
        params[":TYPE"]="\'"+addressObject[fwPeriphAddress_TYPE]+"\'";
        params[":DRIVER"]=(int)addressObject[fwPeriphAddress_DRIVER_NUMBER];
        params[":DIRECTION"]=(int)addressObject[fwPeriphAddress_DIRECTION];
        params[":DATATYPE"]=(int)addressObject[fwPeriphAddress_DATATYPE];
        params[":ACTIVE"]=(int)addressObject[fwPeriphAddress_ACTIVE];

        string options=addressObject[fwPeriphAddress_REFERENCE]; // this is stored at the beginning of options!
        for (int j=FW_PARAMETER_FIELD_LOWLEVEL;j<=dynlen(addressObject);j++) {
            options=options+"|"+(string)addressObject[j]; // fix #19075 we need to cast here explicitely to string-otherwise it breaks!
        }
        params[":OPTIONS"]="\'"+options+"\'";
        _fwConfigurationDB_bindExecuteCommand(cmd, params, exceptionInfo);
        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    }

    _fwConfigurationDB_finishCommand(cmd, exceptionInfo);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    _fwConfigurationDB_endFunction("saveAddressingConfiguration", t0);
}


/** Loads addressing configuration from Configuration Database
 *
 * @param dpes      list of the data point element names for which configuration is loaded
 * @param ipropIds  corresponding list of IPROP_ID identifiers (in the database) of the data-point elements
 * @param addressingData on return contains the configuration data loaded from database (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_loadAddressingConfiguration(dyn_dyn_mixed &addressingData, dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("loadAddressingConfiguration", t0);

    dynClear(addressingData);
//    string propidsListAsSql=_fwConfigurationDB_listToSQLString("IPROP_ID", propIds);

    //string sql="SELECT IPROP_ID, TYPE, DRIVER, DIRECTION,DATATYPE, ACTIVE, OPTIONS FROM CONFIG_ADDRESSES "+
    //    " WHERE "+propidsListAsSql;
    string sql="SELECT IPROP_ID, TYPE, DRIVER, DIRECTION,DATATYPE, ACTIVE, OPTIONS FROM CONFIG_ADDRESSES "+
        " where iprop_id in ( select iprop_id from tmp_item_props )";

    string sql = "SELECT TIP.DPENAME, CA.TYPE, CA.DRIVER, CA.DIRECTION,CA.DATATYPE, CA.ACTIVE, CA.OPTIONS "+
                 " FROM CONFIG_ADDRESSES CA, TMP_ITEM_PROPS TIP "+
                 " where CA.iprop_id =TIP.iprop_id";

    dyn_dyn_mixed aRecords;

    _fwConfigurationDB_executeDBQuery(sql,g_fwConfigurationDB_DBConnection, aRecords, exceptionInfo,7,TRUE);
    if (dynlen(exceptionInfo)) return;
    int max=0;
    if (dynlen(aRecords)) max=dynlen(aRecords[1]);

    for (int i=1;i<=max;i++) {
        dyn_anytype addressObject;
        string adrOpt=aRecords[7][i];
        dyn_mixed options=strsplit(adrOpt,"|");
        // fix #58663: cannot restore S7 addresses with empty poll group
        // check if we have "|" as the last character, and if so, fix what strsplit is not doing for us...
        if (adrOpt!="" && substr(adrOpt,strlen(adrOpt)-1)=="|") dynAppend(options,"");
        
        string reference=options[1];
        dynRemove(options,1);
        addressObject[fwPeriphAddress_TYPE]=aRecords[2][i];
        addressObject[fwPeriphAddress_DRIVER_NUMBER]=aRecords[3][i];
        addressObject[fwPeriphAddress_DIRECTION]=aRecords[4][i];
        addressObject[fwPeriphAddress_DATATYPE]=aRecords[5][i];
        addressObject[fwPeriphAddress_ACTIVE]=aRecords[6][i];
        addressObject[fwPeriphAddress_REFERENCE]=reference;
        
        // temporary fix for #49981: poll groups for the S7 driver need to have a leading underscore,
        // and they are extracted (and saved to DB) without one...
        if (addressObject[fwPeriphAddress_TYPE]=="S7") {
    	    string pollGroup=options[dynlen(options)];
    	    if (pollGroup!="" && substr(pollGroup,0,1)!="_") {
    		pollGroup="_"+pollGroup;
    		options[dynlen(options)]=pollGroup;
    	    }
        }                                                                                    
        
        for (int j=1;j<=dynlen(options);j++)
            addressObject[FW_PARAMETER_FIELD_LOWLEVEL-1+j]=options[j];
            
        addressingData[1][i]=aRecords[1][i];// dpes
        addressingData[2][i]=addressObject;
    }

    _fwConfigurationDB_endFunction("loadAddressingConfiguration", t0);
}


////////////

/** Extracts archiving configuration from PVSS
 *
 * @param dpes      list of the data point element names for which configuration is requested
 * @param ipropIds  corresponding list of IPROP_ID identifiers (in the database) of the data-point elements
 * @param archivingData on return contains the configuration data (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_extractArchivingConfiguration(dyn_string dpes,dyn_int ipropIds,
    dyn_dyn_mixed &archivingData, dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("extractArchivingConfiguration", t0);
    dynClear(archivingData);
    archivingData[9][1]="";// make the object valid: (number of columns)

    dyn_bool configExists,isActive;
    dyn_string archiveClass;
    dyn_int archiveType, smoothProcedure;
    dyn_float deadband,timeInterval;


    fwArchive_getMany(dpes, configExists, archiveClass, archiveType, smoothProcedure,
                      deadband, timeInterval,isActive,exceptionInfo);
    if (dynlen(exceptionInfo)) return;

    for (int i=1;i<=dynlen(dpes);i++) {
        if (configExists[i]) {
            int idx=dynlen(archivingData[1])+1;
            archivingData[1][idx]=dpes[i];
            archivingData[2][idx]=ipropIds[i];
            archivingData[3][idx]=archiveClass[i];
            archivingData[4][idx]=archiveType[i];
            archivingData[5][idx]=smoothProcedure[i];
            archivingData[6][idx]=deadband[i];
            archivingData[7][idx]=timeInterval[i];
            archivingData[8][idx]=isActive[i];
        }
    }

    _fwConfigurationDB_endFunction("extractArchivingConfiguration", t0);
}

/** Saves archiving configuration to Configuration Database
 *
 * @param archivingData should contain the configuration data to be stored (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_saveArchivingConfiguration(dyn_dyn_mixed &archivingData, dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("saveArchivingConfiguration", t0);

    if (dynlen(archivingData[1])>0) {
        anytype cmd;
        string sql="INSERT INTO CONFIG_ARCHIVING (IPROP_ID, CLASSNAME, TYPE, "+
            " SMOOTHPROC, DEADBAND, TIMEINTVAL, ACTIVE) "+
            " VALUES (:IPROP_ID, :CLASSNAME, :TYPE, :SMOOTHPROC,:DEADBAND,:TIMEINTVAL, :ACTIVE)";

        _fwConfigurationDB_startCommand(sql, g_fwConfigurationDB_DBConnection,
                                        cmd, exceptionInfo);

        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
        for (int i=1;i<=dynlen(archivingData[1]);i++) {
            mapping params;

            // store generic information...
            params[":IPROP_ID"]=archivingData[2][i];
            params[":CLASSNAME"]="\'"+archivingData[3][i]+"\'";
            params[":TYPE"]=archivingData[4][i];
            params[":SMOOTHPROC"]=archivingData[5][i];
            params[":DEADBAND"]=archivingData[6][i];
            params[":TIMEINTVAL"]=archivingData[7][i];
            params[":ACTIVE"]=(int)archivingData[8][i];
            _fwConfigurationDB_bindExecuteCommand(cmd, params, exceptionInfo);
            if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
        }
        _fwConfigurationDB_finishCommand(cmd, exceptionInfo);
        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    }
    _fwConfigurationDB_endFunction("saveArchivingConfiguration", t0);
}



/** Extracts alert configuration from PVSS
 *
 * @param dpes      list of the data point element names for which configuration is requested
 * @param ipropIds  corresponding list of IPROP_ID identifiers (in the database) of the data-point elements
 * @param alertData on return contains the configuration data (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_extractAlertConfiguration(dyn_string dpes,dyn_int ipropIds,
    dyn_string allDpes, dyn_int allIpropIds,
    dyn_dyn_mixed &alertData, dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("extractAlertConfiguration", t0);
    dynClear(alertData);
    alertData[13][1]="";// make the object valid: create at least 11 columns

    // fix #48430
    // Don't try to extract the addresses for DPE's of types that are not allowed
    dyn_string newDpes,newAllDpes;
    dyn_int newAllIpropIds,newIpropIds;
    for (int i=1;i<=dynlen(dpes);i++) {
	int dpeType=dpElementType(dpes[i]);
	if ( (dpeType != DPEL_DPID) &&
	     (dpeType != DPEL_DYN_DPID) &&
	     (dpeType != DPEL_DPID_STRUCT) &&
	     (dpeType != DPEL_DYN_DPID_STRUCT) ) {
	        dynAppend(newDpes,dpes[i]);
        	dynAppend(newIpropIds,ipropIds[i]);
	        dynAppend(newAllDpes,allDpes[i]);
	        dynAppend(newAllIpropIds,allIpropIds[i]);
	}
    }
    // note! we do not return the modified values to the caller!
    // we substitute the variables to simplify code below
    dpes=newDpes;
    allDpes=newAllDpes;
    allIpropIds=newAllIpropIds;
    ipropIds=newIpropIds;
    
    dyn_bool configExists,isActive;
    dyn_int alertConfigType;
    dyn_dyn_string alertTexts,alertClasses,summaryDpeList,alertPanelParameters;
    dyn_dyn_float alertLimits;
    dyn_string alertPanel,alertHelp;
    fwAlertConfig_getMany( dpes,
                           configExists,
                           alertConfigType,
                           alertTexts,
                           alertLimits,
                           alertClasses,
                           summaryDpeList,
                           alertPanel,
                           alertPanelParameters,
                           alertHelp,
                           isActive,
                           exceptionInfo);

    if (dynlen(exceptionInfo)) return;

    for (int i=1;i<=dynlen(dpes);i++) {
        if (configExists[i]) {
            int idx=dynlen(alertData[1])+1;
            alertData[1][idx]=dpes[i];
            alertData[2][idx]=ipropIds[i];
            alertData[3][idx]=alertConfigType[i];
            alertData[4][idx]=alertTexts[i]; // dyn_string!
            alertData[5][idx]=alertLimits[i]; // dyn_float!
            alertData[6][idx]=alertClasses[i]; // dyn_string!


            alertData[8][idx]=alertPanel[i];
            alertData[9][idx]=alertPanelParameters[i];//dyn_string!
            alertData[10][idx]=alertHelp[i];
            alertData[11][idx]=isActive[i];

            // decode ipropIds of the alerts included in summary
            // leave only the ones present in the configuration...
            //
            // note that we should get the list of all DPEs in configuration
            // from DB, rather than from ipropIds, which only
            // contains the elements currently being stored...
            dyn_string sumDpes=summaryDpeList[i];
            if (dynlen(sumDpes)==1) {
        	// this may be the case of dpelist specified by pattern...
        	if (strpos(sumDpes[1],"*")>=0 || strpos(sumDpes[1],"?")>=0) {
        	    alertData[12][idx]=makeDynInt(-1);
        	    alertData[7][idx]=sumDpes[1];
        	    continue;
        	}
            }
            dyn_int    includedSumIds;
            dyn_string includedSumDpes;
            for (int j=1;j<=dynlen(sumDpes);j++) {
                int sumIdx=dynContains(allDpes,sumDpes[j]);
                if (sumIdx>0) {
                    int alarmIpropId=allIpropIds[sumIdx];
                    //DebugN("element of summary alert for "+dpes[i]+" found!",sumDpes[j],alarmIpropId);
                    dynAppend(includedSumIds,alarmIpropId);
                    dynAppend(includedSumDpes,sumDpes[j]);
                } else {
                    //DebugN("element of summary alert for "+dpes[i]+" not present:",sumDpes[j]);
                }
            }
            alertData[7][idx]=includedSumDpes; // dyn_string!
            alertData[12][idx]=includedSumIds; // dyn_int!

        }
    }

    _fwConfigurationDB_endFunction("extractAlertConfiguration", t0);
}

/** Saves alert configuration to Configuration Database
 *
 * @param alarmData should contain the configuration data to be stored (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_saveAlertConfiguration(dyn_dyn_mixed &alertData, dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("saveAlertConfiguration", t0);

    if (dynlen(alertData[1])>0) {
        anytype cmdAlert, cmdAlertDpe, cmdAlertSum, cmdAlertMisc;
        string sqlAlert="INSERT INTO CONFIG_ALERTS (IPROP_ID, TYPE, ACTIVE) "+
            " VALUES (:IPROP_ID, :TYPE, :ACTIVE)";
        string sqlAlertDpe="INSERT INTO CONFIG_DPEALERTS (IPROP_ID, TEXTS, LIMITS, CLASSES) "+
            " VALUES (:IPROP_ID, :TEXTS, :LIMITS, :CLASSES)";
        string sqlAlertSum="INSERT INTO CONFIG_SUMALERTS (IPROP_ID, TEXTS, CLASS, SUMDPELIST) "+
            " VALUES (:IPROP_ID, :TEXTS, :CLASS, :SUMDPELIST)";
        string sqlAlertMisc="INSERT INTO CONFIG_ALERTS_MISC (IPROP_ID,PANEL, PANELPARAMS, HELP ) "+
            " VALUES (:IPROP_ID, :PANEL, :PPARAMS, :HELP)";

        _fwConfigurationDB_startCommand(sqlAlert, g_fwConfigurationDB_DBConnection,
                                    cmdAlert, exceptionInfo);
        _fwConfigurationDB_startCommand(sqlAlertDpe, g_fwConfigurationDB_DBConnection,
                                        cmdAlertDpe, exceptionInfo);
        _fwConfigurationDB_startCommand(sqlAlertSum, g_fwConfigurationDB_DBConnection,
                                        cmdAlertSum, exceptionInfo);
        _fwConfigurationDB_startCommand(sqlAlertMisc, g_fwConfigurationDB_DBConnection,
                                        cmdAlertMisc, exceptionInfo);

        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
        for (int i=1;i<=dynlen(alertData[1]);i++) {



            mapping paramsAlert, paramsAlertDpe,paramsAlertSum, paramsAlertMisc;
            int iprop_id=alertData[2][i];
            int alertType=alertData[3][i];

            // store generic information...
            paramsAlert[":IPROP_ID"]=iprop_id;
            paramsAlert[":TYPE"]=alertType;
            paramsAlert[":ACTIVE"]=(int)alertData[11][i];
            _fwConfigurationDB_bindExecuteCommand(cmdAlert, paramsAlert, exceptionInfo);
            if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}


            if (alertType==DPCONFIG_SUM_ALERT){
                // store alert summary information...
                string sval;
                paramsAlertSum[":IPROP_ID"]=iprop_id;
                dyn_string ds=alertData[4][i];
                _fwConfigurationDB_dataToString(ds , DPEL_DYN_STRING,"|", sval,exceptionInfo);

        	// fix #17872
        	strreplace(sval,"\'","\'\'");
		string escClass=alertData[6][i];
        	strreplace(escClass,"\'","\'\'");


                paramsAlertSum[":TEXTS"]="\'"+sval+"\'";
                paramsAlertSum[":CLASS"]="\'"+escClass+"\'";


                //ds=alertData[7][i];
                //_fwConfigurationDB_dataToString(ds , DPEL_DYN_STRING,"|",sval,exceptionInfo);

                dyn_int di=alertData[12][i];

		// the DPE list may be specified using DP PATTERN
		if (dynlen(di)==1 && di[1]==-1) {
		    // then we will take it as it is, prepending with keyword "PATTERN:"
		    sval="PATTERN:"+alertData[7][i];
		} else {
		    // otherwise we use the list of dp-alert IDs,
            	    _fwConfigurationDB_dataToString(di , DPEL_DYN_INT,"|",sval,exceptionInfo);
            	}
                paramsAlertSum[":SUMDPELIST"]="\'"+sval+"\'";
                if (dynlen(exceptionInfo)) return;
                _fwConfigurationDB_bindExecuteCommand(cmdAlertSum, paramsAlertSum, exceptionInfo);
                if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

            } else {
                // store dpe-alert information...

                string sval;
                paramsAlertDpe[":IPROP_ID"]=iprop_id;
                dyn_string ds=alertData[4][i];
                _fwConfigurationDB_dataToString(ds , DPEL_DYN_STRING,"|", sval,exceptionInfo);
        	// fix #17872
        	strreplace(sval,"\'","\'\'");

                paramsAlertDpe[":TEXTS"]="\'"+sval+"\'";
                dyn_float df=alertData[5][i];
                _fwConfigurationDB_dataToString(df , DPEL_DYN_FLOAT,"|", sval,exceptionInfo);
                paramsAlertDpe[":LIMITS"]="\'"+sval+"\'";
                ds=alertData[6][i];
                _fwConfigurationDB_dataToString(ds , DPEL_DYN_STRING,"|", sval,exceptionInfo);
        	// fix #17872
        	strreplace(sval,"\'","\'\'");

                paramsAlertDpe[":CLASSES"]="\'"+sval+"\'";
                if (dynlen(exceptionInfo)) return;
                _fwConfigurationDB_bindExecuteCommand(cmdAlertDpe, paramsAlertDpe, exceptionInfo);
                if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
            }

            string alertPanel=alertData[8][i];
            if (alertPanel!="") {
                string sval;
                paramsAlertMisc[":IPROP_ID"]=iprop_id;
                paramsAlertMisc[":PANEL"]="\'"+alertPanel+"\'";
                dyn_string ds=alertData[9][i];
                _fwConfigurationDB_dataToString(ds , DPEL_DYN_STRING,"|",sval,exceptionInfo);
        	// fix #17872
        	strreplace(sval,"\'","\'\'");

                paramsAlertMisc[":PPARAMS"]="\'"+sval+"\'";
                paramsAlertMisc[":HELP"]   ="\'"+alertData[10][i]+"\'";
                if (dynlen(exceptionInfo)) return;
                _fwConfigurationDB_bindExecuteCommand(cmdAlertMisc, paramsAlertMisc, exceptionInfo);
                if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
            }
        }

    _fwConfigurationDB_finishCommand(cmdAlert, exceptionInfo);
    _fwConfigurationDB_finishCommand(cmdAlertDpe, exceptionInfo);
    _fwConfigurationDB_finishCommand(cmdAlertSum, exceptionInfo);
    _fwConfigurationDB_finishCommand(cmdAlertMisc, exceptionInfo);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    }

_fwConfigurationDB_endFunction("saveAlertConfiguration", t0);
}


/** Loads alert configuration from Configuration Database
 *
 * @param dpes      list of the data point element names for which configuration is loaded
 * @param ipropIds  corresponding list of IPROP_ID identifiers (in the database) of the data-point elements
 * @param alertData on return contains the configuration data loaded from database (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_loadAlertConfiguration(dyn_int propIds, dyn_string dpes,
                                               dyn_dyn_mixed &alertData,
                                               dyn_dyn_mixed &sumAlertData,
                                               dyn_dyn_mixed &sumAlertByPatternData,
                                               dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("loadAlertConfiguration", t0);
    dynClear(alertData);
    dynClear(sumAlertData);
    dynClear(sumAlertByPatternData);
    alertData[13][1]="";// make the object valid: create at least 11 columns
    sumAlertData[13][1]="";
    sumAlertByPatternData[13][1]="";

//    string propidsListAsSql=_fwConfigurationDB_listToSQLString("IPROP_ID", propIds);

    //string sql="SELECT IPROP_ID, TYPE, ACTIVE, TEXTS, LIMITS, CLASSES, "+
    //    " SUMDPELIST,PANEL,PANELPARAMS,HELP FROM V_CONFIG_ALERTS "+
    //    " WHERE "+propidsListAsSql;
    string sql="SELECT IPROP_ID, TYPE, ACTIVE, TEXTS, LIMITS, CLASSES, "+
        " SUMDPELIST,PANEL,PANELPARAMS,HELP FROM V_CONFIG_ALERTS "+
        " where iprop_id in ( select iprop_id from tmp_item_props )";

    dyn_dyn_mixed aRecords;

    _fwConfigurationDB_executeDBQuery(sql,g_fwConfigurationDB_DBConnection, aRecords, exceptionInfo,10,TRUE);
    if (dynlen(exceptionInfo)) return;
    if (dynlen(aRecords)<1) { _fwConfigurationDB_endFunction("loadAlertConfiguration", t0); return;}

    for (int i=1;i<=dynlen(aRecords[1]);i++) {
	int alertType=aRecords[2][i];
        int idx=0;
        int propidx=dynContains(propIds,aRecords[1][i]);
        switch (alertType) {

        case DPCONFIG_SUM_ALERT:
    	    if (strpos((string)aRecords[7][i],"PATTERN:")==0) {
    		//DP list specified by pattern
                idx=dynlen(sumAlertByPatternData[1])+1;
                sumAlertByPatternData[1][idx]=dpes[propidx]; //dpe
                sumAlertByPatternData[2][idx]=aRecords[1][i]; //iprop_id
                sumAlertByPatternData[3][idx]=aRecords[2][i]; // type
                sumAlertByPatternData[8][idx]=aRecords[8][i]; // alertPanel
                sumAlertByPatternData[10][idx]=aRecords[10][i]; // alertHelp
                sumAlertByPatternData[11][idx]=aRecords[3][i]; // active

		// explicit cast to string needed by substr: thank you Robert Stringer!
                sumAlertByPatternData[7][idx]=makeDynString(substr((string)aRecords[7][i],8)); // dp pattern
                sumAlertByPatternData[12][idx]=makeDynInt(-1); // dp pattern

            // alert texts:
                _fwConfigurationDB_stringToData(aRecords [4][i], DPEL_DYN_STRING, "|",
                                                sumAlertByPatternData[4][idx], exceptionInfo);
            // alert limits:
                _fwConfigurationDB_stringToData(aRecords [5][i], DPEL_DYN_FLOAT, "|",
                                                sumAlertByPatternData[5][idx], exceptionInfo);
            // alert classes:
                _fwConfigurationDB_stringToData(aRecords [6][i], DPEL_DYN_STRING, "|",
                                                sumAlertByPatternData[6][idx], exceptionInfo);
                // panel params
                _fwConfigurationDB_stringToData(aRecords [9][i], DPEL_DYN_STRING, "|",
                                                sumAlertByPatternData[9][idx], exceptionInfo);


	       } else {
		// DP list specified explicitely
                idx=dynlen(sumAlertData[1])+1;
                sumAlertData[1][idx]=dpes[propidx]; //dpe
                sumAlertData[2][idx]=aRecords[1][i]; //iprop_id
                sumAlertData[3][idx]=aRecords[2][i]; // type
                sumAlertData[8][idx]=aRecords[8][i]; // alertPanel
                sumAlertData[10][idx]=aRecords[10][i]; // alertHelp
                sumAlertData[11][idx]=aRecords[3][i]; // active
                            // alert texts:
                _fwConfigurationDB_stringToData(aRecords [4][i], DPEL_DYN_STRING, "|",
                                                sumAlertData[4][idx], exceptionInfo);
            // alert limits:
                _fwConfigurationDB_stringToData(aRecords [5][i], DPEL_DYN_FLOAT, "|",
                                                sumAlertData[5][idx], exceptionInfo);
            // alert classes:
                _fwConfigurationDB_stringToData(aRecords [6][i], DPEL_DYN_STRING, "|",
                                                sumAlertData[6][idx], exceptionInfo);
                // panel params
                _fwConfigurationDB_stringToData(aRecords [9][i], DPEL_DYN_STRING, "|",
                                                sumAlertData[9][idx], exceptionInfo);


        	// we have it as a list of ID's
            	dyn_int alertDpeIds;
            	dyn_string alertDpeNames;
            	_fwConfigurationDB_stringToData(aRecords [7][i], DPEL_DYN_INT, "|",
                                                alertDpeIds, exceptionInfo);
        	// now convert ids to dps for dpelist...
            	sumAlertData[12][idx]=alertDpeIds;
            	for (int j=1;j<=dynlen(alertDpeIds);j++) {
            	    int alertIdx=dynContains(propIds,alertDpeIds[j]);
            	    if (alertIdx<1) {
                        fwException_raise(exceptionInfo,"ERROR","Inconsistency in summary alert data","");
                        DebugN("Inconsistency in alert data! Cannot resolve ipropid="+alertDpeIds+" for summary alert of "+sumAlertData[1][idx]);
                        DebugN("IpropId of dp with sumalert:",aRecords[1][i]);
                        return;
            	    }
            	    dynAppend(alertDpeNames,dpes[alertIdx]);
            	}
            	sumAlertData[7][idx]=alertDpeNames;
	       }

	    break;

        default:

                idx=dynlen(alertData[1])+1;
                alertData[1][idx]=dpes[propidx]; //dpe
                alertData[2][idx]=aRecords[1][i]; //iprop_id
                alertData[3][idx]=aRecords[2][i]; // type
                alertData[8][idx]=aRecords[8][i]; // alertPanel
                alertData[10][idx]=aRecords[10][i]; // alertHelp
                alertData[11][idx]=aRecords[3][i]; // active
                        // alert texts:
                _fwConfigurationDB_stringToData(aRecords [4][i], DPEL_DYN_STRING, "|",
                                                alertData[4][idx], exceptionInfo);
            // alert limits:
                _fwConfigurationDB_stringToData(aRecords [5][i], DPEL_DYN_FLOAT, "|",
                                                alertData[5][idx], exceptionInfo);
            // alert classes:
                _fwConfigurationDB_stringToData(aRecords [6][i], DPEL_DYN_STRING, "|",
                                                alertData[6][idx], exceptionInfo);
                // panel params
                _fwConfigurationDB_stringToData(aRecords [9][i], DPEL_DYN_STRING, "|",
                                                alertData[9][idx], exceptionInfo);



            break;

        }// end of switch()

    }// end of for loop

    _fwConfigurationDB_endFunction("loadAlertConfiguration", t0);
}



/** Extracts dp function configuration from PVSS
 *
 * @param dpes      list of the data point element names for which configuration is requested
 * @param ipropIds  corresponding list of IPROP_ID identifiers (in the database) of the data-point elements
 * @param dpFunctionData on return contains the configuration data (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_extractDpFunctionConfiguration(dyn_string dpes,dyn_int ipropIds,
    dyn_dyn_mixed &dpFunctionData, dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("extractDpFunctionConfiguration", t0);
    dynClear(dpFunctionData);
    dpFunctionData[6][1]="";// make the object valid: (number of columns)

    dyn_bool configExists;
    dyn_dyn_string functionParams,functionGlobals;
    dyn_string functionDefinition;

    fwDpFunction_getDpeConnectionMany(dpes, configExists, functionParams,
                                      functionGlobals, functionDefinition, exceptionInfo);
    if (dynlen(exceptionInfo)) return;
    for (int i=1;i<=dynlen(dpes);i++) {
        if (configExists[i]) {
            int idx=dynlen(dpFunctionData[1])+1;
            dpFunctionData[1][idx]=dpes[i];
            dpFunctionData[2][idx]=ipropIds[i];
            dpFunctionData[3][idx]=functionParams[i];
            dpFunctionData[4][idx]=functionGlobals[i];
            dpFunctionData[5][idx]=functionDefinition[i];
        }
    }

    _fwConfigurationDB_endFunction("extractDpFunctionConfiguration", t0);
}

/** Saves dp function configuration to Configuration Database
 *
 * @param dpFunctionData should contain the configuration data to be stored (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_saveDpFunctionConfiguration(dyn_dyn_mixed &dpFunctionData, dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("saveDpFunctionConfiguration", t0);
    if (dynlen(dpFunctionData[1])>0) {
        anytype cmd;
        string sql="INSERT INTO CONFIG_DPFUNCTIONS (IPROP_ID, PARAMS, GLOBALS, DEFINITION)"+
            " VALUES (:IPROP_ID, :PARAMS, :GLOBALS, :DEFINITION)";

        _fwConfigurationDB_startCommand(sql, g_fwConfigurationDB_DBConnection,
                                        cmd, exceptionInfo);

        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
        for (int i=1;i<=dynlen(dpFunctionData[1]);i++) {
            mapping params;
            params[":IPROP_ID"]=dpFunctionData[2][i];
            string sval;
            dyn_string ds=dpFunctionData[3][i];
            _fwConfigurationDB_dataToString(ds , DPEL_DYN_STRING,"|", sval,exceptionInfo);
            params[":PARAMS"]="\'"+sval+"\'";
            ds=dpFunctionData[4][i];
            _fwConfigurationDB_dataToString(ds , DPEL_DYN_STRING,"|", sval,exceptionInfo);
            params[":GLOBALS"]="\'"+sval+"\'";
            params[":DEFINITION"]="\'"+dpFunctionData[5][i]+"\'";

            _fwConfigurationDB_bindExecuteCommand(cmd, params, exceptionInfo);
            if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
        }
        _fwConfigurationDB_finishCommand(cmd, exceptionInfo);
        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    }

    _fwConfigurationDB_endFunction("saveDpFunctionConfiguration", t0);
}





/** Extracts conversion configuration from PVSS
 *
 * @param dpes      list of the data point element names for which configuration is requested
 * @param ipropIds  corresponding list of IPROP_ID identifiers (in the database) of the data-point elements
 * @param conversionData on return contains the configuration data (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_extractConversionConfiguration(dyn_string dpes,dyn_int ipropIds,
    dyn_dyn_mixed &conversionData, dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("extractConversionConfiguration", t0);
    dynClear(conversionData);
    conversionData[7][1]="";// make the object valid: (number of columns)


    dyn_bool configExistsCmd,configExistsMsg;
    dyn_int conversionTypeCmd,conversionTypeMsg, orderCmd, orderMsg;
    dyn_dyn_float argumentsCmd,argumentsMsg;

    // message conversion
    int configTypeMsg=DPCONFIG_CONVERSION_RAW_TO_ENG_MAIN;
    int configTypeCmd=DPCONFIG_CONVERSION_ING_TO_RAW_MAIN;
    fwConfigConversion_getMany(dpes, configExistsMsg, configTypeMsg,
                               conversionTypeMsg, orderMsg,argumentsMsg, exceptionInfo);
    fwConfigConversion_getMany(dpes, configExistsCmd, configTypeCmd,
                               conversionTypeCmd, orderCmd,argumentsCmd, exceptionInfo);

    if (dynlen(exceptionInfo)) return;
    for (int i=1;i<=dynlen(dpes);i++) {
        if (configExistsMsg[i]) {
            int idx=dynlen(conversionData[1])+1;
            conversionData[1][idx]=dpes[i];
            conversionData[2][idx]=ipropIds[i];
            conversionData[3][idx]=configTypeMsg;
            conversionData[4][idx]=conversionTypeMsg[i];
            conversionData[5][idx]=orderMsg[i];
            conversionData[6][idx]=argumentsMsg[i];
        }
        if (configExistsCmd[i]) {
            int idx=dynlen(conversionData[1])+1;
            conversionData[1][idx]=dpes[i];
            conversionData[2][idx]=ipropIds[i];
            conversionData[3][idx]=configTypeCmd;
            conversionData[4][idx]=conversionTypeCmd[i];
            conversionData[5][idx]=orderCmd[i];
            conversionData[6][idx]=argumentsCmd[i];
        }
    }

    _fwConfigurationDB_endFunction("extractConversionConfiguration", t0);
}

/** Saves conversion configuration to Configuration Database
 *
 * @param conversionData should contain the configuration data to be stored (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_saveConversionConfiguration(dyn_dyn_mixed &conversionData,
    dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("saveConversionConfiguration", t0);

    if (dynlen(conversionData[1])>0) {
        anytype cmd;
        string sql="INSERT INTO CONFIG_CONVERSIONS (IPROP_ID, TYPE, CONVTYPE, CONVORDER, ARGUMENTS)"+
            " VALUES (:IPROP_ID, :TYPE, :CONVTYPE, :CONVORDER, :ARGUMENTS)";

        _fwConfigurationDB_startCommand(sql, g_fwConfigurationDB_DBConnection,
                                        cmd, exceptionInfo);

        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
        for (int i=1;i<=dynlen(conversionData[1]);i++) {
            mapping params;
            params[":IPROP_ID"]=conversionData[2][i];
            params[":TYPE"]=conversionData[3][i];
            params[":CONVTYPE"]=conversionData[4][i];
            params[":CONVORDER"]=conversionData[5][i];

            string sval;
            dyn_float ds=conversionData[6][i];
            _fwConfigurationDB_dataToString(ds , DPEL_DYN_FLOAT,"|", sval,exceptionInfo);
            params[":ARGUMENTS"]="\'"+sval+"\'";
            _fwConfigurationDB_bindExecuteCommand(cmd, params, exceptionInfo);
            if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
        }
        _fwConfigurationDB_finishCommand(cmd, exceptionInfo);
        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    }

    _fwConfigurationDB_endFunction("saveConversionConfiguration", t0);
}

/** Extracts pv range configuration from PVSS
 *
 * @param dpes      list of the data point element names for which configuration is requested
 * @param ipropIds  corresponding list of IPROP_ID identifiers (in the database) of the data-point elements
 * @param pvRangeData on return contains the configuration data (internal format)
 *					columns 1-
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_extractPvRangeConfiguration(dyn_string dpes,dyn_int ipropIds,
    dyn_dyn_mixed &pvRangeData, dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("extractPvRangeConfiguration", t0);

    dynClear(pvRangeData);

    dyn_bool configExists;
    dyn_int pvRangeTypes;
    dyn_dyn_mixed configData;
    fwPvRange_getObjectMany( dpes, configExists, pvRangeTypes,
                            configData, exceptionInfo);
    if (dynlen(exceptionInfo)) return;
    for (int i=1;i<=dynlen(configData);i++) {
	if (!configExists[i]) continue;
	dyn_mixed obj=configData[i];

	obj[9]=dpes[i];
	obj[10]=ipropIds[i];
	obj[11]=pvRangeTypes[i];
	dynAppend(pvRangeData,obj);
    }
/*
    dyn_bool configExists,negateRange,ignoreOutside,inclusiveMin,inclusiveMax;
    dyn_float minValue, maxValue;

    //// #17981/#17982 - pv ranges of type RANGECHECK not supported by Framework.
    //// we skip saving pvrange for them as of now...

    dyn_string chkDpes,pvrDpes;
    dyn_int pvrIpropIds;
    dyn_int chktypes;
    for (int i=1;i<=dynlen(dpes);i++) {
        dynAppend(chkDpes,dpes[i]+":_pv_range.._type");
    }
    dpGet(chkDpes,chktypes);
    for (int i=1;i<=dynlen(chkDpes);i++) {
        switch(chktypes[i]) {
        case DPCONFIG_NONE:
            break;
        case DPCONFIG_MINMAX_PVSS_RANGECHECK:
            dynAppend(pvrDpes,dpes[i]);
            dynAppend(pvrIpropIds,ipropIds[i]);
            break;
        case DPCONFIG_SET_PVSS_RANGECHECK:
            DebugN("WARNING! Unsupported PVRANGE of type RANGECHECK for "+dpes[i]+" ->skipping it");
            break;
        default:
            fwException_raise(exceptionInfo,"ERROR",
                              "PVRange type "+chktypes[i]+" not supported for "+dpes[i],"");
            return;
            break;
        }

    }
// then substitute lists...
    dpes=pvrDpes;
    ipropIds=pvrIpropIds;

//DebugN("calling getMany for",dpes);

    fwPvRange_getMany(dpes, configExists, minValue, maxValue,
                      negateRange, ignoreOutside,
                      inclusiveMin, inclusiveMax, exceptionInfo);
    if (dynlen(exceptionInfo)) return;
    for (int i=1;i<=dynlen(dpes);i++) {
        if (configExists[i]) {
            int idx=dynlen(pvRangeData[1])+1;
            pvRangeData[1][idx]=dpes[i];
            pvRangeData[2][idx]=ipropIds[i];
            pvRangeData[3][idx]=minValue[i];
            pvRangeData[4][idx]=maxValue[i];
            pvRangeData[5][idx]=negateRange[i];
            pvRangeData[6][idx]=ignoreOutside[i];
            pvRangeData[7][idx]=inclusiveMin[i];
            pvRangeData[8][idx]=inclusiveMax[i];
        }
    }
*/
    _fwConfigurationDB_endFunction("extractPvRangeConfiguration", t0);
}

/** Saves pv range configuration to Configuration Database
 *
 * @param pvRangeData should contain the configuration data to be stored (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_savePvRangeConfiguration(dyn_dyn_mixed &pvRangeData,
    dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("savePvRangeConfiguration", t0);
    if ((dynlen(pvRangeData)>0) && (dynlen(pvRangeData[1])>0)) {
        anytype cmd;
        string sql="INSERT INTO CONFIG_PVRANGES (IPROP_ID, PVRTYPE, MINVALUE, MAXVALUE, "+
                    " NEGRANGE, IGNOUTSIDE, INCLMIN, INCLMAX, PVRVALS)"+
                    " VALUES (:IPROP_ID, :PVRTYPE, :MINVALUE, :MAXVALUE, :NEGRANGE, "+
                    " :IGNOUTSIDE, :INCLMIN, :INCLMAX, :PVRVALS)";

        _fwConfigurationDB_startCommand(sql, g_fwConfigurationDB_DBConnection,
                                        cmd, exceptionInfo);

        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
        for (int i=1;i<=dynlen(pvRangeData);i++) {
            mapping params;
            params[":MINVALUE"]  = (float)pvRangeData[i][fwPvRange_MINIMUM_VALUE];
            params[":MAXVALUE"]  = (float)pvRangeData[i][fwPvRange_MAXIMUM_VALUE];
            params[":NEGRANGE"]  = (int)pvRangeData[i][fwPvRange_NEGATE_RANGE];
            params[":IGNOUTSIDE"]= (int)pvRangeData[i][fwPvRange_IGNORE_OUTSIDE];
            params[":INCLMIN"]   = (int)pvRangeData[i][fwPvRange_INCLUSIVE_MINIMUM];
            params[":INCLMAX"]   = (int)pvRangeData[i][fwPvRange_INCLUSIVE_MAXIMUM];
            if (pvRangeData[i][11]==DPCONFIG_MATCH_PVSS_RANGECHECK) {
        	params[":PVRVALS"]   = "\'"+pvRangeData[i][fwPvRange_VALUE_PATTERN]+"\'";
            } else if (pvRangeData[i][11]==DPCONFIG_SET_PVSS_RANGECHECK) {
        	dyn_string data=pvRangeData[i][fwPvRange_VALUE_SET];
        	string encodedData;
        	_fwConfigurationDB_dataToString(data,DPEL_DYN_STRING, "|", encodedData,exceptionInfo);
		if (dynlen(exceptionInfo)) return;
        	params[":PVRVALS"] = "\'"+encodedData+"\'";
            } else {
        	params[":PVRVALS"]="''";
            }

            params[":IPROP_ID"]  = pvRangeData[i][10];
            params[":PVRTYPE"]   = (int) pvRangeData[i][11];
            _fwConfigurationDB_bindExecuteCommand(cmd, params, exceptionInfo);
            if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
        }
        _fwConfigurationDB_finishCommand(cmd, exceptionInfo);
        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    }
    _fwConfigurationDB_endFunction("savePvRangeConfiguration", t0);
}

/** Extracts smoothing configuration from PVSS
 *
 * @param dpes      list of the data point element names for which configuration is requested
 * @param ipropIds  corresponding list of IPROP_ID identifiers (in the database) of the data-point elements
 * @param smoothingData on return contains the configuration data (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_extractSmoothingConfiguration(dyn_string dpes,dyn_int ipropIds,
    dyn_dyn_mixed &smoothingData, dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("extractSmoothingConfiguration", t0);
    dynClear(smoothingData);
    smoothingData[6][1]="";// make the object valid: (number of columns)
    dyn_bool configExists;
    dyn_int smoothProcedure;
    dyn_float deadband, timeInterval;

    fwSmoothing_getMany(dpes, configExists, smoothProcedure, deadband, timeInterval, exceptionInfo);
    if (dynlen(exceptionInfo)) return;
    for (int i=1;i<=dynlen(dpes);i++) {
        if (configExists[i]) {
            int idx=dynlen(smoothingData[1])+1;
            smoothingData[1][idx]=dpes[i];
            smoothingData[2][idx]=ipropIds[i];
            smoothingData[3][idx]=smoothProcedure[i];
            smoothingData[4][idx]=deadband[i];
            smoothingData[5][idx]=timeInterval[i];
        }
    }
    _fwConfigurationDB_endFunction("extractSmoothingConfiguration", t0);
}

/** Saves smoothing configuration to Configuration Database
 *
 * @param smoothingData should contain the configuration data to be stored (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_saveSmoothingConfiguration(dyn_dyn_mixed &smoothingData,
    dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("saveSmoothingConfiguration", t0);
    if (dynlen(smoothingData[1])>0) {
        anytype cmd;
        string sql="INSERT INTO CONFIG_SMOOTHING (IPROP_ID, PROC, DEADBAND, TIMEINTVAL) "+
            " VALUES (:IPROP_ID, :PROC, :DEADBAND, :TIMEINTVAL) ";

        _fwConfigurationDB_startCommand(sql, g_fwConfigurationDB_DBConnection,
                                        cmd, exceptionInfo);

        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
        for (int i=1;i<=dynlen(smoothingData[1]);i++) {
            mapping params;
            params[":IPROP_ID"]  = smoothingData[2][i];
            params[":PROC"]      = smoothingData[3][i];
            params[":DEADBAND"]  = smoothingData[4][i];
            params[":TIMEINTVAL"]= smoothingData[5][i];
            _fwConfigurationDB_bindExecuteCommand(cmd, params, exceptionInfo);
            if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
        }
        _fwConfigurationDB_finishCommand(cmd, exceptionInfo);
        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    }

    _fwConfigurationDB_endFunction("saveSmoothingConfiguration", t0);
}


/** Extracts unit and format configuration from PVSS
 *
 * @param dpes      list of the data point element names for which configuration is requested
 * @param ipropIds  corresponding list of IPROP_ID identifiers (in the database) of the data-point elements
 * @param unitAndFormatData on return contains the configuration data (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_extractUnitAndFormatConfiguration(dyn_string dpes,dyn_int ipropIds,
    dyn_dyn_mixed &unitAndFormatData, dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("extractUnitAndFormatConfiguration", t0);
    dynClear(unitAndFormatData);
    unitAndFormatData[5][1]="";// make the object valid: (number of columns)

    dyn_bool unitExists,formatExists;
    dyn_string unit,format;

    fwUnit_getMany  (dpes, unitExists,   unit,  exceptionInfo);
    fwFormat_getMany(dpes, formatExists, format,exceptionInfo);
    if (dynlen(exceptionInfo)) return;
    for (int i=1;i<=dynlen(dpes);i++) {
        if ((unitExists[i])||(formatExists[i])) {
            int idx=dynlen(unitAndFormatData[1])+1;
            unitAndFormatData[1][idx]=dpes[i];
            unitAndFormatData[2][idx]=ipropIds[i];
            if (unitExists[i])
                unitAndFormatData[3][idx]=unit[i];
            else
                unitAndFormatData[3][idx]="";
            if (formatExists[i])
                unitAndFormatData[4][idx]=format[i];
            else
                unitAndFormatData[4][idx]="";
        }
    }
    _fwConfigurationDB_endFunction("extractUnitAndFormatConfiguration", t0);

}

/** Saves unit and format configuration to Configuration Database
 *
 * @param unitAndFormatData should contain the configuration data to be stored (internal format)
 * @param exceptionInfo standard exception handling variable
 */
void _fwConfigurationDB_saveUnitAndFormatConfiguration(dyn_dyn_mixed &unitAndFormatData,
    dyn_string &exceptionInfo)
{
    time t0;
    _fwConfigurationDB_startFunction("saveUnitAndFormatConfiguration", t0);
    if (dynlen(unitAndFormatData[1])>0) {
        anytype cmd;
        string sql="INSERT INTO CONFIG_UF (IPROP_ID, UNIT, FMT) "+
            " VALUES (:IPROP_ID, :UNIT, :FMT) ";

        _fwConfigurationDB_startCommand(sql, g_fwConfigurationDB_DBConnection,
                                        cmd, exceptionInfo);

        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
        for (int i=1;i<=dynlen(unitAndFormatData[1]);i++) {
            mapping params;
            params[":IPROP_ID"]  = unitAndFormatData[2][i];
            params[":UNIT"]      = "\'"+unitAndFormatData[3][i]+"\'";
            params[":FMT"]       = "\'"+unitAndFormatData[4][i]+"\'";
            _fwConfigurationDB_bindExecuteCommand(cmd, params, exceptionInfo);
            if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
        }
        _fwConfigurationDB_finishCommand(cmd, exceptionInfo);
        if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    }

    _fwConfigurationDB_endFunction("saveUnitAndFormatConfiguration", t0);
}

void _fwConfigurationDB_resolveLogicalIds(dyn_dyn_mixed &deviceListObject, dyn_string &exceptionInfo, string systemName)
{

    // the dpalias->dpname thing was not resolved yet... do it!
    for (int i=1;i<=dynlen(deviceListObject[fwConfigurationDB_DLO_DPNAME]);i++) {
	string itemname=deviceListObject[fwConfigurationDB_DLO_DPNAME][i];
	string dpname=dpAliasToName(itemname);
	dpname=dpSubStr(dpname,DPSUB_SYS_DP);
	deviceListObject[fwConfigurationDB_DLO_REFDP][i]=dpname;
    }


    _fwConfigurationDB_ensureDeviceListObjectValid(deviceListObject);

    dyn_int refIds,refIdx;
    dyn_string refDPNames;
    // we will look only for the ones that are not Nodes, because they are not stored in HW view!
    for (int i=1;i<=dynlen(deviceListObject[fwConfigurationDB_DLO_REFDP]);i++) {
    	if (deviceListObject[fwConfigurationDB_DLO_DPTYPE][i]=="FwNode") {
	    // give a link to itself!
	    deviceListObject[fwConfigurationDB_DLO_REFID][i]=deviceListObject[fwConfigurationDB_DLO_ITEMID][i];
	    continue;
	}
	dynAppend(refIdx,i);
	dynAppend(refDPNames,deviceListObject[fwConfigurationDB_DLO_REFDP][i]);
    }


     _fwConfigurationDB_DBFindItems(fwDevice_HARDWARE, refDPNames, refIds, exceptionInfo);
    if (dynlen(exceptionInfo)) return;
    for (int i=1;i<=dynlen(refIdx);i++) {
	deviceListObject[fwConfigurationDB_DLO_REFID][refIdx[i]]=refIds[i];
    }
}


/** Creates devices in DB

The function stores the devices in the appropriate hierarchy, without storing
their properties.
Note that the devices that are already present in the db will not be re-created.

CONVENTIONS:
     1) for HARDWARE hierarchy:
        *) systemName needs to be set. "" is equal to passing getSystemName()
        *) specifying the name of other (remote) system allows to save its hierarchy!
        *) topDevice and deviceDpNames needs to contain system name
        *) if sysname is not specified at all - use the one from systemName
        *) if sysname is included but differs from systemName: report an error
     2) for LOGICAL hierarchy
        A) when systemName!="" - use specified system name
           *) topDevice and deviceDpNames need to contain system name
           *) if sysname not specified - complete it from systemName
           *) if if differs: report an error
        B) when systemName=="", we have a hierarchy with common root across all systems
           (i.e. logical hierarchy without system names)
           *) topDevice and deviceDpNames must not include system name
           *) we simply cut it away if it was passed

@param topDevice - a "hint" information that tells where to start verification of whether device
		    exists, etc. Empty string ("") means the top of hierarchy
*/
void _fwConfigurationDB_saveDeviceHierarchyToDB(string topDevice, string hierarchyType, dyn_string deviceDpNames,
                                         dyn_string &exceptionInfo, string systemName, dyn_dyn_mixed &deviceListObject)
{

    time t0;
    _fwConfigurationDB_startFunction("saveDeviceHierarchyToDB", t0);

    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveHierarchyToDB,"Saving hierarchy in DB", 0.0,
        exceptionInfo)) return;

    _fwConfigurationDB_ensureDeviceListObjectValid(deviceListObject);
    if (hierarchyType==fwDevice_LOGICAL) {
    	_fwConfigurationDB_DBCheckCreateSystemNode(fwDevice_LOGICAL,exceptionInfo);
        if (dynlen(exceptionInfo)) return;

	if (systemName=="") {
	    //unified-root logical hierarchy
	    if (topDevice=="") {
		//topDevice="/"; // NO! The top of the unified-root hierarchy (as "DPNAME") is empty string! It is "NAME" that is "/"
	    } else {
		topDevice= _fwConfigurationDB_NodeNameWithoutSystem(topDevice);
	    }
	    for (int i=1;i<=dynlen(deviceDpNames);i++) {
		deviceDpNames[i]=_fwConfigurationDB_NodeNameWithoutSystem(deviceDpNames[i]);
	    }
	} else { // logical hierarchy with sys names
	    topDevice=_fwConfigurationDB_NodeNameWithSystem(topDevice, systemName, exceptionInfo);
	    if (dynlen(exceptionInfo)) return;
	    for (int i=1;i<=dynlen(deviceDpNames);i++) {
		deviceDpNames[i]=_fwConfigurationDB_NodeNameWithSystem(deviceDpNames[i], systemName, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
	    }
	}

    } else if (hierarchyType==fwDevice_HARDWARE) {
	_fwConfigurationDB_DBCheckCreateSystemNode(fwDevice_HARDWARE,exceptionInfo);
        if (dynlen(exceptionInfo)) return;

	if (systemName=="") systemName=getSystemName();
	topDevice=_fwConfigurationDB_NodeNameWithSystem(topDevice, systemName, exceptionInfo);
	if (dynlen(exceptionInfo)) return;
	for (int i=1;i<=dynlen(deviceDpNames);i++) {
	    deviceDpNames[i]=_fwConfigurationDB_NodeNameWithSystem(deviceDpNames[i], systemName, exceptionInfo);
	    if (dynlen(exceptionInfo)) return;
	}

    } else {
	fwException_raise(exceptionInfo,"ERROR","Unsupported hierarchy type: "+hierarchyType,"");
	return;
    }


    // separate the ones that exist and the ones that does not exist
    dyn_dyn_mixed missingDevicesListObject;
    _fwConfigurationDB_ensureDeviceListObjectValid(missingDevicesListObject);
    dyn_string missingDevicesList;
    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveHierarchyToDB,"Resolving devices in DB",10.0, exceptionInfo)) return;
    fwConfigurationDB_findDevicesInDB (topDevice, hierarchyType, deviceDpNames,
                                       deviceListObject, missingDevicesList,
                                       exceptionInfo, systemName);
    if (dynlen(exceptionInfo)) return;

    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveHierarchyToDB,"Verifying existing devices in DB",30.0, exceptionInfo)) return;
    // verify that the ones that exist are OK...
    // maybe we should not return an error, but rather update
    // the information ? or suggest creating new hierarchy?

    anytype dummyMissing;
    _fwConfigurationDB_verifyDatapoints (hierarchyType, deviceListObject, dummyMissing, exceptionInfo,TRUE);
    if (dynlen(exceptionInfo)) return;

    // stop here if there is nothing to store...
    if ( dynlen(missingDevicesList)==0) {
	fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveHierarchyToDB,"Nothing to store",100.0, exceptionInfo);
	fwConfigurationDB_progress(fwConfigurationDB_OPER_UpdateDBHierarchyInfo,"Nothing to update", 100.0,exceptionInfo);
	_fwConfigurationDB_endFunction("saveDeviceHierarchyToDB", t0);
	return;
    }


    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveHierarchyToDB,"Resolving device IDs",50.0, exceptionInfo)) return;


    // for the missing ones, complete the information about the types, etc
    // note, we do not use the information about missing devices, gathered in
    // the function above!
    fwConfigurationDB_expandToDeviceListObject(hierarchyType, missingDevicesList, missingDevicesListObject,exceptionInfo);
    if (dynlen(exceptionInfo)) return;

    _fwConfigurationDB_DBAssignItemIds(hierarchyType,missingDevicesListObject, exceptionInfo,systemName);
    if (dynlen(exceptionInfo)) return;





    if (dynlen(missingDevicesList) > 0 ) {
	_fwConfigurationDB_saveItemsToDB_OLD(missingDevicesListObject, hierarchyType, exceptionInfo);
	if (dynlen(exceptionInfo)) return;
    }



/*
    // trigger the update of the materialized view:
    _fwConfigurationDB_updateHierarchyView(g_fwConfigurationDB_DBConnection, exceptionInfo);
    if (dynlen(exceptionInfo)) return;
*/

    // the missing devices were stored by now, re-combine them
    // to the deviceListObject...
    for (int i=1;i<=fwConfigurationDB_DLO_MAX_IDX;i++) {
	dyn_mixed row=missingDevicesListObject[i];
	dynAppend(deviceListObject[i],row);
    }


    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveHierarchyToDB,"Saved hierarchy in DB", 100.0,
        exceptionInfo)) return;

    _fwConfigurationDB_endFunction("saveDeviceHierarchyToDB", t0);

}


void fwConfigurationDB_saveReferences(string configurationName, dyn_dyn_mixed deviceListObject, dyn_string &exceptionInfo)
{
    // choose the things to store...
    dyn_string devices;
    dyn_string aliases;
    for (int i=1;i<=dynlen(deviceListObject[fwConfigurationDB_DLO_DPNAME]);i++) {
	if (deviceListObject[fwConfigurationDB_DLO_DPTYPE][i]=="FwNode") continue;
	if (deviceListObject[fwConfigurationDB_DLO_REFID][i]==0) continue;
	dynAppend(aliases,deviceListObject[fwConfigurationDB_DLO_DPNAME][i]);
	dynAppend(devices,deviceListObject[fwConfigurationDB_DLO_REFDP][i]);
    }

    dyn_dyn_mixed dummyOutput;
    fwConfigurationDB_callPlSqlApi( "setReferences",
                        makeDynMixed(configurationName),
                        "S1,S2", "",
                        makeDynMixed(aliases,devices), dummyOutput,
                        exceptionInfo);
    if (dynlen(exceptionInfo)) return;

}

/** Store items in DB (LOW LEVEL!)

All ID's for items already need to be there.
The things it stores: item id, parent id, name, type, model (or detail of type), comment, dpid
Only stores ITEMS-related part; logical mapping, configs, etc are stored elsewhere.
*/
void _fwConfigurationDB_saveItemsToDB_OLD(dyn_dyn_mixed deviceListObject, string hierarchyType, dyn_string &exceptionInfo)
{

    // prepare the "insert" command:
    anytype insertDeviceCmd;
    string insertDeviceSql="INSERT INTO ITEMS (HVER,ID,PARENT,NAME,TYPE,DPT_ID,DETAIL,DESCRIPTION,DPID,DPNAME) "+
			   " VALUES (:HVER, :ID, :PARENT, ':NAME',':DPTYPE',"+
			   " (SELECT DPT_ID FROM DEVICE_TYPES WHERE DPTYPE=':DPTYPE' AND VALID_TO IS NULL),"+
			   " ':DETAIL',:COMMENT,:DPID,':DPNAME')";

    _fwConfigurationDB_startCommand(insertDeviceSql, g_fwConfigurationDB_DBConnection,
                                    insertDeviceCmd, exceptionInfo);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
    //DebugN("To Store:",missingDevicesListObject);

    for (int i=1;i<=dynlen(deviceListObject[fwConfigurationDB_DLO_DPNAME]);i++) {
	    if (i%100==1) {
		//DebugN(i,dynlen(missingDevicesListObject[fwConfigurationDB_DLO_DPNAME]));
    		if (fwConfigurationDB_progress(fwConfigurationDB_OPER_SaveHierarchyToDB,"Saving devices in DB",
        	    70+20.0*i/dynlen(deviceListObject[fwConfigurationDB_DLO_DPNAME]), exceptionInfo)) return;
	    }

    	    mapping params;
    	    params[":HVER"]     =   g_fwConfigurationDB_DBHierarchyIDs[hierarchyType];
    	    params[":ID"]       =   deviceListObject[fwConfigurationDB_DLO_ITEMID][i];
    	    params[":PARENT"]   =   deviceListObject[fwConfigurationDB_DLO_PARENTID][i];
    	    params[":NAME"]     =   deviceListObject[fwConfigurationDB_DLO_NAME][i];
    	    params[":DPTYPE"]   =   deviceListObject[fwConfigurationDB_DLO_DPTYPE][i];
    	    params[":DPNAME"]   =   deviceListObject[fwConfigurationDB_DLO_DPNAME][i];

    	    // fix #17872
    	    string escapedComment=deviceListObject[fwConfigurationDB_DLO_COMMENT][i];
    	    strreplace(escapedComment,"\'","\'\'");
    	    params[":COMMENT"]  =   "\'"+escapedComment+"\'";

    	    params[":DETAIL"]   =   deviceListObject[fwConfigurationDB_DLO_MODEL][i];

	    if (hierarchyType==fwDevice_HARDWARE)
    		params[":DPID"]     =  "\'"+deviceListObject[fwConfigurationDB_DLO_DPID][i]+"\'";
	    else
		params[":DPID"]     =   "NULL";
    	    _fwConfigurationDB_bindExecuteCommand(insertDeviceCmd, params, exceptionInfo);
    	    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

	}; // end of for loop
	_fwConfigurationDB_finishCommand(insertDeviceCmd, exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

}

void fwConfigurationDB_reconnectDevices(dyn_string deviceList, string configurationName,
	string hierarchyType, string targetSystem, dyn_string &exceptionInfo)
{
    if (hierarchyType!=fwDevice_HARDWARE) {
	fwException_raise(exceptionInfo,"ERROR in fwConfigurationDB_reconnectDevices",
	    "Hierarchy type "+hierarchyType+" not supported","");
	return ;
    }

    // find eventual parrent devices - we'll need this to save the hierarchy on the new machine...
    dyn_string allDeviceList=deviceList;
    string system="";

//    string system=getSystemName();
//    _fwConfigurationDB_completeDevicesInHierarchy(allDeviceList, hierarchyType, system, exceptionInfo);
//    if (dynlen(exceptionInfo)) return;


    // find the configuration...
    int refVer=0;
    string refVerSql="SELECT CONF_ID FROM CONFIG_DESC WHERE CONF_TAG=\'"+configurationName+"\'";
    dyn_dyn_mixed arefverdata;
    _fwConfigurationDB_executeDBQuery(refVerSql,g_fwConfigurationDB_DBConnection, arefverdata, exceptionInfo,1,TRUE);
    if (dynlen(exceptionInfo)) return;
    if (dynlen(arefverdata)<1) {
	fwException_raise(exceptionInfo,"ERROR","Cannot find configuration:"+configurationName,"");
	return;
    }
    refVer=arefverdata[1][1];


    _fwConfigurationDB_dbCompleteDevicesInHierarchy(allDeviceList, hierarchyType, configurationName, exceptionInfo);
    if (dynlen(exceptionInfo)) return;

    // locate the existing devices in the DB at first...
    dyn_dyn_mixed deviceListObject,allDeviceListObject;
    time validOn=0;

    // we call the public one, i.e. with commiting the transaction and thus cleanup of tmp_item_props
    fwConfigurationDB_getDeviceConfigurationFromDB(configurationName,
	hierarchyType, deviceListObject, exceptionInfo, validOn, deviceList);
    if (dynlen(exceptionInfo)) return;

    //and get hierarchical info for all ones:
    dyn_string dumMissing;
    fwConfigurationDB_findDevicesInDB("", hierarchyType,allDeviceList, allDeviceListObject, dumMissing, exceptionInfo);
    if (dynlen(exceptionInfo)) return;


    // identify the aliases that we remap...

    string idListSql=_fwConfigurationDB_listToSQLString("ref_id", deviceListObject[fwConfigurationDB_DLO_ITEMID]);

    string alsql="SELECT ID,REF_ID FROM REFERENCES WHERE VALID_TO IS NULL";
    alsql+=" and refver="+refVer;
    alsql+=" and "+idListSql;

    dyn_dyn_mixed aliasData;
    // aliasData[1] - ID (i.e. logical)
    // aliasData[2] - REF_ID (i.e. hardware)
    _fwConfigurationDB_executeDBQuery(alsql,g_fwConfigurationDB_DBConnection, aliasData, exceptionInfo,2,TRUE);
    if (dynlen(exceptionInfo)) return;

    // check if the specified ones exist...
    for (int i=1;i<=dynlen(deviceList);i++) {
	if (!dynContains(allDeviceListObject[fwConfigurationDB_DLO_DPNAME],deviceList[i])) {
	    // if exception not yet thrown - throw it
	    if (!dynlen(exceptionInfo))	{
		fwException_raise(exceptionInfo,"ERROR in fwConfigurationDB_reconnectDevices",
		    "Requested reconnect for device(s) not present in the DB. See log for details","");
		    DebugN("--- ERROR! Requested reconnect for device(s) not present in the DB:");
		    DebugN("    in configuration:"+configurationName);
	    }
	    DebugN("  -> device not found in DB:",deviceList[i]);
	}
    }
    if (dynlen(exceptionInfo)) return;


    // form lists with new system names...
    dyn_string newDeviceList, newAllDeviceList;
    for (int i=1;i<=dynlen(deviceList);i++) {
	string devNoSys=_fwConfigurationDB_NodeNameWithoutSystem(deviceList[i]);
	string devNewSys= _fwConfigurationDB_NodeNameWithSystem(devNoSys, targetSystem, exceptionInfo);
	if (dynlen(exceptionInfo)) return;
	newDeviceList[i]=devNewSys;
    }

    for (int i=1;i<=dynlen(allDeviceList);i++) {
	string devNoSys=_fwConfigurationDB_NodeNameWithoutSystem(allDeviceList[i]);
	string devNewSys= _fwConfigurationDB_NodeNameWithSystem(devNoSys, targetSystem, exceptionInfo);
	if (dynlen(exceptionInfo)) return;
	newAllDeviceList[i]=devNewSys;
    }

    //make sure we have a root node for the target system...
    _fwConfigurationDB_DBCheckCreateSystemNode(hierarchyType,exceptionInfo,targetSystem);
    if (dynlen(exceptionInfo)) return;


/*
    int origRootNodeId, newRootNodeId;
    _fwConfigurationDB_getDBRootNodeId(hierarchyType, origRootNodeId, exceptionInfo, getSystemName());
    _fwConfigurationDB_getDBRootNodeId(hierarchyType, newRootNodeId, exceptionInfo, targetSystem);

*/

    // find out all root nodes in the hierarchy

    dyn_string rootNodesNames;
    dyn_int rootNodesIds;
    string getRootNodesSql= "SELECT name,item_id FROM v_items "+
		    	    " WHERE parent_id IS NULL "+
		    	    " AND hver="+g_fwConfigurationDB_DBHierarchyIDs[hierarchyType];
    dyn_dyn_mixed aRecords;
    _fwConfigurationDB_executeDBQuery(getRootNodesSql,g_fwConfigurationDB_DBConnection, aRecords, exceptionInfo,2,TRUE);
    if (dynlen(exceptionInfo)) return;

    rootNodesNames=aRecords[1];
    rootNodesIds=aRecords[2];

    // then find out the id of the new root node...
    int newRootIdx=dynContains(rootNodesNames,targetSystem);
    int newRootNodeId=rootNodesIds[newRootIdx];

    // make sure that the devices in the target system do not exist yet...
    dyn_dyn_mixed newDeviceListObject;

    // we use the public one, which commits the connection
    fwConfigurationDB_getDeviceConfigurationFromDB(configurationName,
	hierarchyType, newDeviceListObject, exceptionInfo, validOn, newDeviceList);


    if (dynlen(newDeviceListObject[fwConfigurationDB_DLO_DPNAME])) {
	fwException_raise(exceptionInfo,"ERROR in fwConfigurationDB_reconnectDevices",
	    "Some of reconnected devices already exist in the target system. See log for details","");
	DebugN("--- ERROR! Requested reconnect for device(s) already exists in DB:");
	DebugN("    in configuration:"+configurationName);
	DebugN(newDeviceListObject[fwConfigurationDB_DLO_DPNAME]);
	DebugN("-------------------------------------------------------------------");
	return ;
    }


    // store the raw hierarchy at first, including all the devices (also: parents)

    // find out what we already have in the hierarchy for target system...
    dyn_string missingDeviceList;
    dyn_dyn_mixed existingNewDeviceListObject;
    fwConfigurationDB_findDevicesInDB(targetSystem, hierarchyType,
                                       newAllDeviceList,
                                       existingNewDeviceListObject, missingDeviceList,
                                       exceptionInfo, targetSystem);

    if (dynlen(exceptionInfo)) return;

    int nItems=dynlen(missingDeviceList);

    dyn_int newItemIds;
    _fwConfigurationDB_getBatchOfSequenceNumbers(g_fwConfigurationDB_DBConnection, "ITEMS_ID_SEQ", nItems,
			    newItemIds, exceptionInfo);
    if (dynlen(exceptionInfo)) return;

    // copy allDeviceListObject->deviceListObject,
    // remove what we do not know...
    dynClear(newDeviceListObject);
    _fwConfigurationDB_ensureDeviceListObjectValid(newDeviceListObject);

//DebugN("newAllDeviceList",newAllDeviceList);
//DebugN("allDeviceList",allDeviceList);
    dyn_int tmpIdx;
    for (int i=1;i<=dynlen(missingDeviceList);i++) {
	newDeviceListObject[fwConfigurationDB_DLO_DPNAME][i]=missingDeviceList[i];
//DebugN("processing",missingDeviceList[i]);
	// find counter-part in the source system
	int idx=dynContains(newAllDeviceList, missingDeviceList[i]);
//DebugN("index  in newAllDeviceList is ",idx);
	string srcName=allDeviceList[idx];
//DebugN("SourceName is ",srcName);
	idx= dynContains(allDeviceListObject[fwConfigurationDB_DLO_DPNAME],srcName);
//DebugN("ultimately index is",idx);
	tmpIdx[i]=idx;
	newDeviceListObject[fwConfigurationDB_DLO_DPTYPE][i]=allDeviceListObject[fwConfigurationDB_DLO_DPTYPE][idx];
	newDeviceListObject[fwConfigurationDB_DLO_NAME][i]=allDeviceListObject[fwConfigurationDB_DLO_NAME][idx];
//	newDeviceListObject[fwConfigurationDB_DLO_TYPE][i]=allDeviceListObject[fwConfigurationDB_DLO_TYPE][idx];
	newDeviceListObject[fwConfigurationDB_DLO_MODEL][i]=allDeviceListObject[fwConfigurationDB_DLO_MODEL][idx];
	newDeviceListObject[fwConfigurationDB_DLO_COMMENT][i]=allDeviceListObject[fwConfigurationDB_DLO_COMMENT][idx];

	// assign new item id...
	newDeviceListObject[fwConfigurationDB_DLO_ITEMID][i]=newItemIds[i];
	newDeviceListObject[fwConfigurationDB_DLO_PARENTID][i]=-1; // this will be resolved below

        newDeviceListObject[fwConfigurationDB_DLO_DPID][i]="";
	newDeviceListObject[fwConfigurationDB_DLO_REFDP][i]="";
	newDeviceListObject[fwConfigurationDB_DLO_REF_STATUS][i]="";
	newDeviceListObject[fwConfigurationDB_DLO_PROPIDS][i]=makeDynInt();
	newDeviceListObject[fwConfigurationDB_DLO_PROPNAMES][i]=makeDynString();
	newDeviceListObject[fwConfigurationDB_DLO_CITEM_ID][i]=""; // we need to resolve it elsewhere

    }

    // resolve parents now...
    for (int i=1;i<=dynlen(missingDeviceList);i++) {
	int idx=tmpIdx[i]; // restore the pointer to original in allDeviceListObject
	int origParentId=allDeviceListObject[fwConfigurationDB_DLO_PARENTID][idx];


	//if (origParentId==origRootNodeId) {
	int origParentIdx=dynContains(rootNodesIds,origParentId);
	if (origParentIdx) {
	    // means: parent is a root note
	    newDeviceListObject[fwConfigurationDB_DLO_PARENTID][i]=newRootNodeId;
	} else {
	    // reconstruct the name of the parent ...
	    int idxPar=dynContains(allDeviceListObject[fwConfigurationDB_DLO_ITEMID],origParentId);
	    string origParentName=allDeviceListObject[fwConfigurationDB_DLO_DPNAME][idxPar];
	    int idx2=dynContains(allDeviceList, origParentName);
	    string newParentName=newAllDeviceList[idx2];
	    // try to find it in the existing ones;
	    idx2=dynContains(existingNewDeviceListObject[fwConfigurationDB_DLO_DPNAME],newParentName);
	    if (idx2>0) {
		newDeviceListObject[fwConfigurationDB_DLO_PARENTID][i]=existingNewDeviceListObject[fwConfigurationDB_DLO_ITEMID][idx2];
	    } else {

	    // try to find in the new ones...
	    idx2= dynContains(newDeviceListObject[fwConfigurationDB_DLO_DPNAME],newParentName);
	    if (idx2>0) {
//DebugN("OK. Found in new ones", idx2,newDeviceListObject[fwConfigurationDB_DLO_ITEMID][idx2]);
		newDeviceListObject[fwConfigurationDB_DLO_PARENTID][i]=newDeviceListObject[fwConfigurationDB_DLO_ITEMID][idx2];
	    } else {
//DebugN("not found...");
		fwException_raise(exceptionInfo,"ERROR","Could not resolve all relations","");
		return;
	    }

	    }

	}
    }


    // commit previous transaction if existing...
    rdbCommitTransaction(g_fwConfigurationDB_DBConnection);

    // save the hierarchy...
    int rc=rdbBeginTransaction(g_fwConfigurationDB_DBConnection);
    if (rc) {
	//bool invalidConnection;
        //_fwConfigurationDB_catchDbError(g_fwConfigurationDB_DBConnection, invalidConnection, exceptionInfo);
        fwException_raise(exceptionInfo,"ERROR","Cannot start new transaction","");
        if (dynlen(exceptionInfo)) return;
    };



if (dynlen(newDeviceListObject[1])) {
DebugN("saving new devices");
//DebugN("=-NEW-=",newDeviceListObject);
    _fwConfigurationDB_saveItemsToDB_OLD(newDeviceListObject, hierarchyType, exceptionInfo);
    if (dynlen(exceptionInfo))  {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

/*
DebugN("Updating the hierarchy...");
    // trigger the update of the materialized view:
    _fwConfigurationDB_updateHierarchyView(g_fwConfigurationDB_DBConnection, exceptionInfo);
    if (dynlen(exceptionInfo))  {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
*/
}

DebugN("moving the devices...");


    // prepare the "move" command
    anytype moveDevCmd;
    string moveDevSql="UPDATE config_items SET ITEM_ID = :NEWITEMID WHERE CITEM_ID = :CITEM_ID ";

    _fwConfigurationDB_startCommand(moveDevSql, g_fwConfigurationDB_DBConnection, moveDevCmd, exceptionInfo);
    if (dynlen(exceptionInfo))  {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

    dyn_string updatedCitemIds;

    for (int i=1;i<=dynlen(deviceListObject[fwConfigurationDB_DLO_DPNAME]);i++) {
	int citemId=deviceListObject[fwConfigurationDB_DLO_CITEM_ID][i];
	string oldItemName=deviceListObject[fwConfigurationDB_DLO_DPNAME][i];
	int newItemId=0;

	// find the corresponding new name
	int idx=dynContains(deviceList,oldItemName);
	string newItemName=newDeviceList[idx];

	// try find this item in the new devices...
	idx=dynContains(newDeviceListObject[fwConfigurationDB_DLO_DPNAME],newItemName);
	if (idx>0) {
	    newItemId=newDeviceListObject[fwConfigurationDB_DLO_ITEMID][idx];
	} else {
	    // try to find it in the ones that were already stored...
	    idx=dynContains(existingNewDeviceListObject[fwConfigurationDB_DLO_DPNAME],newItemName);
	    if (idx>0) newItemId=existingNewDeviceListObject[fwConfigurationDB_DLO_ITEMID][idx];

	}

	if (newItemId!=0) {
    	    mapping params;
    	    params[":NEWITEMID"] = newItemId;
	    params[":CITEM_ID"]=citemId;
	    dynAppend(updatedCitemIds,citemId);
    	    _fwConfigurationDB_bindExecuteCommand(moveDevCmd, params, exceptionInfo);
    	    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

	} else {
	    DebugN("unresolved![ProtectMe!]",i, citemId,oldItemName);
	}
    }

    _fwConfigurationDB_finishCommand(moveDevCmd, exceptionInfo);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}


    // update the data for consistency
    string citemIdsAsSql=_fwConfigurationDB_listToSQLString("citem_id", updatedCitemIds);

    DebugN("updating dpfunctions data");

//    string updateDPfSql="update config_dpfunctions set params=REPLACE(params,SUBSTR(params,1,INSTR(params,':',1,1)),\'"+targetSystem+"\') "+
//	       " where iprop_id in (select iprop_id from config_item_properties where "+citemIdsAsSql+")";
    string updateDPfSql="update config_dpfunctions set params=("+
			" case(INSTR(params,'|',1,1)) "+
			"	when 1 then REPLACE(params,SUBSTR(params,1,INSTR(params,':',1,1)),\'|"+targetSystem+"\')"+
			"	else        REPLACE(params,SUBSTR(params,1,INSTR(params,':',1,1)),\'"+targetSystem+"\')"+
			" END )"+
    	    		" where iprop_id in (select iprop_id from config_item_properties where "+citemIdsAsSql+")";


    fwConfigurationDB_executeSqlSimple(updateDPfSql, g_fwConfigurationDB_DBConnection,exceptionInfo);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

    DebugN("updating alerts classes data");

    //string updateACSql="update config_dpealerts set classes=REPLACE(classes,SUBSTR(classes,1,INSTR(classes,':',1,1)),\'"+targetSystem+"\') "+
    //       " where iprop_id in (select iprop_id from config_item_properties where "+citemIdsAsSql+")";


    string updateACSql="update config_dpealerts set classes=("+
			" case(INSTR(classes,'|',1,1)) "+
			"	when 1 then REPLACE(classes,SUBSTR(classes,1,INSTR(classes,':',1,1)),\'|"+targetSystem+"\')"+
			"	else        REPLACE(classes,SUBSTR(classes,1,INSTR(classes,':',1,1)),\'"+targetSystem+"\')"+
			" END ) "+
    	    		" where iprop_id in (select iprop_id from config_item_properties where "+citemIdsAsSql+")";


    fwConfigurationDB_executeSqlSimple(updateACSql, g_fwConfigurationDB_DBConnection,exceptionInfo);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

    DebugN("updating conversions data");

    //string updateConvSql="update config_conversions set arguments=REPLACE(arguments,SUBSTR(arguments,1,INSTR(arguments,':',1,1)),\'"+targetSystem+"\') "+
    //	" where iprop_id in (select iprop_id from config_item_properties where "+citemIdsAsSql+")";
    string updateConvSql="update config_conversions set arguments=("+
			" case(INSTR(arguments,'|',1,1)) "+
			"	when 1 then REPLACE(arguments,SUBSTR(arguments,1,INSTR(arguments,':',1,1)),\'|"+targetSystem+"\')"+
			"	else        REPLACE(arguments,SUBSTR(arguments,1,INSTR(arguments,':',1,1)),\'"+targetSystem+"\')"+
			" END )"+
    	    		" where iprop_id in (select iprop_id from config_item_properties where "+citemIdsAsSql+")";

    fwConfigurationDB_executeSqlSimple(updateConvSql, g_fwConfigurationDB_DBConnection,exceptionInfo);
    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}


    DebugN("updating aliases");
    // identify the aliases that we remap...

    string nodeListSql=_fwConfigurationDB_listToSQLString("refdpname", allDeviceList);

    string alsql="SELECT ITEM_ID,REF_ID, dpname, refdpname FROM v_REFERENCES WHERE ";
    alsql+=" config_id="+refVer;
    alsql+=" and "+nodeListSql;

    dyn_dyn_mixed aliasData;
    _fwConfigurationDB_executeDBQuery(alsql,g_fwConfigurationDB_DBConnection, aliasData, exceptionInfo,4,TRUE);
    if (dynlen(exceptionInfo)) return;

    time t0=getCurrentTime();
    if (dynlen(aliasData)<1) aliasData[1]=makeDynString(); // protect the next line...
    // order the list to be parallel to the allDeviceList, and hence to newAllDeviceList
    for (int i=1;i<=dynlen(aliasData[1]);i++) {
	string alias=aliasData[3][i];
	string oldDevice=aliasData[4][i];
	int oldDeviceId=aliasData[2][i];
	int idx=dynContains(allDeviceList,oldDevice);
	if (!idx) {
	    fwException_raise(exceptionInfo,"ERROR","Cannot find the device in the list","");
	    DebugN("Cannot find "+oldDevice+" in the list of devices being moved for alias "+alias);
	    return;
	}
	string newDevice=newAllDeviceList[idx];
	//DebugN(alias+" # "+oldDevice+" => "+newDevice);

	// find new device id...
	// at first in the newDeviceListObject...
	int newDeviceId=-1;
	int idxNewDev=dynContains(newDeviceListObject[fwConfigurationDB_DLO_DPNAME],newDevice);
	if (idxNewDev>=1) {
	    newDeviceId=newDeviceListObject[fwConfigurationDB_DLO_ITEMID][idxNewDev];
	} else {
	    // if not found, try the existingNewDeviceListObject;
	    idxNewDev=dynContains(existingNewDeviceListObject[fwConfigurationDB_DLO_DPNAME],newDevice);
	    if (idxNewDev>=1) newDeviceId=existingNewDeviceListObject[fwConfigurationDB_DLO_ITEMID][idxNewDev];

	}
	if (newDeviceId<0) {
	    fwException_raise(exceptionInfo,"ERROR","Cannot find the identifier of the new device","");
	    DebugN("Cannot find id for "+newDevice);
	    return;
	}

	//DebugN(oldDeviceId+" => "+newDeviceId);


	string invalidateAliasSql="update REFERENCES SET VALID_TO=TO_DATE(\'"+(string)t0+"\' , \'yyyy.mm.dd hh24:mi:ss.???\') "+
					"where VALID_TO IS NULL and refver="+refVer+
					" and ID="+aliasData[1][i];
	fwConfigurationDB_executeSqlSimple(invalidateAliasSql, g_fwConfigurationDB_DBConnection,exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

	string newAliasSql="insert into references(refver,id,ref_id,valid_from,valid_to) values ("+
		    refVer+" , "  +
		    aliasData[1][i]+" , " +
		    newDeviceId+" , "    +
		    " TO_DATE(\'"+(string)t0+"\' , \'yyyy.mm.dd hh24:mi:ss.???\'), "+
		    " NULL)";
	fwConfigurationDB_executeSqlSimple(newAliasSql, g_fwConfigurationDB_DBConnection,exceptionInfo);
	if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}

    }

    rc=rdbCommitTransaction(g_fwConfigurationDB_DBConnection);
    if (rc) {
	//bool invalidConnection;
        //_fwConfigurationDB_catchDbError(g_fwConfigurationDB_DBConnection, invalidConnection, exceptionInfo);
        fwException_raise(exceptionInfo,"ERROR","Cannot commit transaction","");
        return;
    };


DebugN("OK!");

}


/**
This one is used when storing the devices in DB, it assigns new
identifiers to the devices that are not present, then adjusts
parent pointers, etc.
It will also resolve the ID's of DPElements, if they are defined
*/
void _fwConfigurationDB_DBAssignItemIds(string hierarchyType, dyn_dyn_mixed &deviceListObject, dyn_string &exceptionInfo, string systemName="")
{

    _fwConfigurationDB_ensureDeviceListObjectValid(deviceListObject);
    if ( (systemName=="") && (hierarchyType==fwDevice_HARDWARE)) systemName=getSystemName();

    // find the root node for this system;
    int rootNode=0;
    _fwConfigurationDB_getDBRootNodeId(hierarchyType,rootNode, exceptionInfo, systemName);
    if (dynlen(exceptionInfo)) return;

    if (rootNode==0) {
	fwException_raise(exceptionInfo,"ERROR","Cannot find root node: "+systemName+" with hierarchy:"+hierarchyType,"");
	return;
    }

    // assign new id's to the devices.
    int nItems=dynlen(deviceListObject[fwConfigurationDB_DLO_DPNAME]);
    _fwConfigurationDB_getBatchOfSequenceNumbers(g_fwConfigurationDB_DBConnection, "ITEMS_ID_SEQ", nItems,
			    deviceListObject[fwConfigurationDB_DLO_ITEMID], exceptionInfo);
    if (dynlen(exceptionInfo)) return;



    //  fix the parent id's


    // we should firstly set the parents from the new ones being inserted,
    // then for remaining ones we should make a "guided" select with
    // dpnames specified. It will be much more performant than
    // "select *" - especially for the cases where a lot of devices
    // is already stored in DB...


    // sort-out the id's of the ones that were just given an id,
    // form a parallel list storing parent names...
    // and also a list suitable for inclusion in SQL Statement

    dyn_string parentDpNames;
    string sqlParentDpNames;
    for (int i=1;i<=dynlen(deviceListObject[fwConfigurationDB_DLO_DPNAME]);i++) {

        if(deviceListObject[fwConfigurationDB_DLO_MODEL][i]=="DATAPOINT"){
	    // this is non-framework device... we cannot rely on Framework hierarchies
	    // and we need to set its parent to rootNode
            deviceListObject[fwConfigurationDB_DLO_PARENTID][i]=rootNode;
	    continue;
	}

        string parentDpName;
        fwDevice_getParent(deviceListObject[fwConfigurationDB_DLO_DPNAME][i], parentDpName, exceptionInfo);
        if (dynlen(exceptionInfo)) return;

	if ( (parentDpName=="") || (parentDpName==systemName) ) {
            deviceListObject[fwConfigurationDB_DLO_PARENTID][i]=rootNode;
            continue;
        }


        // try with the newly created devices
        int idx=dynContains(deviceListObject[fwConfigurationDB_DLO_DPNAME],parentDpName);
        if (idx>=1) {
            deviceListObject[fwConfigurationDB_DLO_PARENTID][i] = deviceListObject[fwConfigurationDB_DLO_ITEMID][idx];
            continue;
        }

	// otherwise, we'll try to get them from the database.
	// in order to do that, we need to prepare a list
	// of devices to be queried...
	// we also mark that there is no id yet, by setting it to 0
	deviceListObject[fwConfigurationDB_DLO_PARENTID][i] = 0;

	if (!dynContains(parentDpNames, parentDpName)) {
	    sqlParentDpNames=sqlParentDpNames+"\'"+parentDpName+"\'"+",";
	}
	parentDpNames[i]=parentDpName;

    };

    // make a query only if there is anything to query...
    if (sqlParentDpNames!="") {
	// remove the trailing coma ...
	sqlParentDpNames=strrtrim(sqlParentDpNames,",");

	string queryParents="SELECT DPNAME,ID FROM V_ITEM_NAMES  "+
    	    " WHERE HVER="+g_fwConfigurationDB_DBHierarchyIDs[hierarchyType]+
	    " AND dpname IN ("+sqlParentDpNames+")";
	dyn_dyn_mixed aParents;
	_fwConfigurationDB_executeDBQuery(queryParents,g_fwConfigurationDB_DBConnection, aParents, exceptionInfo,2,TRUE);
	if (dynlen(exceptionInfo)) return;
	if (dynlen(aParents)<1) {
	    // ensure that the structure is ok...
	    aParents[1]=makeDynString();
	    aParents[2]=makeDynInt();
	}


	// complete the parent ids...
	for (int i=1;i<=dynlen(deviceListObject[fwConfigurationDB_DLO_DPNAME]);i++) {
	    if (deviceListObject[fwConfigurationDB_DLO_PARENTID][i]!=0) continue;
	    int idx=dynContains(aParents[1],parentDpNames[i]);
	    if (idx<1) {
		fwException_raise(exceptionInfo,"ERROR","Cannot resolve parent id:"+parentDpNames[i],"");
		return;
	    }
	    deviceListObject[fwConfigurationDB_DLO_PARENTID][i]=aParents[2][idx];
	}
    }

    if (dynlen(exceptionInfo)) return;

}

/** This one resolves the DB id's of items and
reports error if device was not found...

@param hierarchyType is dummy as of now!
*/
void _fwConfigurationDB_DBFindItems(string hierarchyType, dyn_string deviceList, dyn_int &itemIds,
                                    dyn_string &exceptionInfo)
{
    if (dynlen(deviceList)==0) return;

    time t0=getCurrentTime();
    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_ResolveHierarchy,"Resolving device hierarchy", 10.0,
                                   exceptionInfo)) return;

    dynClear (itemIds);
    dyn_string uDeviceList=deviceList;
    dynUnique(uDeviceList);
    if (dynlen(uDeviceList)<1) return;


    string sDeviceList=_fwConfigurationDB_listToSQLString("DPNAME", uDeviceList);

//    string queryDevicesSql="SELECT DPNAME, ID FROM V_ITEM_NAMES WHERE "+
//        " HVER="+g_fwConfigurationDB_DBHierarchyIDs[hierarchyType]+" AND "+sDeviceList;
    string queryDevicesSql="SELECT DPNAME, ID FROM V_ITEM_NAMES WHERE "+sDeviceList;

    dyn_dyn_mixed aRecords;
    _fwConfigurationDB_executeDBQuery(queryDevicesSql,g_fwConfigurationDB_DBConnection, aRecords, exceptionInfo,2,TRUE);
    if (dynlen(exceptionInfo)) return;

    dyn_string dbDevices;
    dyn_int dbIds;

    if (dynlen(aRecords)>=2) {
	dbDevices=aRecords[1];
	dbIds=aRecords[2];
    }

    // and now resolve the names
    for (int i=1;i<=dynlen(deviceList);i++) {
        int idx=dynContains(dbDevices,deviceList[i]);
        if (idx>0) {
            itemIds[i]=dbIds[idx];
        } else {
    	    fwException_raise(exceptionInfo,"ERROR","Device not found in DB:\n"+deviceList[i],"");
        }
    };
    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_ResolveHierarchy,"Resolved device hierarchy", 100.0,
                                   exceptionInfo)) return;

    time t1=getCurrentTime();
    time dt=t1-t0;
//DebugN("Resolving IDs of "+dynlen(uDeviceList)+" items took "+minute(dt)+":"+second(dt)+":"+milliSecond(dt));
}



void _fwConfigurationDB_getDBRootNodeId(string hierarchyType, int &rootNode,
                                        dyn_string &exceptionInfo, string systemName="")
{
    if (!mappingHasKey(g_fwConfigurationDB_DBHierarchyIDs,hierarchyType)) {
	fwException_raise(exceptionInfo,"ERROR","Hierarchy type: "+hierarchyType+" not supported in this ConfigurationDB","");
    	return;
    }

    rootNode=0;
    string queryRootNodeSql;

if ((hierarchyType==fwDevice_LOGICAL) && systemName=="") {
    // NOTE! in a LOGICAL hierarchy with common root, the DPNAME for the root is "",
    // but the NAME(!) of the root is "/"!
    // because we do this query on ITEMS table rather than on V_ITEM_NAMES view,
    // we need to use name, and hence have specific query!
	queryRootNodeSql="SELECT ID FROM ITEMS WHERE NAME=\'/\'"+
    	    " AND HVER="+g_fwConfigurationDB_DBHierarchyIDs[hierarchyType];
    } else {
	if (systemName=="") systemName=getSystemName();

	queryRootNodeSql="SELECT ID FROM ITEMS WHERE NAME=\'"+systemName+"\'"+
    	    " AND HVER="+g_fwConfigurationDB_DBHierarchyIDs[hierarchyType];
    }
    dyn_dyn_mixed aRootNode;
    _fwConfigurationDB_executeDBQuery(queryRootNodeSql,g_fwConfigurationDB_DBConnection, aRootNode, exceptionInfo);
    if (dynlen(exceptionInfo)) return;

    if (dynlen(aRootNode)==0) {
        fwException_raise(exceptionInfo,"ERROR",
                          "Inconsistency: no root node in DB for system "+systemName,"");
        return;
    } else  if (dynlen(aRootNode)>1){
        fwException_raise(exceptionInfo,"ERROR",
                          "Inconsistency: multiple root nodes in DB for system "+systemName,"");
        return;
    } else {
        rootNode=aRootNode[1][1];
    };
}




/** Creates missing devices in PVSS, so the structure reflects the one in the DB

note that it should work only for the local system

the systemName parameter is only meaning for logical hierarchy:
it indicates which type of logical hierarchy should be loaded: the one that includes system name,
or the one that does not (empty systemName). In the latter case, only the leaves that are currently
mapped to devices that are present on the current system are loaded, plus all the "branch" nodes that
lead to them...

@Deprecated use @ref fwConfigurationDB_updateDeviceConfigurationFromDB instead
*/
void fwConfigurationDB_updateDeviceHierarchyFromDB(string topDevice, string hierarchyType, dyn_string &exceptionInfo, string systemName="")
{
    fwException_raise(exceptionInfo, "ERROR","fwConfigurationDB_updateDeviceHierarchyFromDB is deprecated","");
    return;
    /*
    time t0;
    _fwConfigurationDB_startFunction("updateDeviceHierarchyFromDB", t0);

    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_ApplyHierarchyToSystem,"Applying hierarchy in system", 10.0, exceptionInfo)) return;

    dyn_string dbDeviceDpNames,dbDeviceTypes,dbDeviceDetails,dbDeviceComments;
    dyn_int dbDeviceIds;

    if ( (hierarchyType!=fwDevice_HARDWARE) && (hierarchyType!=fwDevice_LOGICAL)) {
	fwException_raise(exceptionInfo,"ERROR","Hierarchy type: "+hierarchyType+" not supported.","");
        return;
    };

    // find out the system name of the top device,
    // and check if it is local system...
    string sysName=getSystemName();
    if (topDevice=="") {
	if (hierarchyType==fwDevice_HARDWARE) {
    	    topDevice=sysName;
	} // otherwise: we may have a LOGICAL hierarchy with common root...
    }

    dyn_dyn_mixed deviceListObject, missingDeviceListObject;
    _fwConfigurationDB_ensureDeviceListObjectValid(deviceListObject);
    _fwConfigurationDB_ensureDeviceListObjectValid(missingDeviceListObject);
    // find the devices in the database:
    fwConfigurationDB_extractHierarchyFromDB(topDevice, hierarchyType,
                                             deviceListObject,
                                             exceptionInfo);
    if (dynlen(exceptionInfo)) return;

    _fwConfigurationDB_verifyDatapoints(hierarchyType,
                                      deviceListObject, missingDeviceListObject,
                                      exceptionInfo,TRUE);

    if (dynlen(exceptionInfo)) return;

    _fwConfigurationDB_createDevices(missingDeviceListObject,exceptionInfo);
    if (dynlen(exceptionInfo)) return;
    _fwConfigurationDB_(deviceListObject,exceptionInfo);
    if (dynlen(exceptionInfo)) return;

    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_ApplyHierarchyToSystem,"Applied hierarchy in system", 100.0, exceptionInfo)) return;

    _fwConfigurationDB_endFunction("updateDeviceHierarchyFromDB", t0);
*/
}


/** Finds the devices that exist and devices that do not exist,
 * returns them as deviceListObject variables, ready for further
 * processing.
 *
 * WARNING! Empty system name here means: don't filter!
 *
*/
void fwConfigurationDB_findDevicesInDB(string topDevice, string hierarchyType,
                                       dyn_string deviceList,
                                       dyn_dyn_mixed &deviceListObject, dyn_string &missingDevicesList,
                                       dyn_string &exceptionInfo, string system="")
{
    time t0;
    _fwConfigurationDB_startFunction("findDevicesInDB", t0);
    //if ((hierarchyType==fwDevice_HARDWARE) && (system=="")) system=getSystemName();
    dynClear(deviceListObject);
    _fwConfigurationDB_ensureDeviceListObjectValid(deviceListObject);

    dyn_dyn_mixed dbDeviceListObject;
    fwConfigurationDB_extractHierarchyFromDB(topDevice, hierarchyType,
                                             dbDeviceListObject,
                                             exceptionInfo, system);
    if (dynlen(exceptionInfo)) return;

    for (int i=1;i<=dynlen(deviceList);i++) {

        int idx=dynContains(dbDeviceListObject[fwConfigurationDB_DLO_DPNAME],deviceList[i]);
        if (idx>=1){
            _fwConfigurationDB_copyDeviceListObjectEntry(dbDeviceListObject, idx,
                deviceListObject,0,exceptionInfo);
            if (dynlen(exceptionInfo)) return;
            continue; // ok, already in DB; move on to the next device in the list
        }
        // otherwise - device is missing:
        dynAppend(missingDevicesList,deviceList[i]);
    }


    _fwConfigurationDB_endFunction("findDevicesInDB", t0);
}



/**  adds the required (parent) devices to the list, based on the data in the database,
     puts the hierarchy in order.
     this function is used by @ref fwConfigurationDB_reconnectDevices

*/
void _fwConfigurationDB_dbCompleteDevicesInHierarchy(dyn_string &deviceList, string hierarchyType,
    string configurationName, dyn_string &exceptionInfo)
{

    string devListSql;

    // change my mind! Must not specify empty deviceList!
    if (dynlen(deviceList)==0) {
	fwException_raise(exceptionInfo,"ERROR in dbCompleteDevicesInHierarchy",
			    "Empty device list specified","");
	return;
    }


//    if (dynlen(deviceList)!=0) {
	devListSql=_fwConfigurationDB_listToSQLString("DPNAME", deviceList);
//    }

/*
    string sql= "SELECT UNIQUE citem_id, item_id, dpname from v_config_items "+
	        " WHERE tag=\'"+configurationName+"\' "+
	        " AND HVER="+g_fwConfigurationDB_DBHierarchyIDs[hierarchyType]+
	        " CONNECT BY PRIOR parent_id=item_id "+
	        " START WITH "+devListSql+
	        "  ORDER BY dpname" ;
*/

    string sql= "select unique dpname, item_id, parent_id from v_items "+
		" connect by prior parent_id=item_id start with "+
	        " item_id in ( "+
		"	select ci.item_id from v_config_items ci where "+devListSql+
			"and hver="+g_fwConfigurationDB_DBHierarchyIDs[hierarchyType]+
			"and tag=\'"+configurationName+"\')"+
		"order by dpname";



    dyn_dyn_mixed aRecords;
    _fwConfigurationDB_executeDBQuery(sql,g_fwConfigurationDB_DBConnection, aRecords, exceptionInfo,3,TRUE);
    if (dynlen(exceptionInfo)) return;
//DebugN("Complete devs returns",aRecords);
    if (dynlen(deviceList)) {
	// check if all devices were found...
	for (int i=1;i<=dynlen(deviceList);i++) {
	    //if (!dynContains(aRecords[3],deviceList[i])) {
	    if (!dynContains(aRecords[1],deviceList[i])) {
		fwException_raise(exceptionInfo,"ERROR in dbCompleteDevicesInHierarchy",
		"Device was not found in DB: "+deviceList[i],"");
		return;
	    }
	}
    }

    // otherwise: it's OK; substitute completed list
    //deviceList=aRecords[3];
    deviceList=aRecords[1];
}

/** Resolves device names
 *
 * This functions takes the list of device names - devices being
 * in either hierarchy, and tells what are the actual hierarchy
 * to which they belong, and what are the datapoint to which they
 * refer
 *
 */
void fwConfigurationDB_resolveDevices(dyn_string deviceList,
    dyn_string &devHierarchies, dyn_string &devDatapoints,
    dyn_string &exceptionInfo , bool errorOnNotFound=TRUE, dyn_string deviceSystems="")
{
    int systems;
    dyn_string allDps;
    dyn_string allAliases;
    
    systems = dynlen(deviceSystems);
//DebugN("RESOLVING:", deviceSystems);
    if(systems > 0)
    {
      dyn_string tempDps, tempAliases;
      for(int i=1; i<=systems; i++)
      {
        dpGetAllAliases (tempDps, tempAliases, "*", deviceSystems[i] + "*.");
        dynAppend(allDps, tempDps);
        dynAppend(allAliases, tempAliases);
      }
    }
    else
      dpGetAllAliases (allDps, allAliases, "*", "*.");

    for (int i=1;i<=dynlen(deviceList);i++) {
	string deviceName=deviceList[i];
	if (strpos(deviceName,":")>0) {
	    // hardware hierarchy
	    if ( (!dpExists(deviceName)) && errorOnNotFound) {
		fwException_raise(exceptionInfo,"ERROR","Device (HARDWARE) not found:\n"+deviceName,"");
		return;
	    }
	    devHierarchies[i]=fwDevice_HARDWARE;
	    devDatapoints[i]=dpSubStr(deviceName,DPSUB_SYS_DP);
	} else {
	    // it is an alias
	    string alias=deviceList[i];
	    int idx=dynContains(allAliases,alias);
	    if ((idx<1) && errorOnNotFound) {
		fwException_raise(exceptionInfo,"ERROR","Device (LOGICAL) not found:\n"+alias,"");
		return;
	    }
	    devHierarchies[i]=fwDevice_LOGICAL;
	    devDatapoints[i]=dpSubStr(allDps[idx],DPSUB_SYS_DP);
	} // some day: else FSM ...

    }

}

/**
Completes the information for the logical hierarchy

requires fields: fwConfigurationDB_DLO_DPNAME, fwConfigurationDB_DLO_DPTYPE
fills-in fields: fwConfigurationDB_DLO_REFDP,  fwConfigurationDB_DLO_REFID

skips the fields that has type == "Node"

*/
void _fwConfigurationDB_getReferencesFromDB( dyn_dyn_mixed &deviceListObject, string configurationName, dyn_string &exceptionInfo)
{

    dyn_string devList, refList;

    string sql="SELECT item_ID, REF_ID , refdpname FROM V_REFERENCES "+
		"WHERE CONFIG_TAG=\'"+configurationName+"\'";


    dyn_dyn_mixed aRecords;
    _fwConfigurationDB_executeDBQuery(sql,g_fwConfigurationDB_DBConnection, aRecords, exceptionInfo,3,TRUE);
    if (dynlen(exceptionInfo)) return;


    // protect from errors when empty recordset returned...
    if (dynlen(aRecords)==0) {
	dyn_mixed empty;
	aRecords[1]=empty;
	aRecords[2]=empty; // fix #55830
    };

    dyn_int dbItemIds=aRecords[1];
    dyn_int dbRefIds=aRecords[2];

    for (int i=1;i<=dynlen(deviceListObject[fwConfigurationDB_DLO_DPNAME]);i++) {
	if (deviceListObject[fwConfigurationDB_DLO_DPTYPE][i]=="FwNode") {
	    deviceListObject[fwConfigurationDB_DLO_REFDP][i]="";//deviceListObject[fwConfigurationDB_DLO_DPNAME][i];
	    deviceListObject[fwConfigurationDB_DLO_REFID][i]=0;
	    continue;
	}
	int idx=dynContains(dbItemIds,deviceListObject[fwConfigurationDB_DLO_ITEMID][i]);
	if (idx >0) {
	    deviceListObject[fwConfigurationDB_DLO_REFDP][i]=aRecords[3][idx];
	    deviceListObject[fwConfigurationDB_DLO_REFID][i]=dbRefIds[idx];
	} else {
	    deviceListObject[fwConfigurationDB_DLO_REFDP][i]="";
	    deviceListObject[fwConfigurationDB_DLO_REFID][i]=0;
	}
    }

}



void fwConfigurationDB_extractHierarchyFromDB(string topDevice, string hierarchyType,
                                              dyn_dyn_mixed &deviceListObject,
                                              dyn_string &exceptionInfo, string system="")
{
    time t0;
    _fwConfigurationDB_startFunction("extractHierarchyFromDB", t0);

    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_LoadHierachyFromDB,"Loading hierarchy info from DB", 10.0, exceptionInfo)) return;

    dynClear (deviceListObject);
    _fwConfigurationDB_ensureDeviceListObjectValid(deviceListObject);

    //if ((hierarchyType==fwDevice_HARDWARE) && (system=="")) system=getSystemName();

    string sqlTopDevice="rd.DPNAME=\'"+topDevice+"\'";
    bool hasRootDevice=TRUE;

    if (topDevice=="") {
	if (hierarchyType==fwDevice_LOGICAL) {
	    sqlTopDevice="rd.DPNAME IS NULL";
	    hasRootDevice=TRUE;
	} else {
	    if (system!="") {
		sqlTopDevice="rd.DPNAME=\'"+system+"\'";
		hasRootDevice=TRUE;
	    } else {
		hasRootDevice=FALSE;
		DebugN("WARNING! Querying the whole hierarchy in extractHierarchyFromDB. This may take a lot of time!");
	    }
	}
    }
    string queryRootDevice="SELECT rd.ID FROM V_ITEM_NAMES rd WHERE "+
        sqlTopDevice+" AND "+
        "rd.HVER="+g_fwConfigurationDB_DBHierarchyIDs[hierarchyType];

    string queryDeviceHierarchy="SELECT H.ITEM_ID,H.PARENT_ID,H.NAME,H.DPNAME,"+
        "H.DPTYPE,H.DETAIL,H.DESCRIPTION,h.dpid "+
        "FROM V_ITEMS H ";

    if (hasRootDevice) queryDeviceHierarchy=queryDeviceHierarchy+" START WITH H.ITEM_ID=("+queryRootDevice+") ";

    queryDeviceHierarchy=queryDeviceHierarchy+" CONNECT BY H.PARENT_ID = PRIOR H.ITEM_ID";

    queryDeviceHierarchy=queryDeviceHierarchy+" ORDER BY H.ITEM_ID";

    string sql = queryDeviceHierarchy;
    dyn_dyn_mixed aRecords;
    _fwConfigurationDB_executeDBQuery(sql,g_fwConfigurationDB_DBConnection, aRecords, exceptionInfo);
    if (dynlen(exceptionInfo)) return;
    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_LoadHierachyFromDB,"Processing the data", 70.0, exceptionInfo)) return;

    for (int i=1;i<=dynlen(aRecords);i++) {

        dynAppend(deviceListObject[fwConfigurationDB_DLO_ITEMID],(int)aRecords[i][1]);
        dynAppend(deviceListObject[fwConfigurationDB_DLO_PARENTID],(int)aRecords[i][2]);
        dynAppend(deviceListObject[fwConfigurationDB_DLO_NAME],aRecords[i][3]);
        dynAppend(deviceListObject[fwConfigurationDB_DLO_DPNAME],aRecords[i][4]);
        dynAppend(deviceListObject[fwConfigurationDB_DLO_DPTYPE],aRecords[i][5]);
        dynAppend(deviceListObject[fwConfigurationDB_DLO_MODEL],aRecords[i][6]);
        dynAppend(deviceListObject[fwConfigurationDB_DLO_COMMENT],aRecords[i][7]);
        dynAppend(deviceListObject[fwConfigurationDB_DLO_DPID],(int)aRecords[i][8]);

	// fill the remaining space
	mixed dummy;
	for (int idx=fwConfigurationDB_DLO_REFDP;idx<=fwConfigurationDB_DLO_MAX_IDX;idx++) dynAppend(deviceListObject[idx],dummy);
    };

    if (dynlen(exceptionInfo)) return;


    if (fwConfigurationDB_progress(fwConfigurationDB_OPER_LoadHierachyFromDB,"Loading hierarchy info from DB", 100.0, exceptionInfo)) return;

    _fwConfigurationDB_endFunction("extractHierarchyFromDB", t0);
}



/** Stores "minimal" information about the devices to DB

NOTE: USES DEPRECATED "saveDeviceHierarchyToDB"

  This function stores minimal information about devices into the DB,
  so that recipes for these devices may also be stored.

  @param[in,out] deviceList - list of names of devices; may include a mixture of devices
              from the HARDWARE and LOGICAL hierarchies; on exit, it will contain the
              list of devices that were actually stored/verified, namely: the non-existing
              devices are removed from the list, and devices that are needed to satisfy
              the hierarchical constraints (parents, grandparents,...) are added to the list
  @param[out] exceptionInfo - standard exception-handling parameter
  @param[in]  abortWhenDeviceMissing (optional, default FALSE) - if set to TRUE, the
              function will abort (and return an error) if some of specified devices
              does not exist, i.e. it will not store the good ones in the database; otherwise
              (the default behaviour), the missing devices will be skipped, the good ones
              will be stored in the database, and the return value of 1 will signify that
              there were missing devices.

  @retval 0 - everything OK: all devices are in DB now
  @retval 1 - some devices in the deviceList do not exist, and were skipped; the list
              of them is in the exceptionInfo; apart from that, everything was OK
  @retval -1 - there was an error while storing the devices; details in the exceptionInfo


  How to handle the return value and the exceptionInfo, when
  the @c abortWhenDeviceMissing parameter is @c FALSE.
  This example shows how to make sure that the recipeObject can be
  saved to DB. The devices that exist - are saved. For all devices
  that are missing, we post-process the recipeObject to remove the data
  for non-existing devices, then save the recipeObject to DB
  @code
    dyn_dyn_mixed recipeObject;
    // ... get the recipe into the recipeObject somehow...

    dyn_string deviceList=recipeObject[fwConfigurationDB_RO_DP_NAME];

    int rc=fwConfigurationDB_saveDevicesToDBMinimal(deviceList,exceptionInfo,FALSE);
    if (rc!=0) {
      // means something happen
      if (rc==-1) {
        fwExceptionHandling_display(exceptionInfo);
        // abort!
        return FALSE;
      } else if (rc==1) {
        // it is still OK, even though some of the devices were missing:
        // we choose not to display the exception, but only print a message to a log
        DebugN("There were some missing devices - the others were saved OK");
        // post-process the recipeObject, to remove the missing devices...
        // the good ones are returned in the (modified!) deviceList
        for (int i=1;i<=dynlen(recipeObject[fwConfigurationDB_RO_DP_NAME]);i++) {
          string dev=recipeObject[fwConfigurationDB_RO_DP_NAME][i];
          if (!dynContains(deviceList,dev)) {
            // remove the line from all data columns of the recipeObject
            for (int j=1;j<=fwConfigurationDB_RO_MAXIDX;j++) {
              dynRemove(recipeObject[j],i);
             }
             i--;//reset iterator
           }

         }
      }
    }

    // we could save the recipe into the DB now...
    fwConfigurationDB_saveRecipeToDB(recipeObject,"","MyTestRecipe",exceptionInfo);
    if (dynlen(exceptionInfo)){fwExceptionHandling_display(exceptionInfo);return;};
  @endcode

  */
int fwConfigurationDB_saveDevicesToDBMinimal(dyn_string &deviceList,
                                             dyn_string &exceptionInfo,
                                             bool abortWhenDeviceMissing=FALSE)
{
DebugTN("NOTE: _saveDevicesToDBMinimal USES DEPRECATED saveDeviceHierarchyToDB");
  fwConfigurationDB_checkInit(exceptionInfo);
  if (dynlen(exceptionInfo)) return;

  dynUnique(deviceList);

  dyn_string hwDevList,logDevList,devTypes;
  dyn_string dpTs;
  dyn_string mdExcInfo; // exceptionInfo for missing devices...

  // separate HW from LOGICAL...
    for (int i=1;i<=dynlen(deviceList);i++) {
      if (strpos(deviceList[i],":")<0) {
       if (dpAliasToName(deviceList[i])=="") {
         fwException_raise(mdExcInfo,"INFO",
                            "fwConfigurationDB_saveDevicesToDBMinimal: skipping non-existing alias:"+deviceList[i],
                            "");
         if (abortWhenDeviceMissing) {exceptionInfo=mdExcInfo;return 1;};
         continue;
       }
       dynAppend(logDevList,deviceList[i]);
      } else {
        if (!dpExists(deviceList[i])) {
          fwException_raise(mdExcInfo,"INFO",
                            "fwConfigurationDB_saveDevicesToDBMinimal: skipping non-existing dp:"+deviceList[i],
                            "");
          if (abortWhenDeviceMissing) {exceptionInfo=mdExcInfo;return 1;};
          continue;
        }
       dynAppend(hwDevList,deviceList[i]);
      }
    }

    // complete the logical hierarchy...
    _fwConfigurationDB_completeDevicesInHierarchy(logDevList, fwDevice_LOGICAL,"",exceptionInfo);
    if (dynlen(exceptionInfo)) return -1;

    // resolve aliases, and append the aliased dps to hardware list
    for (int i=1;i<=dynlen(logDevList);i++) {
       string dpname=dpSubStr(dpAliasToName(logDevList[i]),DPSUB_SYS_DP);
       if (dpTypeName(dpname)!="FwNode") dynAppend(hwDevList,dpname);
    }

    //complete the hardware hierarchy
    _fwConfigurationDB_completeDevicesInHierarchy(hwDevList, fwDevice_HARDWARE,getSystemName(),exceptionInfo);
    if (dynlen(exceptionInfo)) return -1;

    // now extract all dpTypes that we need
    // it is sufficient to do it for HW only (!)
    for (int i=1;i<=dynlen(hwDevList);i++) {
      string dpt=dpTypeName(hwDevList[i]);
      dynAppend(dpTs,dpt);
    }
    dynUnique(dpTs);

    //DebugN("Saving DPTs",dpTs);
    dyn_int dpTypeIds;
    dyn_mixed dpTypeElements, dpTypeElementIds;
    _fwConfigurationDB_checkSaveDpTypes(dpTs,dpTypeIds,dpTypeElements,dpTypeElementIds, getCurrentTime(), exceptionInfo);
    if (dynlen(exceptionInfo)) return -1;

    dyn_dyn_mixed deviceListObject;
    //DebugN("saving hw devs:",hwDevList);
    _fwConfigurationDB_saveDeviceHierarchyToDB("", fwDevice_HARDWARE,
                                             hwDevList,exceptionInfo,
                                             "",deviceListObject);
    if (dynlen(exceptionInfo)) return -1;
    //DebugN("Saving logical devs:",logDevList);
    _fwConfigurationDB_saveDeviceHierarchyToDB("", fwDevice_LOGICAL,
                                             logDevList,exceptionInfo,
                                             "",deviceListObject);
   if (dynlen(exceptionInfo)) return -1;
   //DebugN("Devices saved OK.");

   // on return, we want to give back the modified deviceList...
   dyn_string retDeviceList=hwDevList;
   dynAppend(retDeviceList,logDevList);
   deviceList=retDeviceList;

   // "rethrow" all the info about missing devices
   exceptionInfo=mdExcInfo;
   if (dynlen(exceptionInfo)) return 1;

   return 0;
}


void _fwConfigurationDB_startCommand(string sql, dbConnection &connection, anytype &dbCmd, dyn_string &exceptionInfo)
{
    // once we will get variable binding, we will do dbStartCommand (connection, cmdStr, command);
    // and return the actual dbCommand handle in "cmd". For now, we leave it dummy,
    // because we need to prepare all the statements individually anyway...

    // as of now we will store the SQL string and some information about the place-holders

    dyn_mixed  cmdStruct;
    dbCommand  command;
    dyn_string paramsNames;

    // find the positional parameters for binding
    dyn_string tmp=strsplit(sql,":");
    // the first one in the command, like "INSERT", so we start with number 2...
    for (int i=2;i<=dynlen(tmp);i++) {
        //use the characters only up to the next space, comma, brace
        string token=":"; // start the name with ":" which was removed by strsplit
        for (int k=0;k<strlen(tmp[i]);k++) {
            bool endOfToken=FALSE; // we will not be able to use "break" because we use switch()...
            switch ((tmp[i])[k]) {
                case ' ':
                case '\'':
                case ',':
                case '(':
                case ')':
                case '=':
                case '<':
                case '>':
                case '!':
                    endOfToken=TRUE;
                    break;
                default:
                    token=token+(tmp[i])[k];
                    break;
            };
            if (endOfToken || (k==strlen(tmp[i]))) {
                dynAppend(paramsNames,token);
                break; // now we can use it
            };
        };
    }
    cmdStruct[1]=sql;        // the SQL string
    cmdStruct[2]=connection; // the dbConnection
    cmdStruct[3]=paramsNames;// names of the positional parameters
    dbCmd=cmdStruct;


}

void _fwConfigurationDB_finishCommand(anytype &dbCmd, dyn_string &exceptionInfo)
{
    // dummy in the implementation with no bound variables ...
    //dynClear(dbCmd);
}

void _fwConfigurationDB_bindExecuteCommand(anytype &dbCmd, mapping params, dyn_string &exceptionInfo)
{
    if (getType(dbCmd)!=DYN_MIXED_VAR) {
        fwException_raise(exceptionInfo,"ERROR","Invalid DB Command passed for execution: bad type "+getType(dbCmd),"");
        return;
    }
    if (dynlen(dbCmd)!=3) {
        fwException_raise(exceptionInfo,"ERROR","Invalid DB Command passed for execution","");
        return;
    }

    dyn_string ReqParams=dbCmd[3];
    dyn_string availParams=mappingKeys(params);
    string sql=dbCmd[1];
    for (int i=1;i<=dynlen(ReqParams);i++) {
        if (!dynContains(availParams,ReqParams[i])) {
            fwException_raise(exceptionInfo,"ERROR","Parameter "+ReqParams[i]+" not bound.","");
            return;
        }
        strreplace(sql,ReqParams[i],params[ReqParams[i]]);

    };
    fwConfigurationDB_executeSqlSimple(sql, dbCmd[2], exceptionInfo);
    if (dynlen(exceptionInfo)) {
	dynClear(exceptionInfo);
	string errTxt;
	fwDbCheckError(errTxt,dbCmd[2]);
	DebugN("Cannot execute SQL command:\n",sql,"\n"+errTxt);
        fwException_raise(exceptionInfo,"ERROR in _fwConfigurationDB_bindExecuteCommand","Cannot execute command (see log)","");
        return;
    }
}

void fwConfigurationDB_updateDeviceConfigurationFromDB_OLD(string configurationName,
    string hierarchyType, dyn_string &exceptionInfo,  time validOn=0, dyn_string deviceList="", string systemName="",
    int options=fwConfigurationDB_deviceConfig_ALLDEVPROPS)
{
DebugTN("fwConfigurationDB_updateDeviceConfigurationFromDB_OLD");
    /*
    if (configurationName=="") {
	// fall-back to previous method - get device hierarchy, etc
        DebugN("updating whole hierarchy using fwConfigurationDB_updateDeviceHierarchyFromDB");
        string topDevice="";
        fwConfigurationDB_updateDeviceHierarchyFromDB(topDevice, hierarchyType, exceptionInfo,systemName);
        if (dynlen(exceptionInfo)) return;
        return;
    }
    */


//DebugN("loading device configuration:"+configurationName, systemName);

    dyn_dyn_mixed deviceListObject, missingDeviceListObject;
DebugTN("We call _fwConfigurationDB_getDeviceConfigurationFromDB");
    // we call the private one - the one that do not commit transaction!
    _fwConfigurationDB_getDeviceConfigurationFromDB(configurationName, hierarchyType, deviceListObject,
        exceptionInfo, validOn, deviceList,systemName);

    if (dynlen(exceptionInfo)) {
        rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);
	return;
    }

    // unfold the list of device properties:
    dyn_string dpes;
    dyn_int propIds;
    for (int i=1;i<=dynlen(deviceListObject[fwConfigurationDB_DLO_DPNAME]);i++) {
        dyn_string elements=deviceListObject[fwConfigurationDB_DLO_PROPNAMES][i];
        dyn_int ids=deviceListObject[fwConfigurationDB_DLO_PROPIDS][i];
        for (int j=1;j<=dynlen(ids);j++) {
            dynAppend(dpes,deviceListObject[fwConfigurationDB_DLO_DPNAME][i]+elements[j]);
            dynAppend(propIds,ids[j]);
        }
    }

    // create devices, etc
    _fwConfigurationDB_verifyDatapoints(hierarchyType,
                                        deviceListObject, missingDeviceListObject,
                                        exceptionInfo,FALSE);
                                        //exceptionInfo,TRUE);

    if (dynlen(exceptionInfo)) {
        rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);
	return;
    }


if (options & fwConfigurationDB_deviceConfig_NO_DEVICE_CREATE) {
    DebugTN("CONFIGURE WITHOUT RE-CREATING DEVICES: Filtering the list of properties");
    // simply remove the data for the missing devices (already listed 
    // in missingDeviceListObject) from the deviceListObject;

//    DebugTN("Missing devices",missingDeviceListObject[fwConfigurationDB_DLO_DPNAME]);
//    DebugTN("Full list",deviceListObject[fwConfigurationDB_DLO_DPNAME]);
//    DebugTN("Before modifying deviceListObject is",deviceListObject);
    dyn_string missingDevNames=missingDeviceListObject[fwConfigurationDB_DLO_DPNAME];
    dyn_string allDevNames=deviceListObject[fwConfigurationDB_DLO_DPNAME];
    for (int i=1;i<=dynlen(missingDevNames);i++) {
	int idx=dynContains(allDevNames,missingDevNames[i]);
	if (idx) {
	    // we need to remove it...
	    dynRemove(allDevNames,idx);
	    dynRemove(missingDevNames,i);
	    
	    // (!) remove the selected ipropids from the temp table(!)
	    string sqlForTempTable="DELETE FROM TMP_ITEM_PROPS WHERE "+_fwConfigurationDB_listToSQLString("iprop_id", deviceListObject[fwConfigurationDB_DLO_PROPIDS][idx]);
//DebugTN("We are going to SQL",sqlForTempTable);
	    fwConfigurationDB_executeSqlSimple(sqlForTempTable, g_fwConfigurationDB_DBConnection, exceptionInfo);                                                                              
	    if (dynlen(exceptionInfo)) {rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);return;}
	    
	    // now remove also from the deviceListObject, yet from all the columns!
	    for (int k=1;k<=fwConfigurationDB_DLO_MAX_IDX;k++) {
		dynRemove(deviceListObject[k],idx);
	    }
	// mitigate the iterator, because we removed the item:
	i--;
	}
    }
//    DebugTN("After modifying deviceListObject is",deviceListObject);
    DebugTN("CONFIGURE WITHOUT RE-CREATING DEVICES: Filtering done");

} else {

    // verify if the dptypes for the missing devices exist...
    dyn_string devTypes=missingDeviceListObject[fwConfigurationDB_DLO_DPTYPE];
    dynUnique(devTypes);
    dyn_string existingDpTypes=dpTypes();
    for (int i=1;i<=dynlen(devTypes);i++) {
	if (devTypes[i]=="") {
	    fwException_raise(exceptionInfo,"ERROR","Requested creation of device with empty DPType","");
	    int idx=dynContains(devTypes,"");
	    if (idx) {
		DebugN("Requested creation of a device with empty/unknown dptype",missingDeviceListObject[fwConfigurationDB_DLO_DPNAME][idx]);
	    }
    	    rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);
	    return;
	}

	if (!dynContains(existingDpTypes,devTypes[i])) {
	    fwException_raise(exceptionInfo,"ERROR","Requested creation of device of unknown type ("+devTypes[i]+") Maybe a component missing?","");
	    int idx=dynContains(devTypes, devTypes[i]);
	    if (idx) {
		DebugN("Requested creation of a device with unknown dptype "+devTypes[i],
		    missingDeviceListObject[fwConfigurationDB_DLO_DPNAME][idx]);
	    }
	    rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);
	    return;

	}
    }


    _fwConfigurationDB_createDevices(missingDeviceListObject,hierarchyType, exceptionInfo);
    if (dynlen(exceptionInfo)) {
        rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);
	return;
    }
}

    _fwConfigurationDB_configureDevices(deviceListObject,hierarchyType,exceptionInfo, options);
    if (dynlen(exceptionInfo)) {
	rdbRollbackTransaction(g_fwConfigurationDB_DBConnection);
	return;
    }

    rdbCommitTransaction(g_fwConfigurationDB_DBConnection);

    DebugN("DONE!");

}

