/**@name LIBRARY: pcExportDevices.ctl

@author: Herve Milcent (AB/CO)

Creation Date: 08/05/2007

Modification History: 
	- 16/08/2007: Frederic	
		update the coment line with the new format for archiving
		
	- 02/07/2007: Herve
		new format of archiving
		
	- 08/06/207: Herve
		add format

	- 29/05/2007: Herve
		add ST_GTWDEV

version 1.0

External Function : 

Internal Functions :
		
Purpose: This library contains the functions to export the PC devices.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. PC devices (DPType etc.)
	. PVSS version: 3.0
	. operating system: WXP and Linux, but tested only under WXP.
	. distributed system: yes.
*/

// constant declaration

//@{

//------------------------------------------------------------------------------------------------------------------------

// GW_FGC_Com_ExportConfig
/**
Purpose: Export the GW_FGC front-end config 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

GW_FGC_Com_ExportConfig(dyn_string dsParam, dyn_string &exceptionInfo)
{
	int iRes, i, j, iLen, digitalOKRange, alertRanges;
	dyn_int limitNumbers;
	dyn_float alertLimits;
	string sAlarmAddress, sAlarmType, sAlertText, sPlcName, sPlcDrv, sSplitModPlc;
	string sType, sDp, sCounterAddress;
	dyn_string dsDpParameters, dsAlarmDps, dsAlarmAddress, dsSplitIpAddress, dsAlertText;
	dyn_string dsAnalogArchive, dsEventArchive, dsBoolArchive, dsAddressDpParameters;
	string sFunction, sNbr;
	dyn_string dsTemp;
	unsigned uNbr;
	dyn_string exInfo;
	float fArchiveTimeFilter;
	
	// DebugN("FUNCTION: GW_FGC_Com_ExportConfig(dsParam= "+dsParam);
	
	if (dynlen(dsParam)==UN_CONFIG_EXPORT_LENGTH)
	{
		sPlcName = dsParam[UN_CONFIG_EXPORT_PLCNAME];
		sDp = c_pcSystemIntegrity_GW_FGC + sPlcName;
		iRes = dpGet(sDp + ".configuration.archive_bool", dsBoolArchive,
									sDp + ".configuration.archive_analog", dsAnalogArchive,
									sDp + ".configuration.archive_event", dsEventArchive,
									sDp + ".communication.driver_num", sPlcDrv);
		TextFieldDriver.text = sPlcDrv;
	
		if (dynlen(dsBoolArchive)>0 && dynlen(dsAnalogArchive)>0 && dynlen(dsEventArchive)>0)
		{
			TextFieldBoolArchive.text = dsBoolArchive[1];
			TextFieldAnalogArchive.text = dsAnalogArchive[1];
			TextFieldEventArchive.text = dsEventArchive[1];
		}

		//Delete
		dynClear(dsDpParameters);
		pcExportDevice_writeDeviceTypeLineFormat(PC_GW_FGC_DPTYPE, UN_DELETE_COMMAND);
		dynAppend(dsDpParameters, "#" + UN_DELETE_COMMAND);
		dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_PLCNAME]);
		dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_APPLICATION]);
		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
	
		//PLCCONFIG
		pcExportDevice_writeDeviceTypeLineFormat(PC_GW_FGC_DPTYPE, UN_PLC_COMMAND);
		dynClear(dsDpParameters);
		dynAppend(dsDpParameters, UN_PLC_COMMAND);
		dynAppend(dsDpParameters, PC_GW_FGC_DPTYPE);
		dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_PLCNAME]);
		dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_APPLICATION]);

		dpGet(sDp+".configuration.WorldFIP", sCounterAddress);
		dynAppend(dsDpParameters, sCounterAddress);
		
		//Get counter Address
		pcExportDevices_getAddressConfig(sDp + ".communication.counter", dsAddressDpParameters);

		dynAppend(dsDpParameters, dsAddressDpParameters);
		unExportDevices_getArchiveDPE(sDp+".communication.ST_FAULTS", sCounterAddress, fArchiveTimeFilter);
		dynAppend(dsDpParameters, sCounterAddress);

// write to file
		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);

// front-end system alarm
		fputs("#" + "\n" , g_fileToExport);
		dsAlarmDps = dpNames(c_unSystemAlarm_dpPattern + FE_pattern + c_pcSystemIntegrity_GW_FGC + dsParam[UN_CONFIG_EXPORT_PLCNAME] + "_*", c_unSystemAlarm_dpType);

//DebugN(dsAlarmDps);	
		iLen=dynlen(dsAlarmDps);
		if(iLen > 0) {
			pcExportDevice_writeDeviceTypeLineFormat(UN_FESYSTEMALARM_COMMAND, "");
		}
		for(i=1; i<=iLen; i++)
		{
			if (dpExists(dsAlarmDps[i]))
			{
				sSplitModPlc = substr(dsAlarmDps[i], strpos(dsAlarmDps[i], dsParam[UN_CONFIG_EXPORT_PLCNAME]) + strlen(dsParam[UN_CONFIG_EXPORT_PLCNAME]) + 1);
				
// get the number (stored in the description)

				sNbr = unGenericDpFunctions_getDescription(dsAlarmDps[i]+".");
				unConfigGenericFunctions_checkUnsigned(sNbr, uNbr, exInfo);
//DebugN(sNbr, uNbr);
				if(uNbr > 0) {
					//Get Alarm Address
					dpGet(dsAlarmDps[i] + ".alarm:_address.._drv_ident" , sAlarmAddress);
					pcExportDevices_getAddressConfig(dsAlarmDps[i] + ".alarm", dsAddressDpParameters);

					//Get alarm type
					dpGet(dsAlarmDps[i] + ".alarm:_alert_hdl.._type", sAlarmType);
					if(sAlarmType == DPCONFIG_ALERT_NONBINARYSIGNAL)
					{
						fwAlertConfig_getLimits(dsAlarmDps[i] + ".alarm", limitNumbers, alertLimits, exceptionInfo);
						alertRanges = (dynMax(limitNumbers) + 1);
						dsAlertText = makeDynString();
						for(j=1; j<=alertRanges; j++)
						{
							dpGet(dsAlarmDps[i] + ".alarm:_alert_hdl." + j + "._text", sAlertText);
							dynAppend(dsAlertText, sAlertText);
						}
																				
	//DebugN(dsAlertText);
						if(dynlen(dsAlertText) > 0)
						{
							if (dsAlertText[1]==UN_SYSTEMINTEGRITY_OK)
							{
								sType = "TRUE";
							}
							else
							{
								sType = "FALSE";
							}
						}
						else
							sType = UN_CONFIG_EXPORT_UNDEF;
					}
					else
						sType = UN_CONFIG_EXPORT_UNDEF;	
	
					dynClear(dsDpParameters);
					dynAppend(dsDpParameters, UN_FESYSTEMALARM_COMMAND);
					dynAppend(dsDpParameters, sNbr);																															//???? nbr
					dynAppend(dsDpParameters, sSplitModPlc);																									//identifier
					dynAppend(dsDpParameters, sNbr);																															//???? nbr
					dynAppend(dsDpParameters, sType);																													//alarm type
					dynAppend(dsDpParameters, unGenericDpFunctions_getDescription(dsAlarmDps[i] + ".alarm"));	//description
					dynAppend(dsDpParameters, dsAddressDpParameters);																									//address
	
					fputs("#For GW_FGC " + sNbr + "\n" , g_fileToExport);
	// write to file
					unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}
			}
		}
	}
	else
		fwException_raise(exceptionInfo, "ERROR", "GW_FGC_Com_ExportConfig(): Wrong Parameters", "");		
}

//------------------------------------------------------------------------------------------------------------------------

// pcExportDevice_writeDeviceTypeLineFormat
/**
Purpose: Write config line format

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

pcExportDevice_writeDeviceTypeLineFormat(string sDeviceType, string sCommand)
{
	string sFormat, sMandatory, sDeviceConfig;
	
	sMandatory = sDeviceType+";Device number (unique, WorldFipAgent Number);Device name (Device Identifier);description;Diagnostic panel;HTML page;Default panel;subsystem1 (Domain)=Location,Circuit Names;subsystem2 (Nature)=Interlock Type,Circuit Types;Widget Name";
	switch(sDeviceType) 
	{
		case PC_GW_FGC_DPTYPE:
			switch(sCommand)
			{
				case UN_PLC_COMMAND:
					sFormat = UN_PLC_COMMAND+";"+PC_GW_FGC_DPTYPE+";Front-end device name;Front-end application;WorldFIP bus;CMW device name;archive (Y/A/N/O/VA/VR);";
					fputs("#format: " +sFormat + "\n" , g_fileToExport);
					break;
				case UN_DELETE_COMMAND:
					sFormat =  UN_DELETE_COMMAND+";Front-end device name;Front-end application (or *);[Device type;[Device id;]]";
					fputs("#format: " +sFormat + "\n" , g_fileToExport);
					break;
				default:
					break;
			}
			break;
		case "FGC2_LHC_PC":	
			sDeviceConfig =	"I FORMAT;V FORMAT;MIN_I;MAX_I;MIN_V;MAX_V;deadband I_REF|deadband I_MEAS|deadband V_REF|deadband V_MEAS;deadband type I_REF|deadband type I_MEAS|deadband type V_REF|deadband type V_MEAS;Archive Active|Archive_I_REF|Archive_I_MEAS|Archive_V_REF|Archive_V_MEAS|Archive_ST_FAULTS|Archive_ST_UNLATCHED|Archive_ST_WARNINGS|Archive_ST_GTWDEV;CMW Device;output address (Y/N);DI/DT min,DIT/DT max;[[archive bool];[archive analog];[archive event];]";
			sFormat = sMandatory+";"+sDeviceConfig;
			fputs("#format: " +sFormat + "\n" , g_fileToExport);
			break;
		case "FGC2_PS_MPC":	
			sDeviceConfig =	"B FORMAT;V FORMAT;I FORMAT;MIN_B;MAX_B;MIN_V;MAX_V;MIN_I;MAX_I;deadband B_REF|deadband B_MEAS|deadband V_REF|deadband V_MEAS|deadband I_MEAS;deadband type B_REF|deadband type B_MEAS|deadband type V_REF|deadband type V_MEAS|deadband type I_MEAS;Archive Active|Archive_B_REF|Archive_B_MEAS|Archive_V_REF|Archive_V_MEAS|Archive_I_MEAS|Archive_ST_FAULTS|Archive_ST_UNLATCHED|Archive_ST_WARNINGS|Archive_ST_GTWDEV|Archive_PAL_CHAN_DIVPAM_ID|Archive_CONFIG_MODE;CMW Device;output address (Y/N);[[archive bool];[archive analog];[archive event];]";
			sFormat = sMandatory+";"+sDeviceConfig;
			fputs("#format: " +sFormat + "\n" , g_fileToExport);
			break;
		default:
			fputs("#no format: \n" , g_fileToExport);
			break;
	}
}

//------------------------------------------------------------------------------------------------------------------------

// GW_FGC_FGC2_LHC_PC_ExportConfig
/**
Purpose: Export FGC2_LHC_PC Devices for GW_FGC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

void GW_FGC_FGC2_LHC_PC_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	dyn_string dsAllCommonParameters, dsDpParameters, dsArchiveNames, dsAddressDpParameters;
	string sDeviceType, sArchiveActive, sArchiveName;
	float fArchiveTimeFilter;
	int i,iLeni, j, iLenj;
	dyn_int diSmoothingType;
	dyn_float dfDeadBand;
	bool bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax, bDoesExist;
	float fMinValue , fMaxValue;
	string sFormatToConvert, sFormatConverted;
	
	sDeviceType="FGC2_LHC_PC";
	
	iLeni=dynlen(dsDpList);
	if(iLeni > 0) {
		pcExportDevice_writeDeviceTypeLineFormat(sDeviceType, "");
	}
	for(i=1; i<=iLeni; i++)
	{									
		if (dpExists(dsDpList[i]))
		{
			dynClear(dsAllCommonParameters);
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
				
			//Get all common Parameters
			unExportDevice_getAllCommonParameters(dsDpList[i], sDeviceType ,  dsAllCommonParameters);
			dsDpParameters = dsAllCommonParameters;

			sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.I_MEAS");
			sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
			dynAppend(dsDpParameters, sFormatConverted);
			sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.V_MEAS");
			sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
			dynAppend(dsDpParameters, sFormatConverted);

			// get min/max I and V
			fwPvRange_get(dsDpList[i]+".ProcessInput.I_MEAS", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
			dynAppend(dsDpParameters, fMinValue);
			dynAppend(dsDpParameters, fMaxValue);
			fwPvRange_get(dsDpList[i]+".ProcessInput.V_MEAS", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
			dynAppend(dsDpParameters, fMinValue);
			dynAppend(dsDpParameters, fMaxValue);
			
			// get deadband value and deadband type 
			unExportDevices_getSmoothingDPE(dsDpList[i], makeDynString(".ProcessInput.I_REF", 
																														".ProcessInput.I_MEAS",
																														".ProcessInput.V_REF",
																														".ProcessInput.V_MEAS"),
																							diSmoothingType, dfDeadBand);
			
			iLenj = dynlen(diSmoothingType);
			sArchiveActive = "";
			for(j=1; j<=iLenj; j++) {
				if(sArchiveActive == "")
					sArchiveActive = dfDeadBand[j];
				else
					sArchiveActive += "|" + dfDeadBand[j];
			}
			dynAppend(dsDpParameters, sArchiveActive);
			sArchiveActive = "";
			for(j=1; j<=iLenj; j++) {
				if(sArchiveActive == "")
					sArchiveActive = diSmoothingType[j];
				else
					sArchiveActive += "|" + diSmoothingType[j];
			}
			dynAppend(dsDpParameters, sArchiveActive);

			//Get Archive Parameters
			unExportDevices_getArchiveDPE(makeDynString(	dsDpList[i]+".ProcessInput.I_MEAS",
																										dsDpList[i]+".ProcessInput.I_REF", 
																										dsDpList[i]+".ProcessInput.I_MEAS",
																										dsDpList[i]+".ProcessInput.V_REF",
																										dsDpList[i]+".ProcessInput.V_MEAS",
																										dsDpList[i]+".ProcessInput.ST_FAULTS",
																										dsDpList[i]+".ProcessInput.ST_UNLATCHED",
																										dsDpList[i]+".ProcessInput.ST_WARNINGS",
																										dsDpList[i]+".ProcessInput.ST_GTWDEV"), 
																										sArchiveActive, fArchiveTimeFilter);
			dynAppend(dsDpParameters, sArchiveActive);
																																						
			//Get address config		
			pcExportDevices_getAddressConfig(dsDpList[i]+".ProcessInput.ST_FAULTS", dsAddressDpParameters);
			dynAppend(dsDpParameters, dsAddressDpParameters);
			
			// get address config for output
			pcExportDevices_getAddressConfig(dsDpList[i]+".ProcessOutput.REF", dsAddressDpParameters);
			if(dsAddressDpParameters[1] == UN_CONFIG_EXPORT_UNDEF)
				dynAppend(dsDpParameters, "N");
			else
				dynAppend(dsDpParameters, "Y");
			
			// get DI/DT min and max
			dpGet(dsDpList[i]+".dIdT",sArchiveActive);
			dynAppend(dsDpParameters, sArchiveActive);
			
			//Get Archive Names for PC devices
			pcExportDevices_getArchiveNames(dsDpList[i], sDeviceType, dsArchiveNames);
			iLenj=dynlen(dsArchiveNames);
			for(j=1; j<=iLenj ; j++)
			{
				dynAppend(dsDpParameters, dsArchiveNames[j]);
			}	
			
			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//dp doesn't exist
	}//end for
}

//------------------------------------------------------------------------------------------------------------------------

// GW_FGC_FGC2_PS_MPC_ExportConfig
/** returns the format of the config line for a device type
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsDpList input, list of device dp
@param exceptionInfo output, the errors are returned here
*/
void GW_FGC_FGC2_PS_MPC_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
  dyn_string dsAllCommonParameters, dsDpParameters, dsArchiveNames, dsAddressDpParameters;
  string sDeviceType, sArchiveActive, sArchiveName;
  float fArchiveTimeFilter;
  int i,iLeni, j, iLenj;
  dyn_int diSmoothingType;
  dyn_float dfDeadBand;
  bool bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax, bDoesExist;
  float fMinValue , fMaxValue;
  string sFormatToConvert, sFormatConverted;
  
  sDeviceType="FGC2_PS_MPC";
  
  iLeni=dynlen(dsDpList);
  if(iLeni > 0) {
    pcExportDevice_writeDeviceTypeLineFormat(sDeviceType, "");
  }
  for(i=1; i<=iLeni; i++)
  {                  
    if (dpExists(dsDpList[i]))
    {
      dynClear(dsAllCommonParameters);
      dynClear(dsDpParameters);
      dynClear(dsArchiveNames);
        
      //Get all common Parameters
      unExportDevice_getAllCommonParameters(dsDpList[i], sDeviceType ,  dsAllCommonParameters);
      dsDpParameters = dsAllCommonParameters;

      sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.B_MEAS");
      sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
      dynAppend(dsDpParameters, sFormatConverted);
      sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.V_MEAS");
      sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
      dynAppend(dsDpParameters, sFormatConverted);
      sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.I_MEAS");
      sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
      dynAppend(dsDpParameters, sFormatConverted);

      // get min/max B, V and I
      fwPvRange_get(dsDpList[i]+".ProcessInput.B_MEAS", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
      dynAppend(dsDpParameters, fMinValue);
      dynAppend(dsDpParameters, fMaxValue);
      fwPvRange_get(dsDpList[i]+".ProcessInput.V_MEAS", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
      dynAppend(dsDpParameters, fMinValue);
      dynAppend(dsDpParameters, fMaxValue);
      fwPvRange_get(dsDpList[i]+".ProcessInput.I_MEAS", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
      dynAppend(dsDpParameters, fMinValue);
      dynAppend(dsDpParameters, fMaxValue);
      
      // get deadband value and deadband type 
      unExportDevices_getSmoothingDPE(dsDpList[i], makeDynString(".ProcessInput.B_REF", 
                                                            ".ProcessInput.B_MEAS",
                                                            ".ProcessInput.V_REF",
                                                            ".ProcessInput.V_MEAS",
                                                            ".ProcessInput.I_MEAS"),
                                              diSmoothingType, dfDeadBand);
      
      iLenj = dynlen(diSmoothingType);
      sArchiveActive = "";
      for(j=1; j<=iLenj; j++) {
        if(sArchiveActive == "")
          sArchiveActive = dfDeadBand[j];
        else
          sArchiveActive += "|" + dfDeadBand[j];
      }
      dynAppend(dsDpParameters, sArchiveActive);
      sArchiveActive = "";
      for(j=1; j<=iLenj; j++) {
        if(sArchiveActive == "")
          sArchiveActive = diSmoothingType[j];
        else
          sArchiveActive += "|" + diSmoothingType[j];
      }
      dynAppend(dsDpParameters, sArchiveActive);

      //Get Archive Parameters
      unExportDevices_getArchiveDPE(makeDynString(  dsDpList[i]+".ProcessInput.B_MEAS",
                                                    dsDpList[i]+".ProcessInput.B_REF", 
                                                    dsDpList[i]+".ProcessInput.B_MEAS",
                                                    dsDpList[i]+".ProcessInput.V_REF",
                                                    dsDpList[i]+".ProcessInput.V_MEAS",
                                                    dsDpList[i]+".ProcessInput.I_MEAS",
                                                    dsDpList[i]+".ProcessInput.ST_FAULTS",
                                                    dsDpList[i]+".ProcessInput.ST_UNLATCHED",
                                                    dsDpList[i]+".ProcessInput.ST_WARNINGS",
                                                    dsDpList[i]+".ProcessInput.ST_GTWDEV",
                                                    dsDpList[i]+".ProcessInput.PAL_CHAN_DIVPAM_ID",
                                                    dsDpList[i]+".ProcessInput.CONFIG_MODE"), 
                                                    sArchiveActive, fArchiveTimeFilter);
      dynAppend(dsDpParameters, sArchiveActive);
                                                                            
      //Get address config    
      pcExportDevices_getAddressConfig(dsDpList[i]+".ProcessInput.ST_FAULTS", dsAddressDpParameters);
      dynAppend(dsDpParameters, dsAddressDpParameters);
      
      // get address config for output
      pcExportDevices_getAddressConfig(dsDpList[i]+".ProcessOutput.PAL_CHAN_DIVPAM_ID", dsAddressDpParameters);
      if(dsAddressDpParameters[1] == UN_CONFIG_EXPORT_UNDEF)
        dynAppend(dsDpParameters, "N");
      else
        dynAppend(dsDpParameters, "Y");
      
      //Get Archive Names for PC devices
      pcExportDevices_getArchiveNames(dsDpList[i], sDeviceType, dsArchiveNames);
      iLenj=dynlen(dsArchiveNames);
      for(j=1; j<=iLenj ; j++)
      {
        dynAppend(dsDpParameters, dsArchiveNames[j]);
      }  
      
      unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }//dp doesn't exist
  }//end for
}

//------------------------------------------------------------------------------------------------------------------------

// pcExportDevices_getArchiveNames
/**
Purpose: Get Archives names (Bool, Analog and Event values)

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/
void pcExportDevices_getArchiveNames(string sDp, string sDeviceType, dyn_string &dsNames)
{
  int i, j, iLen, iLen_j;
  string sArchiveClass, sName;
  dyn_string dsDpe, dsTemp, dsCheck;       
    
  switch(sDeviceType)
  {
    case "FGC2_LHC_PC":
			dynAppend(dsDpe, ".ProcessInput.ST_FAULTS" + DELIMITER);			
			dynAppend(dsDpe, ".ProcessInput.I_MEAS" + DELIMITER);
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			break;
    case "FGC2_PS_MPC":
      dynAppend(dsDpe, ".ProcessInput.ST_GTWDEV" + DELIMITER);
      dynAppend(dsDpe, ".ProcessInput.B_MEAS" + DELIMITER);
      dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);
      break;
		default:
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			break;
  }
		
		iLen=dynlen(dsDpe);
		for (i=1; i<=iLen; i++)
			{
			if (dsDpe[i]!=UN_CONFIG_EXPORT_UNDEF)
				{
				dynClear(dsTemp);
				dynClear(dsCheck);
				
				dsTemp=strsplit(dsDpe[i], DELIMITER);
				iLen_j=dynlen(dsTemp);
				
				for (j=1; j<=iLen_j; j++)
					{
					sName="";
					sArchiveClass="";
					
					dpGet(sDp + dsTemp[j] + ":_archive.1._class", sArchiveClass);
		
					if(dpExists(sArchiveClass + ".general.arName"))
				 		dpGet(sArchiveClass + ".general.arName:_online.._value", sName);
				 	else
				 		{
				 		if (i==1)
							sName=UN_CONFIG_EXPORT_ARCHIVE_1;
						else if (i==2)
							sName=UN_CONFIG_EXPORT_ARCHIVE_2;
						else if (i==3)
							sName=UN_CONFIG_EXPORT_ARCHIVE_3;
						else
							sName=UN_CONFIG_EXPORT_UNDEF;	
				 		}
				 	
				 	dynAppend(dsCheck, sName);	
					}
				
				dynUnique(dsCheck);
				
				if (dynlen(dsCheck)==1)		//check if same archive for same type
					sName=dsCheck[1];	
				else
					sName=UN_CONFIG_EXPORT_UNDEF;	
					
				}
			else		//default
				{
				if (i==1)
					sName=UN_CONFIG_EXPORT_ARCHIVE_1;
				else if (i==2)
					sName=UN_CONFIG_EXPORT_ARCHIVE_2;
				else if (i==3)
					sName=UN_CONFIG_EXPORT_ARCHIVE_3;
				else
					sName=UN_CONFIG_EXPORT_UNDEF;	
				}
							
			dynAppend(dsNames, sName);	
			}
		
}

//------------------------------------------------------------------------------------------------------------------------
// pcExportDevices_getAddressConfig
/**
Purpose:  Get address config parameters of a dp

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/

pcExportDevices_getAddressConfig(string sDpe, dyn_string &dsAddressParameters)
{
	string sDp=dpSubStr(sDpe, DPSUB_SYS_DP);
	dyn_string dsSplit;
	string sAddress, sTemp, sResult, sTempValue;
	
//DebugN(sDp);
	if(sDp != "") {
		dpGet(sDpe+":_address.._reference", sAddress);
		if(sAddress != "") { // == dev$prop$tag
			sResult = substr(sAddress, 0, strpos(sAddress, "$"));
		}
		else
			sResult = UN_CONFIG_EXPORT_UNDEF;
	}
	else
		sResult = UN_CONFIG_EXPORT_UNDEF;

	dsAddressParameters = makeDynString(sResult);
}

//------------------------------------------------------------------------------------------------------------------------
																												
//@}

