/**@name LIBRARY: unSystemIntegrity_RDB.ctl

@author: 
Francisca Calheiros (IT/CO)
Pawel Macuda (EN-ICE-SCD)

Modification History: 
  28/09/2011: Herve
  - IS-611: unSystemIntegrity  remove the deprecated function calls of the fwInstallation 
 
  05/09/2011: Herve
  - IS-598: system name and component version added in the MessageText log and in the diagnostic  
  
Creation Date: 28/03/2008

version 1.0
	
Purpose: 
	Library of the RDB component used by the systemIntegrity.
	 	
Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

*/

//--------------------------------------------------------------------------------------------------------------------------------
// constant declaration
//------------------
const string UN_SYSTEM_INTEGRITY_RDB = "RDB";
const string RDB_pattern = "RDB";
//const int c_unSystemIntegrity_defaultDiskSpaceThreshold = 20;
const int c_unSystemIntegrity_defaultDbCheckInterval = 600;
const int c_unSystemIntegrity_defaultNumberRoundtripFailures = 6;
const int c_unSystemIntegrity_defaultNumberManagerRestarts = 5;

const string RDB_DP_NAME = "_RDBArchive";
const string RDB_CONNECTION_STATUS =".dbConnection.connected";
const string RDB_CURRENT_BUFFER_BLOCKS = ".buffer.currentBlocks";
const string RDB_MAX_BUFFER_BLOCKS = ".buffer.maxBlocks";
const string RDB_BUFFER_TO_DISCK_INDICATOR = ".buffer.bufferToDiskIndicator";
const string RDB_BUFFER_FLUSH_INTERVAL = ".buffer.flushInterval";
//const string DISK_SPACE_AVAILABLE_PERC = "_ArchivDisk.AvailPerc";

const string RDB_CONNECTION_CHECK_DP_TYPE = "_UnSystemIntegrityRDB";
const string RDB_CONNECTION_CHECK_DP = "RDBConnection_systemIntegrity";
const string RDB_CONNECTION_CHECK_VALUE = ".value";
const string RDB_CONNECTION_CHECK_CONNECTION_STATUS = ".connectionStatus";
//const string RDB_CONNECTION_CHECK_DB_CHECK_INTERVAL = ".dbCheckInterval";

const int RDB_CONFIG_DATA_NUMBER =3;
const int RDB_DB_CHECK_INTERVAL = 1;
const int RDB_MANAGER_RESTARTS_NUMBER = 2;
const int RDB_ROUNDTRIP_FAILURES_NUMBER = 3;

//const int for EDB logging
const int c_unSystemIntegrity_RDB_OK = 0;
const int c_unSystemIntegrity_RDB_Flushing = -1;
const int c_unSystemIntegrity_RDB_BufferingToDisk = 6;
const int c_unSystemIntegrity_RDB_NoConnection = 11;
const int c_unSystemIntegrity_RDB_NoConnection_Buffering = 12;
const int c_unSystemIntegrity_RDB_ManagerStopped_DataLost = 16;
const int c_unSystemIntegrity_RDB_ManagerBlocked_DataLost = 17;
const int c_unSystemIntegrity_RDB_NoConnection_DataLost = 18;
const int c_unSystemIntegrity_RDB_DataNotWritten_DataLost = 19;

//------------------

// global declaration
global dyn_string g_unSystemIntegrity_RDBList; // list of all RDB archives to check
global int g_unSystemIntegrity_RDBmaxBlocks; // max limit of RDB blocks in buffer
global int g_unSystemIntegrity_MonitoringThread; // RDB monitoring thread id
global int g_unSystemIntegrity_BufferToDisk = -1;// 
global int g_unSystemIntegrity_dbConnectionStatus; // value of the PVSS internal data-point element .dbConnection
global int g_unSystemIntegrity_BufferToDiskIndicator; // value of the PVSS internal data-point element .buffer.bufferToDiskIndicator

global int g_unSystemIntegrity_dbCheckInterval; // Time between next write/read do the database
global int g_unSystemIntegrity_writeReadInterval; //Time between write and read form a database, internal PVSS dpe
global int g_unSystemIntegrity_maxManagerRestarts; // Maximum number of tires to restart RDB Manager
global int g_unSystemIntegrity_maxRoundtripFailures; //Maximum, allowed number roundtrips to DB that fails
//@{

//---------------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_RDB_Initialize
/**
Purpose:
Get the list of RDB DPs to be checked by the systemIntegrity.

@param dsResult: dyn_string, output, the list of enabled RDB manager DPs to check

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

*/
unSystemIntegrity_RDB_Initialize(dyn_string &dsResult)
{
  dyn_string dsList, configData;
  int len, i;
  string dpToCheck;
  bool enabled;
  dyn_string exceptionInfo;
  string fileName =  getPath(CONFIG_REL_PATH, "config");

  paCfgReadValue(fileName, "ValueArchiveRDB", "bufferToDisk", g_unSystemIntegrity_BufferToDisk);
  if(g_unSystemIntegrity_BufferToDisk == -1)
    g_unSystemIntegrity_BufferToDisk = 1;
  
  if(!dpExists(RDB_CONNECTION_CHECK_DP))
  {
    int archiveType = DPATTR_ARCH_PROC_VALARCH;
    int smoothProcedure = 0;
    float deadband = 0;
    float timeInterval = 0; 
    bool checkClass = TRUE;
    
    dyn_string archiveClasses;
    dyn_string archiveGroupDps;
    
    int type = false;
    
    dpCreate(RDB_CONNECTION_CHECK_DP, RDB_CONNECTION_CHECK_DP_TYPE);
//    dpSet(RDB_CONNECTION_CHECK_DP + RDB_CONNECTION_CHECK_DB_CHECK_INTERVAL, c_unSystemIntegrity_defaultDbCheckInterval);
    
    dpGet(RDB_CONNECTION_CHECK_DP + RDB_CONNECTION_CHECK_VALUE + ":_" + "archive.._type", type);
    
    if(type == 0)
    {      
      fwArchive_getAllRDBArchiveClasses(makeDynString(getSystemName()), archiveClasses, archiveGroupDps, exceptionInfo);
    
      if(dynlen(archiveClasses) > 0)//== 1 && patternMatch("*RDB*", archiveClasses))
      {
        fwArchive_set(RDB_CONNECTION_CHECK_DP + RDB_CONNECTION_CHECK_VALUE, archiveClasses[1], archiveType, smoothProcedure, deadband, timeInterval, exceptionInfo); 
          if(dynlen(exceptionInfo)){return;}    
      }
      else
        DebugN("unSystemIntegrity_RDB_Initialize(): RDB archive could not be set on " + RDB_CONNECTION_CHECK_DP + RDB_CONNECTION_CHECK_VALUE + " ! archiveClasses, archiveGroupDps: ", archiveClasses, archiveGroupDps);
    }
  }

  if(dpExists(UN_SYSTEM_INTEGRITY_RDB + UN_SYSTEMINTEGRITY_EXTENSION))
  {
    dpGet(UN_SYSTEM_INTEGRITY_RDB + UN_SYSTEMINTEGRITY_EXTENSION + ".config.data", configData);
    if(dynlen(configData)!= RDB_CONFIG_DATA_NUMBER)
    {
      dpSet(UN_SYSTEM_INTEGRITY_RDB + UN_SYSTEMINTEGRITY_EXTENSION + ".config.data", 
            makeDynString(c_unSystemIntegrity_defaultDbCheckInterval, c_unSystemIntegrity_defaultNumberRoundtripFailures, c_unSystemIntegrity_defaultNumberManagerRestarts)
            );
      g_unSystemIntegrity_dbCheckInterval = c_unSystemIntegrity_defaultDbCheckInterval; 
      g_unSystemIntegrity_maxManagerRestarts = c_unSystemIntegrity_defaultNumberRoundtripFailures; 
      g_unSystemIntegrity_maxRoundtripFailures = c_unSystemIntegrity_defaultNumberManagerRestarts; 
    }
    else
    {
      g_unSystemIntegrity_dbCheckInterval = configData[RDB_DB_CHECK_INTERVAL]; 
      g_unSystemIntegrity_maxManagerRestarts = configData[RDB_MANAGER_RESTARTS_NUMBER]; 
      g_unSystemIntegrity_maxRoundtripFailures = configData[RDB_ROUNDTRIP_FAILURES_NUMBER]; 
    }
  }
  
// get the list of RDB manager dp to check
  dsList = dpNames(c_unSystemAlarm_dpPattern + RDB_pattern + "*", c_unSystemAlarm_dpType);

  len = dynlen(dsList);
  for(i = 1; i<=len; i++) 
  {
    dpToCheck = dpSubStr(dsList[i], DPSUB_DP);
    dpToCheck = substr(dpToCheck, strlen(c_unSystemAlarm_dpPattern + RDB_pattern), strlen(dpToCheck));              
 
    dpGet(RDB_DP_NAME + RDB_MAX_BUFFER_BLOCKS, g_unSystemIntegrity_RDBmaxBlocks, //get the max limit of blocks in buffer
	  dsList[i] + ".enabled", enabled);
    
    if(enabled)
      dynAppend(dsResult, dpToCheck);
      }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_RDB_HandleCommand
/**
Purpose: handle any systemIntegrity command.

@param sDpe1: string, input, dpe
@param command: int, input, the systemIntegrity command
@param sDpe2: string, input, dpe
@param parameters: dyn_string, input, the list of parameters of the systemIntegrity command

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

*/
unSystemIntegrity_RDB_HandleCommand(string sDpe1, int command, string sDpe2, dyn_string parameters)
{
	dyn_string exceptionInfo;
	int i, len =dynlen(parameters);

	switch(command) {
		case UN_SYSTEMINTEGRITY_ADD: // add the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				unSystemIntegrity_RDB_checking(parameters[i], true, true, exceptionInfo);
			}
    unSystemIntegrity_manageRDBMonitoring("start");
			break;
		case UN_SYSTEMINTEGRITY_DELETE: // delete the _UnSystemAlarm DP
			for(i=1; i<=len; i++) 
    {
				unSystemIntegrity_RDB_checking(parameters[i], false, false, exceptionInfo);
				// remove dp from the alert list of applicationDP if it is in
				_unSystemIntegrity_modifyApplicationAlertList(c_unSystemAlarm_dpPattern + RDB_pattern+parameters[i], false, exceptionInfo);
				// delete the _UnSystemAlarm dps
				if(dpExists(c_unSystemAlarm_dpPattern + RDB_pattern+parameters[i]))
				   dpDelete(c_unSystemAlarm_dpPattern + RDB_pattern+parameters[i]);
			}
    delay(1);
    unSystemIntegrity_manageRDBMonitoring("stop");
			break;
		case UN_SYSTEMINTEGRITY_ENABLE: // enable the _UnSystemAlarm DP
			for(i=1; i<=len; i++) 
    {
				if(dpExists(c_unSystemAlarm_dpPattern + RDB_pattern + parameters[i]))
				   unSystemIntegrity_RDB_checking(parameters[i], false, true, exceptionInfo);
			}
    unSystemIntegrity_manageRDBMonitoring("start");
			break;
		case UN_SYSTEMINTEGRITY_DISABLE: // disable the _UnSystemAlarm DP
			for(i=1; i<=len; i++) 
    {
				if(dpExists(c_unSystemAlarm_dpPattern + RDB_pattern+parameters[i]))
				   unSystemIntegrity_RDB_checking(parameters[i], false, false, exceptionInfo);
			}
    delay(1);
    unSystemIntegrity_manageRDBMonitoring("stop");
			break;
		case UN_SYSTEMINTEGRITY_DIAGNOSTIC: // give the list of _UnSystemAlarm DP and the state
			dpSet(UN_SYSTEM_INTEGRITY_RDB + UN_SYSTEMINTEGRITY_EXTENSION + ".diagnostic.commandResult", command, 
			      UN_SYSTEM_INTEGRITY_RDB + UN_SYSTEMINTEGRITY_EXTENSION + ".diagnostic.result", g_unSystemIntegrity_RDBList);
			break;
		default:
			break;
	}

	if(dynlen(exceptionInfo) > 0) {
		if(isFunctionDefined("unMessageText_sendException")) {
		   unMessageText_sendException("*", "*", g_SystemIntegrity_sSystemName+"unSystemIntegrity_RDB_HandleCommand", "user", "*", exceptionInfo);
		}
// handle any error in case the send message failed
		if(dynlen(exceptionInfo) > 0) {
			DebugTN(g_SystemIntegrity_sSystemName+"unSystemIntegrity_RDB_HandleCommand", exceptionInfo);
		}
	}
//	DebugN(sDpe1, command, parameters, g_unSystemIntegrity_RDB);
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_RDB_checking
/**
Purpose:
This function register/de-register the callback funtion of RDB. This function can also create the _unSystemAlarm_RDB dp 
with the alarm config but it cannot delete it.

	@param sDp: string, input, data point name
	@param bCreate: bool, input, true create the dp and the alarm config if it does not exist, enable it
	@param bRegister: bool, input, true do a dpConnect, false do a dpDisconnect
	@param exceptionInfo: dyn_string, output, exception are returned here

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

*/

unSystemIntegrity_RDB_checking(string sDp, bool bCreate, bool bRegister, dyn_string &exceptionInfo)
{
	string dpToCheck, dp, description, sCallBack;
	int res;
	dyn_string dsParam;
        dyn_string dsAlertText,dsAlertClass;
        dyn_float dfLimits;

       // remove the system name
	res = strpos(sDp, ":");
	if(res >= 0)
		dp = substr(sDp, res+1, strlen(sDp));
	else
		dp = sDp;

	dpToCheck = dp;

	dp = c_unSystemAlarm_dpPattern + RDB_pattern + dpToCheck;
        
	if(bCreate) 
  {
// create the sDp and its alarm config if it is not existing
    if(!dpExists(dp)) 
    {
      description = sDp;
      
      dsAlertText[1] = "Flushing to database";
      dsAlertText[2] = "OK";
      dsAlertText[3] = "Buffering to disk";    
      dsAlertText[4] = "No connection to database";  //"RDB manager disconnecd";  
//      dsAlertText[3] = "RDB Manager Blocked";  //"Number of Buffer MaxBlocks exceeded";  
//      dsAlertText[4] = "Low disk space";  //"Disk space running low";
      dsAlertText[5] = "Archive data lost";  //"Disk space running low";
  
      dfLimits[1] = 0;
      dfLimits[2] = 5;
      dfLimits[3] = 10;
      dfLimits[4] = 15;
//      dfLimits[4] = 20;

      dsAlertClass[1] = "_fwWarningNack.";
      dsAlertClass[2] = "";
      dsAlertClass[3] = "_fwWarningNack_41.";
      dsAlertClass[4] = "_fwWarningNack_42.";
      dsAlertClass[5] = "_fwErrorNack.";       
//      dsAlertClass[4] = "_fwErrorNack_61.";   
//      dsAlertClass[5] = "_fwErrorNack_62.";

      _unSystemIntegrity_createSystemAlarmBasic(dp, description, dsAlertText, dfLimits, dsAlertClass, exceptionInfo);
          
    }
	}        

      sCallBack = "unSystemIntegrity_RDB_checkingCallback";
      dsParam = makeDynString(dpToCheck + RDB_CONNECTION_STATUS, RDB_DP_NAME + RDB_BUFFER_TO_DISCK_INDICATOR, RDB_CONNECTION_CHECK_DP + RDB_CONNECTION_CHECK_CONNECTION_STATUS); //, DISK_SPACE_AVAILABLE_PERC);
      if(dynlen(exceptionInfo)<=0)
	       _unSystemIntegrity_registerFunction(bRegister, sCallBack, dsParam, g_unSystemIntegrity_RDBList, dp, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_RDB_checkingCallback
/**
Purpose:
Callback function. This function is called when the system alarm is enabled for this device.

@param sDp1: string, input, data point name
@param state: int, input, state of the archive, 1: online, 0: offline
@param sDp2: string, input, data point name
@param error: int, input, error of the archive, 0: no error
@param sDp3: string, input, data point name
@param errBit: bool, input, true: archive overflow, false: no error
@param sDp4: 
Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

*/
unSystemIntegrity_RDB_checkingCallback(string sDp1, int state, string sDp2, int bufferToDiskIndicator, string sDp3, int connectionStatus) //, string sDp3, int diskSpacePrec)
{
  string sDp;
  int value; 
  dyn_string exInfo, configData;
  int maxBufferBlocks;
  int currentBlocks;
//  int connectionStatus;
//  int bufferToDiskIndicator;
  int iManagerState;
  string managerName = "PVSS00rdb";
  string managerOption = "";

  g_unSystemIntegrity_dbConnectionStatus = state;
  g_unSystemIntegrity_BufferToDiskIndicator = bufferToDiskIndicator;
  
// get the dpname without the systemname
  sDp = dpSubStr(sDp1, DPSUB_DP); 

  if(!dpExists(c_unSystemAlarm_dpPattern + RDB_pattern + sDp))
    return;
  
  value = 0;

  unSystemIntegrity_getState(managerName, managerOption, iManagerState);
  if(iManagerState != SYSTEMINTEGRITY_MANAGER_RUNNING) 
  {
    value = c_unSystemIntegrity_RDB_ManagerStopped_DataLost;   //RDB manager is not running. Archive data lost
    if(iManagerState == SYSTEMINTEGRITY_MANAGER_BLOCKED) 
      value = c_unSystemIntegrity_RDB_ManagerBlocked_DataLost; ////RDB manager is in blocking state. Archive data lost
  }
  else //iManagerState == SYSTEMINTEGRITY_MANAGER_RUNNING 
  {
    dpGet(RDB_DP_NAME + RDB_MAX_BUFFER_BLOCKS, maxBufferBlocks
          ,RDB_DP_NAME + RDB_CURRENT_BUFFER_BLOCKS, currentBlocks
          );
    if(connectionStatus == 0)
    {
      if(g_unSystemIntegrity_BufferToDisk == 0)
      {
        if(g_unSystemIntegrity_dbConnectionStatus == 0)
          value = c_unSystemIntegrity_RDB_NoConnection_DataLost; //RDB Manager running but there is no connection to DB and data is not buffered. Archive data lost
      }
      else //g_unSystemIntegrity_BufferToDisk == 1 || == 2
      {
        if(g_unSystemIntegrity_dbConnectionStatus == 0)
        {
          if(g_unSystemIntegrity_BufferToDiskIndicator == 2)
            value = c_unSystemIntegrity_RDB_DataNotWritten_DataLost; //RDB Manager running but there is no connection to DB. Buffers is decreasing. Archive data lost
          else if(g_unSystemIntegrity_BufferToDiskIndicator == 1)
            value = c_unSystemIntegrity_RDB_NoConnection_Buffering; // RDB Manager running. No connection do DB. Data are buffered to disk
          else if(g_unSystemIntegrity_BufferToDiskIndicator == 0)
          {
            value = c_unSystemIntegrity_RDB_NoConnection; // Lost connection to DB
            if(currentBlocks >= maxBufferBlocks) //*************** It's higly probable that we will never be able to see this condition, however thread with periodic roundtrip will be able detect this abnormal state.
              value = c_unSystemIntegrity_RDB_NoConnection_DataLost; // RDB Manager running. No connection do DB. Data are not buffered to disk. Archive data lost
          }
        }
        else //g_unSystemIntegrity_dbConnectionStatus == 1
        {
          if(g_unSystemIntegrity_BufferToDiskIndicator == 2)
            value = c_unSystemIntegrity_RDB_Flushing; //RDB Manager running. Connection to DB is OK. Flushing data to the DB
          else if(g_unSystemIntegrity_BufferToDiskIndicator == 1 && g_unSystemIntegrity_BufferToDisk == 1)
            value = c_unSystemIntegrity_RDB_BufferingToDisk; //RDB Manager running. Connection to DB is OK. Memory buffer is ramping up
        }    
      }
    }
    else //connectionStatus == 1
    {
      if(g_unSystemIntegrity_BufferToDisk == 0)
      {
        if(g_unSystemIntegrity_dbConnectionStatus == 0)
          value = c_unSystemIntegrity_RDB_NoConnection_DataLost; // Connection to DB went down and data are not buffered to disk. Archive data lost.
      }
      else //(g_unSystemIntegrity_BufferToDisk != 0)
      {
        if(g_unSystemIntegrity_dbConnectionStatus == 0)
          value = c_unSystemIntegrity_RDB_NoConnection; // Lost connection to DB
      }
    }    
  }
  
  string timeStamp = getCurrentTime();
  switch(value)
  {
    case c_unSystemIntegrity_RDB_Flushing:
      unSystemIntegrity_throwError("unSystemIntegrity_RDB_checkingCallback() ->  Flushing data to database. " + timeStamp);
      break;
    case c_unSystemIntegrity_RDB_BufferingToDisk:
      unSystemIntegrity_throwError("unSystemIntegrity_RDB_checkingCallback() ->  Data are being bufffered to disk. " + timeStamp);
      break;
    case c_unSystemIntegrity_RDB_NoConnection:
      unSystemIntegrity_throwError("unSystemIntegrity_RDB_checkingCallback() ->  RDB manages has lost connection to the database. " + timeStamp);
      break;
    case c_unSystemIntegrity_RDB_NoConnection_Buffering:
      unSystemIntegrity_throwError("unSystemIntegrity_RDB_checkingCallback() ->  RDB manages has lost connection to the database. Buffering to disk. " + timeStamp);
      break;
    case c_unSystemIntegrity_RDB_DataNotWritten_DataLost:
      unSystemIntegrity_throwError("unSystemIntegrity_RDB_checkingCallback() ->  Data are not written to the database. Losing archive data. " + timeStamp);
      break;
    case c_unSystemIntegrity_RDB_NoConnection_DataLost:
      unSystemIntegrity_throwError("unSystemIntegrity_RDB_checkingCallback() ->  No connection to the database and data are not buffered to disck. Losing archive data. " + timeStamp);
      break;
    case c_unSystemIntegrity_RDB_ManagerStopped_DataLost:
      unSystemIntegrity_throwError("unSystemIntegrity_RDB_checkingCallback() ->  RDB Manager is not running. Losing archive data. " + timeStamp);
      break;
    case c_unSystemIntegrity_RDB_ManagerBlocked_DataLost:
    unSystemIntegrity_throwError("unSystemIntegrity_RDB_checkingCallback() ->  RDB Manager in blocking state. " + timeStamp);
      break;
    default:
      break;
  }  
    
  dpSet( c_unSystemAlarm_dpPattern + RDB_pattern + sDp + ".alarm", value);

}

//------------------------------------------------------------------------------------------------------------------------

//@}

unSystemIntegrity_RDB_DataCallback(string sDPe, dyn_string &dsResult)
{

//  int diskSpacePrec;
//  int currentBlocks;
//  int state;
//  int connectionStatus;
  dyn_string configData;

  dpGet(UN_SYSTEM_INTEGRITY_RDB + UN_SYSTEMINTEGRITY_EXTENSION + ".config.data", configData);

  if(dynlen(configData) == RDB_CONFIG_DATA_NUMBER)
  {
    g_unSystemIntegrity_dbCheckInterval = configData[RDB_DB_CHECK_INTERVAL]; 
    g_unSystemIntegrity_maxManagerRestarts = configData[RDB_MANAGER_RESTARTS_NUMBER]; 
    g_unSystemIntegrity_maxRoundtripFailures = configData[RDB_ROUNDTRIP_FAILURES_NUMBER]; 
  }
  else  
  {
    DebugN("unSystemIntegrity_RDB_DataCallback-> Inconsistency has been found in " + UN_SYSTEM_INTEGRITY_RDB + UN_SYSTEMINTEGRITY_EXTENSION + ".config.data" + " No action has been taken.");
  }
  
/*
  dpGet(RDB_DP_NAME + RDB_CONNECTION_STATUS, state
        ,RDB_DP_NAME + RDB_CURRENT_BUFFER_BLOCKS, currentBlocks
        ,RDB_CONNECTION_CHECK_DP + RDB_CONNECTION_CHECK_CONNECTION_STATUS, connectionStatus
//        ,DISK_SPACE_AVAILABLE_PERC, diskSpacePrec
       );
  unSystemIntegrity_RDB_checkingCallback(RDB_DP_NAME + RDB_CONNECTION_STATUS, state,
                                         RDB_DP_NAME + RDB_CURRENT_BUFFER_BLOCKS, currentBlocks
                                         ,RDB_CONNECTION_CHECK_DP + RDB_CONNECTION_CHECK_CONNECTION_STATUS, connectionStatus 
//                                         ,DISK_SPACE_AVAILABLE_PERC, diskSpacePrec
                                         );
*/                                         
}

//------------------------------------------------------------------------------------------------------------------

unSystemIntegrity_manageRDBMonitoring(string command)
{
  dyn_string dsList;
  bool enable;
  int err;
  
  dsList = dpNames(c_unSystemAlarm_dpPattern + RDB_pattern + "*", c_unSystemAlarm_dpType);
  
  switch(command)
  {
    case "start":
      if(g_unSystemIntegrity_MonitoringThread == 0)
      {
        if(dynlen(dsList)> 0)
        {
          g_unSystemIntegrity_MonitoringThread = startThread("unSystemIntegrity_RDB_checkDbConnection");
//DebugN("***********STARTING THREAD*********");
        }   
      }
      break;
    case "stop":  
      for(int i =1; i<=dynlen(dsList); i++)
      {
        dpGet(dsList[i] + ".enabled", enable);
        if(enable == true)
          return;
      }
  
      if(g_unSystemIntegrity_MonitoringThread != 0)
      {
        err = stopThread(g_unSystemIntegrity_MonitoringThread);
//DebugN("***********STOPPING THREAD*********");
      }
        
      if(err<0)
        DebugN("unSystemIntegrity_manageRDBMonitoring() - stoping thread failed!");
      else 
        g_unSystemIntegrity_MonitoringThread =0;
      
      break;
      
    default:
      break;
  }
}

//----------------------------------------------------------------------------------------------------------
// roundtrip to the database 

unSystemIntegrity_RDB_checkDbConnection()
{
  time currentTime;
  int valueToDb = 0;
  dyn_int valuesFromDb;
  dyn_time ta;
  
  int iManagerState;
  int flushInterval;
  string managerName = "PVSS00rdb";
  string managerOption = ""; //"-num 99";
  int manIndex;
  string startMode = "";
  dyn_string exInfo;
  int iIndex;
  int restartCounter = 0;
  int roundtripFailureCounter = 0;
  int bufferToDiskIndicator;
  int connected;
  string managerAction;
  int currentBuffer, maxBuffer;
  
 
  while(1)
  {
    unSystemIntegrity_getState(managerName, managerOption, iManagerState);
    unSystemIntegrity_RDB_findManager(managerName, managerOption, startMode, manIndex);

    managerAction = "";
    
    if(iManagerState == SYSTEMINTEGRITY_MANAGER_STOPPED && startMode == "always")
       managerAction = "start";
    else if(iManagerState == SYSTEMINTEGRITY_MANAGER_BLOCKED)
      managerAction = "kill";
    else if(iManagerState == SYSTEMINTEGRITY_MANAGER_RUNNING )
    {
      dpGet(RDB_CONNECTION_CHECK_DP + RDB_CONNECTION_CHECK_VALUE, valueToDb
            ,RDB_DP_NAME + RDB_BUFFER_FLUSH_INTERVAL, g_unSystemIntegrity_writeReadInterval
//          ,RDB_CONNECTION_CHECK_DP + RDB_CONNECTION_CHECK_DB_CHECK_INTERVAL, g_unSystemIntegrity_dbCheckInterval
            );

      if(valueToDb >= 10000)
        valueToDb=0;
      
if(roundtripFailureCounter == 0)
{
      valueToDb++;      
      currentTime = getCurrentTime();

      dpSet(RDB_CONNECTION_CHECK_DP + RDB_CONNECTION_CHECK_VALUE, valueToDb);      
      delay(5 * g_unSystemIntegrity_writeReadInterval / 1000); //g_unSystemIntegrity_writeReadInterval is in [ms]
}      
      dpGetPeriod(currentTime, getCurrentTime(), 0, RDB_CONNECTION_CHECK_DP + RDB_CONNECTION_CHECK_VALUE, valuesFromDb, ta);

      if(!dynContains(valuesFromDb, valueToDb)) //roundtrip failed
      {
        roundtripFailureCounter++;
        unSystemIntegrity_throwError("unSystemIntegrity_RDB_checkDbConnection() -> Roundtrip to the database failed: " + roundtripFailureCounter + "/" + g_unSystemIntegrity_maxRoundtripFailures);       
        dpGet(RDB_DP_NAME + RDB_CURRENT_BUFFER_BLOCKS, currentBuffer
              ,RDB_DP_NAME + RDB_MAX_BUFFER_BLOCKS, maxBuffer
              );
        
        if(g_unSystemIntegrity_BufferToDisk == 0)
          managerAction = "kill";
        else
        {
          if(g_unSystemIntegrity_dbConnectionStatus == 0)
          {
            if(g_unSystemIntegrity_BufferToDiskIndicator == 2)
              managerAction = "kill";
            else if(g_unSystemIntegrity_BufferToDiskIndicator == 0)
              if(currentBuffer >= maxBuffer)
                managerAction = "kill";     
          }
        }
        
        if(roundtripFailureCounter == g_unSystemIntegrity_maxRoundtripFailures || g_unSystemIntegrity_maxRoundtripFailures <= 0)
        {
          roundtripFailureCounter = 0;
          managerAction = "kill";
          unSystemIntegrity_throwError("unSystemIntegrity_RDB_checkDbConnection() -> Maximum number of allowed round-trips to the database exceeded (" + g_unSystemIntegrity_maxRoundtripFailures + "). Restarting the RDB Manager.");
        }
      }
      else 
      {
        restartCounter =0 ;    //roundtrip to DB went ok. Resetting restart counter.
        roundtripFailureCounter = 0;
      }
    }
    
    string timeStamp = getCurrentTime();
    
    if(restartCounter <= g_unSystemIntegrity_maxManagerRestarts || g_unSystemIntegrity_maxManagerRestarts < 0)    
    {       
      switch(managerAction)
      {
        case "kill":
          unSystemIntegrity_throwError("unSystemIntegrity_RDB_checkDbConnection() -> Killing RDB Manager. " + timeStamp);
          unSystemIntergity_manageManager(managerName, managerOption, SYSTEMINTEGRITY_KILL_COMMAND, manIndex, exInfo);
          restartCounter++;          
          break;
        case "start":
          unSystemIntegrity_throwError("unSystemIntegrity_RDB_checkDbConnection() -> Starting RDB Manager. " + timeStamp);
          unSystemIntergity_manageManager(managerName, managerOption, SYSTEMINTEGRITY_START_COMMAND, manIndex, exInfo);
          restartCounter++;
          break;
        default:
          break;
      }  
      
    }
    else
      unSystemIntegrity_throwError("unSystemIntegrity_RDB_checkDbConnection() -> Number of automatic recovery action (" + g_unSystemIntegrity_maxManagerRestarts + ") has been exceeded without success.  Aborting current action: " + managerAction + " RDB manager. "+ timeStamp);
    
    delay(g_unSystemIntegrity_dbCheckInterval); //  delay(15);
  }
}


void unSystemIntegrity_throwError(string msg, string severity = "ERROR", int code = 1)
{
  int prio = PRIO_WARNING; //seems that throwing a fatal error makes Pmon stop monitoring this manager????
  int type = ERR_CONTROL;
  
  switch(strtoupper(severity))
  {
    case "INFO": prio = PRIO_INFO; break;
    case "WARNING": prio = PRIO_WARNING; break;
    case "ERROR": prio = PRIO_SEVERE; break;
  }
  
  errClass err = makeError("fwInstallation", prio, type, code, msg);
  throwError(err);
  
  return;
}

int unSystemIntegrity_RDB_findManager(	string manager, string commandLine, string & startMode, int &managerPos)
{
  dyn_mixed managerInfo;
  fwInstallationManager_getProperties(manager, commandLine, managerInfo);
  startMode = managerInfo[FW_INSTALLATION_MANAGER_START_MODE];
  managerPos = managerInfo[FW_INSTALLATION_MANAGER_PMON_IDX];
  
  return 0;
}
